oUF_Lunar_Version = 465
--[[****************************************************************************************************************************************************]]--
--                                                    DECLARE LOCAL VARIABLES
--[[****************************************************************************************************************************************************]]--
---------------------------------------------------- Declare locals for later use
oUF_Lunar = {}
-- Shorthand
local math_floor = math.floor
local math_ceil = math.ceil
local math_abs = math.abs
local unpack = unpack
local strformat = string.format
-- Functions
local ShowHideFrames
-- Frame Records
local OpenUnitFrames = {}
Lunar_OpenUnitFrames = OpenUnitFrames -- Global
local OpenUnitHeaders = {}
local PartyPets = {}
local TargetOf = {}
local DragableFrames = {}
-- Lists
local PartyFilterExceptions = {}
local Custom_Party_NameList
local PriorityRaidList = {}
-- Preferances
local Profile
-- Flags
local Player_Has_Loaded
local Spell_ID_Database_Exists
local RaidMode
local PartyIsFiltered
local Custom_Party_Update_Needed
-- Quick referance vars
local MinimumRaid_Last
local RaidGroupCount = 8
local KA_update;
local OptionsMenu
local UpdateMinimapButton
-- Manager Frames
local PartyMonitor;
-- Static
local OffStatus = {0.7,0.7,0.7,1}
local Trimmed_TexCoord = {0.06, 0.94, 0.06, 0.94}
local Full_TexCoord = {0,1,0,1}
-- ButtonFacade vars
local AuraButtonNameIndex = 1
local LBF_Groups = {}
local AuraIcons = {}
local MOB_key
-- Right Click Menu
local DD_Functions = {}
local DD_MenuTypes = {
    "RAID",
    "PARTY",
    "FRIEND",
    "TEAM",
    "TARGET"
}
local DropDownBackground = {
    bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
    edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
    tile = true, tileSize = 16, edgeSize = 16, 
    insets = { left = 4, right = 4, top = 4, bottom = 4 }
}

-- Textures, fonts
local MediaPath = 'Interface\\Addons\\oUF_Lunar\\media\\'
local Textures_Hash = {
    ['Smooth v2'] = MediaPath..'Smoothv2.tga',
    ['Comet'] = MediaPath..'Comet.tga',
    ['Comet-Reversed'] = MediaPath..'Comet-Reversed.tga',
    ['Melli'] = MediaPath..'Melli.tga',
    ['Outline'] = MediaPath..'Outline.tga',
}
local Font_Hash = {
    ['Friz Quadrata TT'] = "Fonts\\FRIZQT__.TTF",
    ['Carnevalee Freakshow'] = MediaPath..'Fonts\\Carnevalee Freakshow.ttf',
    ['Sofachrome I'] = MediaPath..'Fonts\\sofachri.ttf',
    ['Sofachrome'] = MediaPath..'Fonts\\sofachro.ttf',
    ['Husky Stash'] = MediaPath..'Fonts\\HUSKYSTA.ttf',
    ['SF Toontime'] = MediaPath..'Fonts\\SF Toontime.ttf',
    ['Calibri1'] = MediaPath..'Fonts\\Calibri1.ttf',
    ['CalibriBold'] = MediaPath..'Fonts\\CalibriBold.ttf',
}

local oUF_Lunar_FrameTypes = {}
-- Display Names
local DisplayNames = {
    oUF_Lunar_Player = "Player", 
    oUF_Lunar_PlayerPet = "Pet", 
    oUF_Lunar_PlayerPetTarget = "PetTarget", 
    oUF_Lunar_Party = "Party",
    oUF_Lunar_PartyPet = "PartyPet",
    oUF_Lunar_PartyTarget = "PartyTargets",
    oUF_Lunar_Focus = "Focus", 
    oUF_Lunar_FocusTarget = "FocusTarget", 
    oUF_Lunar_Target = "Target", 
    oUF_Lunar_TargetTarget = "TargetTarget",
    Global = 'Global'
}
for i = 1, 8 do
    DisplayNames['oUF_Lunar_Arena'..i] = 'Arena'..i
    DisplayNames['oUF_Lunar_Arena'..i..'Pet'] = 'ArenaPet'..i
    DisplayNames['oUF_Lunar_Arena'..i..'Target'] = 'ArenaTarget'..i
    DisplayNames['oUF_Lunar_Raid'..i] = 'Raid'..i
end
--
local SingleUnitFrames = {
    'Target',
    'Player',
    'PlayerPet',
    'Focus',
}
local conv = {
	['playerpet'] = 'pet',
	['playertarget'] = 'target',
}
---------------------------------------------------- Class Icon coords  
local ClassTexture = "Interface\\WorldStateFrame\\Icons-Classes"

local classIcons = {
  ["WARRIOR"] = { 0.00, 0.25, 0.00, 0.25 },
  ["MAGE"] = { 0.25, 0.50, 0.00, 0.25 },
  ["ROGUE"] = { 0.50, 0.75, 0.00, 0.25 },
  ["DRUID"] = { 0.75, 1.00, 0.00, 0.25 },
  ["HUNTER"] = { 0.00, 0.25, 0.25, 0.50 },
  ["SHAMAN"] = { 0.25, 0.50, 0.25, 0.50 },
  ["PRIEST"] = { 0.50, 0.75, 0.25, 0.50 },
  ["WARLOCK"] = { 0.75, 1.00, 0.25, 0.50 },
  ["PALADIN"] = { 0.00, 0.25, 0.50, 0.75 },
  ["DEATHKNIGHT"] = { 0.25, 0.50, 0.50, 0.75 },
}

local ClassIndex = {
    DEATHKNIGHT = 1,
    DRUID = 2,
    HUNTER = 3,
    MAGE = 4,
    PALADIN = 5,
    PRIEST = 6,
    ROGUE = 7,
    SHAMAN = 8,
    WARLOCK = 9,    
    WARRIOR = 10
}
---------------------------------------------------- Type icon paths
local typeIcons = {
    ["Beast"] = 'Interface\\Icons\\Spell_Holy_BlessingOfAgility',
    ["Humanoid"] = 'Interface\\Icons\\Spell_Holy_PrayerOfHealing',
    ["Demon"] = 'Interface\\Icons\\Ability_Warlock_DemonicEmpowerment',
    ["Undead"] = 'Interface\\Icons\\INV_Misc_Bone_HumanSkull_02',
    ["Giant"] = 'Interface\\Icons\\Spell_DeathKnight_IceBoundFortitude',
    ["Elemental"] = 'Interface\\Icons\\Spell_Fire_Elemental_Totem',
    ["Dragonkin"] = 'Interface\\Icons\\INV_Misc_Head_Dragon_01',
    ["Critter"] = 'Interface\\Icons\\Spell_Nature_Polymorph_Cow',
    ["Mechanical"] = 'Interface\\Icons\\Trade_Engineering',
    ["Other"] = "Interface\\Icons\\INV_Misc_QuestionMark"
}

---------------------------------------------------- Classification Formats
local classification = { 
    worldboss = '|cffff0000 Boss|r', 
    rareelite = '|cff9988ff+ Rare|r', 
    elite = '|cffffaa22+|r', 
    rare = '|cff9988ff Rare|r', 
    normal = '', 
    trivial = 't', 
} 
--[[****************************************************************************************************************************************************]]--
--                                                    COMMON FUNCTIONS
--[[****************************************************************************************************************************************************]]--

local function debug (message)
    DEFAULT_CHAT_FRAME:AddMessage(message)
end

local function message(msg)
    DEFAULT_CHAT_FRAME:AddMessage(msg)
end
---------------------------------------------------- String Color
local RGB_White = {r=1,g=1,b=1}
local dummy = function() end
local dummyTrue = function() return true end
local function Str_Color(str, r,g,b)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
    local HexColor = strformat("%02x%02x%02x", r*255, g*255, b*255)

    return('|cff'..HexColor..' '..str..'|r')
end
---------------------------------------------------- Table To String
local function TableToString(t) 
    local s = ""
    for value in pairs(t) do
        s = s..value..(t[value] or "")
    end
    return s;
end
---------------------------------------------------- Shorten Value
local function letter(value)
    if value >= 10000000 then 
        return strformat('%.1fm', value / 1000000) 
    elseif value >= 1000000 then 
        return strformat('%.2fm', value / 1000000) 
    elseif value >= 100000 then 
        return strformat('%.0fk', value / 1000) 
    elseif value >= 10000 then 
        return strformat('%.1fk', value / 1000) 
    else 
        return value 
    end 
end 

local function RefreshFrames(element, frame, unit)
    if not Player_Has_Loaded then return end
    if not frame then
        for _, self in ipairs(OpenUnitFrames) do
            if self.unit and (not unit or self.unit==unit)then
                -- Refresh element or all elements
                if element then
                    self:UpdateElement(element)
                else
                    self:PLAYER_ENTERING_WORLD()
                end
            end
        end
    elseif element and frame.unit then
        frame:UpdateElement(element)
    elseif frame.unit then
        frame:PLAYER_ENTERING_WORLD()
    end
end

---------------------------------------------------- Register right click popup menu
local function RegisterRightClick(New_Entry, F, AddToMenu)
    UnitPopupButtons[New_Entry] = { text = New_Entry, dist = 0 };
    DD_Functions[New_Entry] = F
    
    if not AddToMenu then return end
    
    local LastItem, New_LastItem, Menu
    for i, MenuType in ipairs(DD_MenuTypes) do
        Menu = UnitPopupMenus[MenuType]
        Old_LastEntry = getn(Menu);
        New_LastEntry = Old_LastEntry + 1;
        
        -- Check if CANCEL is last option, if so, keep CANCEL as last option.
        if ( Menu[Old_LastEntry] == "CANCEL" ) then
            Menu[Old_LastEntry] = New_Entry;
            Menu[New_LastEntry] = "CANCEL";
        -- Otherwise, make new option the last option.
        else
            Menu[New_LastEntry] = New_Entry;
        end
    end
end
-- GetShowHideState
local function GetShowHideState(fType, CheckTargetOf)
    local CustomFrameData = Profile.CustomFrames[fType]
    if not CustomFrameData then return end
    local state
        
    if GetNumRaidMembers() > Profile.MinimumRaid then
        state = CustomFrameData.ShowWhileRaid
    elseif fType~='oUF_Lunar_Raid' then
        state = CustomFrameData.Enabled
    end
    
    if CheckTargetOf then
        state = state and CustomFrameData.ShowTargetOf and GetShowHideState('oUF_Lunar_X_Target')
    end
    
    return state
end

--[[****************************************************************************************************************************************************]]--
--                                                    FRAME ADJUSTMENT FUNCTIONS
--[[****************************************************************************************************************************************************]]--
---------------------------------------------------- Repeater  
local function Process_OpenUnitFrames(Object, F, frames)
--~     if not Profile then return end
    
    if type(Object) == 'string' then
        for _, self in ipairs(frames) do
            if self.frameType == Object or Object == 'Global' then
                F(self)
            end
        end
    elseif type(Object) == 'table' then
            F(Object)
    end
end

---------------------------------------------------- Adjust enable or disable features
local function SingleAdjustFrameElements(self)
    if not Player_Has_Loaded then return end
    local CustomFrameData = Profile.CustomFrames[self.frameType]
    -- Threat
    if self.IconSpace > 0 and self.ThreatDisabled then
        self.ThreatDisabled = false
        self.Threat = self.Pre_Threat
        self:EnableElement('Threat')
--~         self.IconFrame:Show()
    elseif self.IconSpace == 0 then
        self.ThreatDisabled = true
        self:DisableElement('Threat')
        self.Pre_Threat:Hide()
    end
    -- PvP
    if self.PvP then
        if CustomFrameData.PvpIcon and self.PvP.Disabled then
            self.PvP.Disabled = false
            self:EnableElement('PvP')
        elseif not CustomFrameData.PvpIcon and not self.PvP.Disabled then
            self.PvP.Disabled = true
            self:DisableElement('PvP')
            self.PvP:Hide()
        end
    end
    -- Resting
    if self.Resting then
        if CustomFrameData.RestIcon and self.Resting.Disabled then
            self.Resting.Disabled = false
            self:EnableElement('Resting')
        elseif not CustomFrameData.RestIcon and not self.Resting.Disabled then
            self.Resting.Disabled = true
            self:DisableElement('Resting')
            self.Resting:Hide()
        end
    end     
    -- Portrait
    if CustomFrameData.ModelPortrait and not self.Portrait then
        self.Portrait = self.IconFrame.Portrait
        self:EnableElement('Portrait')
        self.Portrait:Show()
    elseif not CustomFrameData.ModelPortrait and self.Portrait then
        self:DisableElement('Portrait')
        self.Portrait:Hide()
        self.Portrait = nil
    end
end

local function AdjustFrameElements(Object)
    Process_OpenUnitFrames(Object, SingleAdjustFrameElements, OpenUnitFrames)
end
    
---------------------------------------------------- Show / Hide Blizzard Frames
local function ShowHideBlizzardFrames(Object)
    -- Override other addon requests
    if not CastingBarFrame.OldShow then
        CastingBarFrame.OldShow = CastingBarFrame.Show
        CastingBarFrame.OldHide = CastingBarFrame.Hide
        CastingBarFrame.OldUnregisterAllEvents = CastingBarFrame.UnregisterAllEvents
        CastingBarFrame.UnregisterAllEvents = dummy
    end

    local function Set(Object)
        if Object == 'BuffFrame' then
            if Profile.BlizzardFrames[Object] then
                BuffFrame:Show()
                TemporaryEnchantFrame:Show()
                BuffFrame:RegisterEvent("PLAYER_AURAS_CHANGED")
                BuffFrame_Update()
            else
                BuffFrame:Hide()
                TemporaryEnchantFrame:Hide()
                BuffFrame:UnregisterAllEvents()
            end
        end
        if Object == 'Castbars' then
            -- Show or hide
            if Profile.BlizzardFrames[Object] then
                CastingBarFrame.Show = CastingBarFrame.OldShow
                CastingBarFrame.Hide = CastingBarFrame.OldHide
            else
                CastingBarFrame.Show = dummy
                CastingBarFrame:Hide()
            end
        end
    end
    
    if Object == 'Global' then
        for BF in pairs(Profile.BlizzardFrames) do
            Set(BF)
        end
    elseif Object then
        Set(Object)
    end
end
---------------------------------------------------- Fill Icon Position Info
local function Single_Fill_Icon_Position_Info(icons, Width)
    local Position_Info = icons.Position_Info
    local size = icons.Scale or 16
    local slot = (icons.spacing +16)
    
    Position_Info[1] = slot
    Position_Info[2] = size /16 -- Scale
    Position_Info[3] = icons.initialAnchor
    Position_Info[4] = (icons["growth-x"] == "LEFT" and -1) or 1
    Position_Info[5] = (icons["growth-y"] == "DOWN" and -1) or 1
    Position_Info[6] = math_floor(Width / slot) -- Columns
    Position_Info[7] = Width / slot *0.5 --Aspect Ratio
end
local function Fill_Icon_Position_Info(self)
    Single_Fill_Icon_Position_Info(self.Auras, self.Auras:GetWidth())
    Single_Fill_Icon_Position_Info(self.Low_Buffs, self.CurrentWidth or self:GetWidth())
    Single_Fill_Icon_Position_Info(self.Low_Debuffs, self.CurrentWidth or self:GetWidth())
end
---------------------------------------------------- RangeCheck 
local function SingleAdjustRangeAlpha(self) 
    if not self.unit then return end
    local RangeAlpha = Profile.CustomFrames[self.frameType].RangeAlpha /100
    local RangeType = 'SpellRange'
    if RangeAlpha < 1 then
        self.outsideRangeAlpha = RangeAlpha
        if not self[RangeType] then
            self[RangeType] = true
            self:EnableElement(RangeType)
        end
        RefreshFrames(RangeType, self)
    elseif self[RangeType] then
        self[RangeType] = false
        self:DisableElement(RangeType)
        self:SetAlpha(1)
    end
end
local function AdjustRangeAlpha(Object)    
    Process_OpenUnitFrames(Object, SingleAdjustRangeAlpha, OpenUnitFrames)      
end

---------------------------------------------------- Tooltip, Highlight, Mouseover Behavior
local function OnEnter_With_Buffs(self) 
    UnitFrame_OnEnter(self) 
    self.Highlight:Show()
    if MOB_key() then
        self.BuffContainer.Background:Show()
        self.BuffContainer:EnableMouse(true)
        self.BuffContainer:SetParent(UIParent)
        self.BuffContainer:SetFrameLevel(20)
        self.BuffContainer:SetScale(self.CurrentScale)
        self.Auras.MouseOverMode = false
        RefreshFrames("Aura", self)
    end
end 

local function OnLeave_With_Buffs(self, Override) 
    UnitFrame_OnLeave(self) 
    self.Highlight:Hide() 
    if MOB_key() or Override=='Override' then
        self.BuffContainer.Background:Hide()
        self.BuffContainer:EnableMouse(false)
        self.BuffContainer:SetParent(self)
        self.BuffContainer:SetScale(1)
        self.Auras.MouseOverMode = true
        RefreshFrames("Aura", self)
    end
end

local function OnEnter(self) 
    UnitFrame_OnEnter(self) 
    self.Highlight:Show()
end 
	 
local function OnLeave(self) 
    UnitFrame_OnLeave(self) 
    self.Highlight:Hide() 
end
---------------------------------------------------- Adjust Font
local function UpdateFontString(fontstr, Newfont)
    if not fontstr.base_size then
        fontstr.base_size = math_ceil(select(2,fontstr:GetFont()) or 1)
    end
    local Size = fontstr.base_size + (Profile.GlobalFontSizeModify or 0)
    Size = Size > 1 and Size or 1
    fontstr:SetFont(Newfont, Size ,select(3,fontstr:GetFont()))
end
local function SingleAdjustFont(self)
    local Newfont = Font_Hash[Profile.GlobalFont or 'Friz Quadrata TT'] or Font_Hash['Friz Quadrata TT']
    if not Newfont then return end
    UpdateFontString(self.Health.total, Newfont)
    UpdateFontString(self.Health.value, Newfont)
    UpdateFontString(self.Pre_Power.total, Newfont)
    UpdateFontString(self.Pre_Power.value, Newfont)
    UpdateFontString(self.Name, Newfont)
    UpdateFontString(self.Pre_Castbar.Text, Newfont)
    UpdateFontString(self.CombatFeedbackText, Newfont)
end
local function AdjustFont(Object)
    Process_OpenUnitFrames(Object, SingleAdjustFont, OpenUnitFrames)
end
---------------------------------------------------- Adjust Spacers
local function AdjustSpacer_Header(Header, Spacer)
    if not Header then return end
    Header:SetAttribute('yOffset', -Spacer)
    if Spacer <0 then
        Header:SetAttribute('point', 'BOTTOM')
    else
        Header:SetAttribute('point', 'TOP')
    end
end
local function AdjustSpacers(fType, headers)
    local CustomFrameData = Profile.CustomFrames[fType]
    local Spacer = CustomFrameData.DefaultSpacerY
    local FrameHeight = CustomFrameData.FrameHeight
    local TotalHeight = math_abs(Spacer) + FrameHeight
    local Header
    local DragFrame
    if fType == 'oUF_Lunar_Party' or fType == 'oUF_Lunar_PartyPet' then
        DragFrame = DragableFrames[fType]
        if OpenUnitHeaders[fType] then
            DragFrame.CurrentHeight = TotalHeight *4
            DragFrame:SetHeight(TotalHeight *4)
            if not headers then
                AdjustSpacer_Header(OpenUnitHeaders[fType], Spacer)
            end
        end
    elseif fType == 'oUF_Lunar_Raid' then
        for i = 1, RaidGroupCount do
            DragFrame = DragableFrames[fType..i]
            if DragFrame and OpenUnitHeaders[fType..i] then
                DragFrame.CurrentHeight = TotalHeight *5
                DragFrame:SetHeight(TotalHeight *5)
                if not headers then
                    AdjustSpacer_Header(OpenUnitHeaders[fType..i], Spacer)
                end
            end
        end
    elseif fType == 'oUF_Lunar_X_Target' then
        for frame in pairs (TargetOf) do
            DragFrame = DragableFrames['oUF_Lunar_'..frame..'Target']
            if not TargetOf[frame].unit then
                if DragFrame.CurrentHeight ~=TotalHeight *4 and TargetOf[frame] then
                    DragFrame.CurrentHeight =TotalHeight *4
                    DragFrame:SetHeight(TotalHeight *4)
                    AdjustSpacer_Header(TargetOf[frame], Spacer)
                end
            elseif DragFrame.CurrentHeight ~= TotalHeight and DragableFrames['oUF_Lunar_'..frame..'Target'] then
                DragFrame:SetHeight(TotalHeight)
            end
        end
    elseif fType == 'oUF_Lunar_Arena' or fType == 'oUF_Lunar_ArenaPet' then
        for i=1,5 do
            DragFrame = DragableFrames[fType..i] or DragableFrames['oUF_Lunar_Arena'..i..'Pet']
            if not DragFrame then
                return
            elseif DragFrame.CurrentHeight ~= TotalHeight then
                DragFrame.CurrentHeight = TotalHeight 
                DragFrame:SetHeight(TotalHeight)
            end
        end
    elseif DragableFrames[fType] and DragableFrames[fType].CurrentHeight ~= TotalHeight then
        DragableFrames[fType].CurrentHeight = TotalHeight
        DragableFrames[fType]:SetHeight(TotalHeight)
    end
end

---------------------------------------------------- Adjust Textures
local function SingleAdjustTextures(self)
    local CustomFrameData = Profile.CustomFrames[self.frameType]    
    local HealthLoc = Textures_Hash[CustomFrameData.HealthTexture]
    local PowerLoc = Textures_Hash[CustomFrameData.PowerTexture]
    local CastLoc = Textures_Hash[CustomFrameData.CastTexture]

    if HealthLoc then
        self.Health:SetStatusBarTexture(HealthLoc)
        self.Lunar_HealCommBar:SetStatusBarTexture(HealthLoc)
    end
    if PowerLoc then
        self.Pre_Power:SetStatusBarTexture(PowerLoc)
    end                
    if CastLoc then
        self.Pre_Castbar:SetStatusBarTexture(CastLoc)
    end
end

local function AdjustTextures(Object)
    Process_OpenUnitFrames(Object, SingleAdjustTextures, OpenUnitFrames)
end

---------------------------------------------------- Adjust Colors

local function AdjustBarColor(self, CustomFrameData, BarType)
    local bar = self['Pre_'..BarType]
    if CustomFrameData[BarType.."ClassColor"] == 0 then
        local k
        local SG = bar.smoothGradient
        for i=1, 5 do
            k = (i-1)*4
            SG[k+1],SG[k+2],SG[k+3],SG[k+4] = unpack(CustomFrameData[BarType..'Color'..i])
        end

        bar.ClassColor = nil
    elseif self.unit then
        local class = select(2, UnitClass(self.unit))
        if not class then return end
        local Index = ClassIndex[class] + 5
        bar.ClassColor = CustomFrameData[BarType.."Color"..Index]
        bar:SetStatusBarColor(unpack(bar.ClassColor))
    end
    bar:SetStatusBar_BackColor(unpack(CustomFrameData[BarType.."Color99"]))
end
local function SingleAdjustColors(self, SkipRefresh)
    local CustomFrameData = Profile.CustomFrames[self.frameType]
    AdjustBarColor(self, CustomFrameData, "Health")
    AdjustBarColor(self, CustomFrameData, "Power")
    AdjustBarColor(self, CustomFrameData, "Cast")
    
    -- Refresh as needed
    if not SkipRefresh then
        RefreshFrames('Health', self)
        if self.Power then
            RefreshFrames('Power', self)
        end
    end
end
local function AdjustColors(Object)
    Process_OpenUnitFrames(Object, SingleAdjustColors, OpenUnitFrames)
end

---------------------------------------------------- Adjust Scale
local function SingleAdjustScale(self)
    local frameType = self.frameType
    local CustomFrameData = Profile.CustomFrames[frameType]
    local FrameScale = CustomFrameData.FrameScale/100
    self.CurrentScale = FrameScale
    local HeaderName = self.HeaderName or self:GetParent().HeaderName
    if HeaderName then
        DragableFrames[HeaderName]:SetScale(FrameScale)
    end
end

local function AdjustScale(Object)    
    Process_OpenUnitFrames(Object, SingleAdjustScale, OpenUnitFrames)
end
---------------------------------------------------- Adjust Frame Width
local function SingleAdjustFrameWidth(self)
    local frameType = self.frameType
    local FrameWidth = Profile.CustomFrames[frameType].FrameWidth
    local IconSpace = self.IconSpace
    self:SetWidth(FrameWidth)
    self.CurrentWidth = FrameWidth
    self.Pre_Power:SetWidth(FrameWidth -IconSpace)
    self.Health:SetWidth(FrameWidth -IconSpace)
    self.Pre_Castbar:SetWidth(FrameWidth -IconSpace)
    self.Lunar_HealCommBar:SetWidth(FrameWidth -IconSpace)
    self.Lunar_HealCommBar:SetValue(0)
    
    Fill_Icon_Position_Info(self)
    local HeaderName = self.HeaderName or self:GetParent().HeaderName
    if HeaderName and DragableFrames[HeaderName].Width ~= FrameWidth then
        DragableFrames[HeaderName].Width = FrameWidth
        DragableFrames[HeaderName]:SetWidth(FrameWidth)
    end
    if self.TotemBar then
        for i=1,4 do
            self.TotemBar[i]:SetWidth((FrameWidth -IconSpace)/4 -1)
        end
    end
    self:SetAttribute('initial-width', FrameWidth)
end

local function AdjustFrameWidth(Object)    
    Process_OpenUnitFrames(Object, SingleAdjustFrameWidth, OpenUnitFrames)
end
---------------------------------------------------- Adjust Aura Size
local function SingleAdjustAuraSize(self)
    local frameType = self.frameType
    local CustomFrameData = Profile.CustomFrames[frameType]
    local offsetX = CustomFrameData.AuraOffset
    local offsetY
    local orientation
    local opposite_orient
    local max
    max = self.Auras.numBuffs + self.Auras.numDebuffs
    size = CustomFrameData.AuraSize

    if CustomFrameData.AuraLoc == -1 then 
        orientation = 'LEFT'
        opposite_orient = 'RIGHT'
        offsetX = offsetX -(size + 2)
        offsetY = 0
    else
        orientation = 'RIGHT'
        opposite_orient = 'LEFT'
        offsetX = offsetX +(size + 2)
        offsetY = 0
    end

    self.Auras:ClearAllPoints()
    self.Auras:SetPoint('TOP'..orientation, self, 'TOP'..orientation, offsetX, offsetY) 
    self.Auras:SetHeight(size) 
    self.Auras:SetWidth((size+2)*10) 
--~     self.Auras.size = size
    self.Auras.spacing = CustomFrameData.AuraSpacing
    self.Auras.Scale = size
    self.Auras['growth-x'] = orientation
    self.Auras.initialAnchor = 'TOP'..orientation
    
    self.Low_Buffs.initialAnchor = 'TOP'..opposite_orient
    self.Low_Debuffs.initialAnchor = 'BOTTOM'..orientation
    self.Low_Buffs['growth-x'] = orientation
    self.Low_Debuffs['growth-x'] = opposite_orient
    self.Low_Buffs.spacing = CustomFrameData.AuraSpacing
    self.Low_Debuffs.spacing = CustomFrameData.AuraSpacing
    
    Fill_Icon_Position_Info(self)
    RefreshFrames("Aura", self)
end
local function AdjustAuraSize(Object)
    Process_OpenUnitFrames(Object, SingleAdjustAuraSize, OpenUnitFrames)
end
---------------------------------------------------- Adjust Text Visibility
local function SingleAdjustTextVisibility(self)
    local CustomFrameData = Profile.CustomFrames[self.frameType]
    local FrameHeight = CustomFrameData.FrameHeight
    local PowerHeight = CustomFrameData.PowerHeight
    local HealthBarHeight = FrameHeight -PowerHeight -CustomFrameData.CastBarHeight

    -- TotalText
    if CustomFrameData.TotalText then
        if PowerHeight > 7 then
            self.Pre_Power.total:Show()
        else
            self.Pre_Power.total:Hide()
        end
        if HealthBarHeight > 9 then
            self.Health.total:Show()
        else
            self.Health.total:Hide()
        end
    else
        self.Pre_Power.total:Hide()
        self.Health.total:Hide()
    end
    
    -- ValueText
    if CustomFrameData.ValueText then
        self.Health.value:SetWidth(0)
        self.Pre_Power.value:SetWidth(0)
        if PowerHeight > 7 then
            self.Pre_Power.value:Show()
        else
            self.Pre_Power.value:Hide()
        end
        if HealthBarHeight > 9 then
            self.Health.value:Show()
        else
            self.Health.value:Hide()
        end
    else
        self.Pre_Power.value:Hide()
        self.Pre_Power.value:SetWidth(1)
        self.Health.value:Hide()
        self.Health.value:SetWidth(1)
    end
    self.Health.HideTextWhenFull =  CustomFrameData.HideTextWhenFull
    self.Pre_Power.HideTextWhenFull =  CustomFrameData.HideTextWhenFull
        
    -- Names
    self.NameColor_Class = CustomFrameData.NameColor_Class
    self.NameColor_Level = CustomFrameData.NameColor_Level
    
    if not (self.NonGroupFrame or self.NameColor_Class or self.NameColor_Level) then
        self.Name:Hide()
    else
        self.Name:Show()
    end
    
    -- Deficit
    self.Health.Deficit =  CustomFrameData.Deficit
    self.Pre_Power.Deficit =  CustomFrameData.Deficit

    -- Refresh
    RefreshFrames("Health", self)
    if self.Power then
        RefreshFrames("Power", self)
    end
    self:UNIT_NAME_UPDATE('UNIT_MAXHEALTH', self.unit)
end

local function AdjustTextVisibility(Object)
    Process_OpenUnitFrames(Object, SingleAdjustTextVisibility, OpenUnitFrames)
end
---------------------------------------------------- Adjust Bar Height
local function SingleAdjustBarHeight(self)
    -- For shorthand reference
    local frameType = self.frameType
    local CustomFrameData = Profile.CustomFrames[frameType]
    local Power = self.Pre_Power
    local Castbar = self.Pre_Castbar
    local PowerHeight = CustomFrameData.PowerHeight
    local CastBarHeight = CustomFrameData.CastBarHeight
    
    -- Heights 
    Power:SetHeight(PowerHeight > 0 and PowerHeight or 0.01)
    Castbar:SetHeight(CastBarHeight > 0 and CastBarHeight or 0.01)
    Castbar.Spark:SetHeight(CastBarHeight*3)
    
    if PowerHeight > 0 and self.PowerBarDisabled and Player_Has_Loaded then
        self.Pre_Power:Show()
        self.Power = self.Pre_Power
        self.PowerBarDisabled = false
        self:EnableElement('Power')
        RefreshFrames('Power', self)
    elseif PowerHeight == 0 then
        self.PowerBarDisabled = true
        self:DisableElement('Power')
    end

    if CastBarHeight > 0 and self.CastbarDisabled and Player_Has_Loaded then
        self.Castbar = self.Pre_Castbar
        self.Castbar:Show()
        self.CastbarDisabled = false
        self:EnableElement('Castbar')
        -- Set blizzard castbar option
        ShowHideBlizzardFrames('Castbars')
        RefreshFrames('Castbar', self)
    elseif CastBarHeight == 0 then
        self.CastbarDisabled = true
        self:DisableElement('Castbar')
        self.Pre_Castbar:Hide()
        -- Set blizzard castbar option
        ShowHideBlizzardFrames('Castbars')
    end 

    SingleAdjustTextVisibility(self)
end

local function AdjustBarHeight(Object)
    Process_OpenUnitFrames(Object, SingleAdjustBarHeight, OpenUnitFrames)
end

---------------------------------------------------- Adjust For IconSpace
local function AdjustForIconSpace(self)
    local frameType = self.frameType
    local CustomFrameData = Profile.CustomFrames[frameType]
    local FrameHeight = CustomFrameData.FrameHeight    
    local IconSpace = (CustomFrameData.ClassPortrait or CustomFrameData.ModelPortrait or CustomFrameData.FlatPortrait) and (FrameHeight+1) or 0
    local BarWidth = CustomFrameData.FrameWidth -IconSpace
    local Pos_Neg = CustomFrameData.PortraitLoc
    local Point1 = (Pos_Neg == -1) and 'RIGHT' or 'LEFT'
    local Point2 = (Pos_Neg == 1) and 'RIGHT' or 'LEFT'

    self.Pre_Castbar:SetPoint('BOTTOM'..Point1, self, 'BOTTOM'..Point1)
    self.Pre_Castbar:SetPoint('BOTTOM'..Point2, self, 'BOTTOM'..Point2, -IconSpace*Pos_Neg, 0)
    self.Pre_Power:SetWidth(BarWidth)
    self.Health:SetWidth(BarWidth)
    self.Pre_Castbar:SetWidth(BarWidth)
    -- Icons
    self.IconSpace = IconSpace
    self.IconFrame:SetPoint(Point2, self, Point2)
    self.IconFrame:SetPoint(Point1, self, Point2, -Pos_Neg*FrameHeight,0)
    -- Name
    if not self.NonGroupFrame then
        self.Name:ClearAllPoints()
        self.Name:SetPoint('BOTTOM', self, 'BOTTOM')
        self.Name:SetPoint('TOP'..Point2, self, 'TOP'..Point2, -(IconSpace+2)*Pos_Neg, 2)   
        self.Name:SetPoint(Point1, self.Health.value, Point2, 4*Pos_Neg, 0)
        self.Name:SetJustifyH(Point2)
    end
    self:UNIT_NAME_UPDATE('UNIT_MAXHEALTH', self.unit)
end

---------------------------------------------------- Adjust Misc Frame Settings
local function SingleAdjustFrame(self)
    local Power = self.Pre_Power
    local Health = self.Pre_Health
    local Cast = self.Pre_Castbar
    local frameType = self.frameType
    local CustomFrameData = Profile.CustomFrames[frameType]
    local FrameHeight = CustomFrameData.FrameHeight
        
    -- Set IconSpace
    self.IconSpace = (CustomFrameData.ModelPortrait or CustomFrameData.ClassPortrait or CustomFrameData.FlatPortrait) and (FrameHeight+1) or 0
    
    -- Orientation, Icons, Portraits
    do
        local Pos_Neg = CustomFrameData.PortraitLoc
        local Point1 = (Pos_Neg == -1) and 'RIGHT' or 'LEFT'
        local Point2 = (Pos_Neg == 1) and 'RIGHT' or 'LEFT'
         
        -- Clear All Points
        Power.value:ClearAllPoints()
        Power.total:ClearAllPoints()
        Health.value:ClearAllPoints()
        Health.total:ClearAllPoints()
        
        -- Toggle Class or Type or 2d-Portrait in icon
        self.ClassIcon = CustomFrameData.ClassPortrait
        self.Portrait_2d = CustomFrameData.FlatPortrait
        if not CustomFrameData.ModelPortrait and (CustomFrameData.ClassPortrait or CustomFrameData.FlatPortrait) then
            self.IconFrame.Portrait_2d:Show()
        else
            self.IconFrame.Portrait_2d:Hide()
        end
      
        -- Power
        Power.total:SetPoint('BOTTOM'..Point2, Power, 'BOTTOM'..Point2, -4*Pos_Neg, 0)
        Power.value:SetPoint(Point1, Power, Point1, 3*Pos_Neg, 0)
        -- Health
        Health.total:SetPoint('TOP'..Point2, Health, 'TOP'..Point2, -4*Pos_Neg, -1)
        Health.value:SetPoint(Point1, Health, Point1, 3*Pos_Neg, 0)
        -- Icons
        self.RaidIcon:SetPoint("CENTER", 24*Pos_Neg, 0)       
        -- Pvp
        if self.PvP then
            self.PvP:ClearAllPoints()
            self.PvP:SetPoint('BOTTOM'..Point1, self.IconFrame, 'BOTTOM'..Point2, 0, 0) 
        end
    end
    
    -- Toggle elements
    SingleAdjustFrameElements(self)

    -- SmoothBars
    Health.Smooth = CustomFrameData.SmoothBars and true or false
    Power.Smooth = CustomFrameData.SmoothBars and true or false
    
    -- InvertBars, ReverseBars 
    do
        self.ReverseBars = CustomFrameData.ReverseBars
        Health.ReverseBars = CustomFrameData.ReverseBars
        Power.ReverseBars = CustomFrameData.ReverseBars
        
        self.InvertBars = CustomFrameData.InvertBars
        Health.InvertBars = CustomFrameData.InvertBars
        Power.InvertBars = CustomFrameData.InvertBars

        Health:SetDirection()
        Power:SetDirection()
        
        -- HealComm
        if Profile.HealComm then
            local HealCommPoint = Health.ReverseBars and 'RIGHT' or 'LEFT'
            local ParentPoint = (Health.InvertBars ~= Health.ReverseBars) and 'LEFT' or 'RIGHT'
            self.Lunar_HealCommBar:ClearAllPoints()
            self.Lunar_HealCommBar:SetPoint('TOP'..HealCommPoint, Health.MainTexture, 'TOP'..ParentPoint)
            self.Lunar_HealCommBar:SetPoint('BOTTOM', Health.MainTexture, 'BOTTOM')
            self.Lunar_HealCommBar.ReverseBars = Health.ReverseBars
            self.Lunar_HealCommBar:SetDirection()
            self.ignoreHealComm = false
        else
            self.ignoreHealComm = true
        end
        self.Lunar_HealCommBar:SetValue(0)
    end
        
    -- ComboPoints
    if self.CPoints then
        if CustomFrameData.ComboPoints then
            self.CPoints.unit = 'player'
        else
            self.CPoints.unit = ''
        end
    end
    
    -- OnlyShowPlayer
    self.Auras.onlyShowPlayer = CustomFrameData.ShowCastable
    
    -- RuneFrame
    if frameType == 'oUF_Lunar_Player' then
        if CustomFrameData.RuneFrame and UnitClass('player')=='Death Knight' then
            RuneFrame:Show()
        else
            RuneFrame:Hide()
        end
    end
    
    -- FrameBackDrop
    if CustomFrameData.FrameBackDrop then
        self.Background:Show()
    else
        self.Background:Hide()
    end
    
    -- Show normal auras on mouseover
    do    
        -- MouseOverBuffsKey
        if Profile.MOB_key then
            MOB_key = _G[gsub(Profile.MOB_key,' ', '')]
        else
            MOB_key = dummyTrue
        end        
        -- Mouseover script
        local BuffContainer = self.BuffContainer
        OnLeave_With_Buffs(self, 'Override')
        if CustomFrameData.MouseOverAuras and Profile.MOB_key then
            self:SetScript('OnEnter', OnEnter_With_Buffs)
            self:SetScript('OnLeave', OnLeave_With_Buffs)
            BuffContainer:SetScript('OnClick', function()
                OnLeave_With_Buffs(self, 'Override')
            end)
        else
            self.Auras.MouseOverMode = false
            self:SetScript('OnEnter', OnEnter)
            self:SetScript('OnLeave', OnLeave)
            BuffContainer:SetScript('OnClick', nil)
        end
    end
    
    -- Auras
    AdjustForIconSpace(self)
    SingleAdjustAuraSize(self)

    --End
    self:UNIT_NAME_UPDATE('UNIT_MAXHEALTH', self.unit)
    RefreshFrames(nil, self)
end
local function AdjustFrame(Object)    
    Process_OpenUnitFrames(Object, SingleAdjustFrame, OpenUnitFrames)
end

---------------------------------------------------- Adjust Frame Height
local function SingleAdjustFrameHeight(self)
    local frameType = self.frameType
    local FrameHeight = Profile.CustomFrames[frameType].FrameHeight
    self:SetHeight(FrameHeight)
    self.CurrentHeight = FrameHeight
    self:SetAttribute('initial-Height', FrameHeight)
    AdjustForIconSpace(self)
    AdjustSpacers(self.frameType, true)
end

local function AdjustFrameHeight(Object)    
    Process_OpenUnitFrames(Object, SingleAdjustFrameHeight, OpenUnitFrames)
end
---------------------------------------------------- Show / Hide Frames
local function HeaderEnable(Header, State, ForceUpdate)
    if not Header then return end
    if ForceUpdate or (State and Header.Disabled) or (not State and not Header.Disabled) then
        Header:SetAttribute('showParty', State and not Header.RaidMode)  
        Header:SetAttribute('showRaid', State and Header.RaidMode)
        Header.Disabled = not State
    end
end
ShowHideFrames = function(fType, ForceUpdate)
--~     if State == 'ignore' then return end
    -- Party Targets
    if (fType=='oUF_Lunar_X_Target' or fType=='oUF_Lunar_Party'or not fType) then
        HeaderEnable(TargetOf['Party'], GetShowHideState('oUF_Lunar_Party', true), ForceUpdate)
    end
    -- Party
    if (fType == 'oUF_Lunar_Party' or not fType) then
        HeaderEnable(OpenUnitHeaders['oUF_Lunar_Party'], GetShowHideState('oUF_Lunar_Party'), ForceUpdate)      
    end
    -- Party Pets
    if (fType == 'oUF_Lunar_PartyPet' or not fType) and OpenUnitHeaders['oUF_Lunar_PartyPet'] then
        HeaderEnable(OpenUnitHeaders['oUF_Lunar_PartyPet'], GetShowHideState('oUF_Lunar_PartyPet'), ForceUpdate)
    end
    -- Raid
    if fType == 'oUF_Lunar_Raid' or not fType then
        local State = GetShowHideState('oUF_Lunar_Raid')
        for Header in pairs(OpenUnitHeaders) do
            if strmatch(Header, 'oUF_Lunar_Raid') then
                HeaderEnable(OpenUnitHeaders[Header], State, ForceUpdate)
            end
        end
    end
    -- Everything else    
    local State
    for _, self in ipairs(OpenUnitFrames) do
        if (not fType) or (self.frameType == fType) or (self.MasterUnitType == fType) or (fType=='oUF_Lunar_X_Target') then
            if self.MasterUnitType then
                State = GetShowHideState(self.MasterUnitType, true)  
            elseif fType~='oUF_Lunar_X_Target' then
                State = GetShowHideState(self.frameType)
            else
                State = 'ignore'
            end
            if State~='ignore' then
                if State and self.Disabled then
                    self:Enable()
                elseif not State and not self.Disabled then
                    self:Disable()
                end
                self.Disabled = not State
            end
        end
    end
end

--[[****************************************************************************************************************************************************]]--
--                                                    EVENT HOOK FUNCTIONS
--[[****************************************************************************************************************************************************]]--
---------------------------------------------------- Custom Party Filter
local function Party_Filter_Refresh(ForceUpdate)
    if Profile.PartyFilter.Enabled and RaidMode then
        -- Refresh if needed
        if Custom_Party_Update_Needed or ForceUpdate then
            PartyIsFiltered = true
            Custom_Party_Update_Needed = false
            local Header
            for i, FilteredHeader in ipairs(OpenUnitHeaders) do
                Header = OpenUnitHeaders[i]
                Header.RaidMode = true
                Header:SetAttribute('nameList', Custom_Party_NameList)
                ShowHideFrames(Header.frameType, true)
            end
        end
    elseif PartyIsFiltered then
        local Header    
        PartyIsFiltered = false
        Custom_Party_NameList = nil
        for i, FilteredHeader in ipairs(OpenUnitHeaders) do
            Header = OpenUnitHeaders[i]
            Header.RaidMode = false
            Header:SetAttribute('nameList', nil)
            ShowHideFrames(Header.frameType, true)
        end
    end
end
---------------------------------------------------- Update Priority Raid List
local function UpdatePriorityRaidLists(event)
    if not event then return end
    PartyMonitor.UpdateLists = false
	local MyName = UnitName('player')
    local CustomFrameData = Profile.PartyFilter
    
    wipe(PriorityRaidList)
    local PartyFilterExists
    
    -- Add guildmates
    if CustomFrameData.PartyFilterAllowGuild then
        PartyMonitor:RegisterEvent('GUILD_ROSTER_UPDATE')
        for i = 1, GetNumGuildMembers() do
            local gName = GetGuildRosterInfo(i);
            if ( ( gName ) and ( gName ~= "" ) and (gName ~= MyName)) then
                PartyFilterExists = true
                PriorityRaidList[gName] = true;
            end
        end
    else
        PartyMonitor:UnregisterEvent('GUILD_ROSTER_UPDATE')
    end
    
    -- Add friends
    if CustomFrameData.PartyFilterAllowFriends then
        PartyMonitor:RegisterEvent('FRIENDLIST_UPDATE')
        for i = 1, GetNumFriends(), 1 do
            local gName = GetFriendInfo(i);
            if ( ( gName ) and ( gName ~= "" ) and (gName ~= MyName)) then
                PartyFilterExists = true
                PriorityRaidList[gName] = true;
            end
        end
    else
        PartyMonitor:UnregisterEvent('FRIENDLIST_UPDATE')
    end
    
    -- Add manually selected
    for name in pairs(PartyFilterExceptions) do
        if PartyFilterExceptions[name] == 'add' then
            PartyFilterExists = true
            PriorityRaidList[name] = true
        else
            PriorityRaidList[name] = nil
        end
    end
    
    -- Update List
    local Updated_List = ""
    if PartyFilterExists then    
        local Name
        for i=1, GetNumRaidMembers() do
            Name = GetRaidRosterInfo(i)
            if PriorityRaidList[strsplit('-', (Name or ''))] then
                Updated_List = strjoin(',', Name, Updated_List)
            end
        end
    end
    Custom_Party_NameList = Updated_List
    Custom_Party_Update_Needed = true
end
---------------------------------------------------- Party update
local function PartyUpdate(event)
    if event == 'GUILD_ROSTER_UPDATE' or event == 'FRIENDLIST_UPDATE' then
        PartyMonitor.UpdateLists = true
    end
    if(InCombatLockdown()) then
        -- Schedule update on leave combat
        PartyMonitor:RegisterEvent('PLAYER_REGEN_ENABLED')
    else
        --Update, unschedule update on leave combat
        PartyMonitor:UnregisterEvent('PLAYER_REGEN_ENABLED') 

        -- Check if all frames should be shown
        local NumRaidMembers = GetNumRaidMembers()
        if MinimumRaid_Last~=Profile.MinimumRaid or (not RaidMode and (NumRaidMembers > MinimumRaid_Last)) or (RaidMode and NumRaidMembers < MinimumRaid_Last) then
            MinimumRaid_Last = (Profile.MinimumRaid==5 and 5) or 0 -- done to prevent pointer
            RaidMode = NumRaidMembers > MinimumRaid_Last
            UpdatePriorityRaidLists(true)
            ShowHideFrames(nil,true)
            Party_Filter_Refresh()
        elseif PartyMonitor.UpdateLists then
            UpdatePriorityRaidLists(true)
            Party_Filter_Refresh()
        end
    end
end

---------------------------------------------------- Modify_PriorityRaid
local function Modify_PriorityRaid(name, enabled)
    if enabled then
        PartyFilterExceptions[name] = 'add'
    else
        PartyFilterExceptions[name] = 'remove'
    end
    PartyUpdate('GUILD_ROSTER_UPDATE')
end
---------------------------------------------------- Right click player menu
local function RightClickMenu(self)
    -- Check if in party or raid
    local unit = self.unit
    if unit == 'playerpet' then unit='pet' end
    if unit ~= 'player' and unit ~= 'target' then
        for i=1, (GetNumRaidMembers() or GetNumPartyMembers()) do
            if UnitIsUnit(self.unit, "raid"..i) then 
                unit = "raid"..i
            elseif UnitIsUnit(self.unit, "party"..i) then 
                unit = "party"..i
            end
        end
    end
    -- Determine dropdown type
	local cunit = unit:gsub('(.)', strupper, 1) 
    local DropDown
	if strmatch(unit, 'party') then
		DropDown = _G['PartyMemberFrame'..self.id..'DropDown']
	elseif(_G[cunit..'FrameDropDown']) then 
		DropDown = _G[cunit..'FrameDropDown']
    elseif strmatch(unit,'raid') then
        FriendsDropDown.unit = unit
		FriendsDropDown.id = unit:match"^.-(%d+)"
		FriendsDropDown.initialize = RaidFrameDropDown_Initialize
		DropDown = FriendsDropDown
	end
    -- Update priorty list option
    if not DropDown then return end
    local Name = UnitName(unit)
    ToggleDropDownMenu(1, nil, DropDown, 'cursor')
    local Menu = UnitPopupMenus[DropDown.which]
    if Menu then
        local AddText = "Lunar Unit Frames: Add to priority raid"
        local RemoveText = "Lunar Unit Frames: Remove from priority raid"
        local OnList = PriorityRaidList[Name] and true or false
        local SF

        for i, Entry in ipairs(Menu) do
            if (Entry == AddText) or (Entry == RemoveText) then
                if OnList then
                    Menu[i] = RemoveText
                else
                    Menu[i] = AddText
                end
            elseif Entry == 'SET_FOCUS' then
                for MenuLineNumber = i, getn(Menu) do
                    Menu[MenuLineNumber] = Menu[MenuLineNumber+1]
                end
            end
        end
    end
    
    -- Finish
    ToggleDropDownMenu(1, nil, DropDown, 'cursor')
    ToggleDropDownMenu(1, nil, DropDown, 'cursor')
end 
---------------------------------------------------- Unit popup menu addition
local function UnitPopup_OnClick()
    local Entry = this.value
    
    local dropdownFrame = UIDROPDOWNMENU_INIT_MENU;
    if ( type(dropdownFrame) == "string" ) then
		dropdownFrame = getglobal(dropdownFrame);
	end
    local name = tostring( dropdownFrame.name )

    if DD_Functions[Entry] then
        local F = DD_Functions[Entry]
        F(name)
    end
end

---------------------------------------------------- DifficultyColor  
local function getDifficultyColor(level)
	if level == '??' then
		return  1.00, 0.10, 0.10
	else
	local levelDiff = level - UnitLevel('player')
		if levelDiff >= 5 then
			return 1.00, 0.10, 0.10
		elseif levelDiff >= 3 then
			return 1.00, 0.50, 0.25
		elseif levelDiff >= -2 then
			return 1.00, 0.99, 0.28
		elseif -levelDiff <= GetQuestGreenRange() then
			return 0.25, 0.75, 0.25
		else
			return  0.50, 0.50, 0.50
		end
	end
end
---------------------------------------------------- Name, type, level, opacity
local function OnUpdate_NameCheck(self)
    local Name = UnitName(self.Main.unit)
    if Name and Name ~= 'Unknown'  and Name~= '' or not self.Main:IsVisible() or (GetTime() - self.StartTime > 60) then
        self:SetScript('OnUpdate', nil)
        RefreshFrames(nil, self.Main)
    end
end
local function Setup_Name_Monitor(self)
    if not self.Name_Monitor then
        self.Name_Monitor = CreateFrame('frame', nil, self)
        self.Name_Monitor.Main = self
    end
    self.Name_Monitor.StartTime = GetTime()
    self.Name_Monitor:SetScript('OnUpdate', OnUpdate_NameCheck)
end
local function UpdateName(self, event, unit)
    if (not self.unit) then return end
    local Name, Server = UnitName(unit)
    local Level = UnitLevel(unit)
    if not Name or (event~='UNIT_MAXHEALTH' and self.Unit_Name == Name and self.Unit_Level == Level) then 
        return 
    elseif (Name~= 'Unknown' and Name~= '') then
        self.Unit_Name = Name
        self.Unit_Level = Level
    else
        self.Unit_Name = nil
        Setup_Name_Monitor(self)
        return
    end
    -- Set Range
    SingleAdjustRangeAlpha(self) 
    
    -- Remember class
    self.CurrentUnitClass = select(2, UnitClass(unit))
    
    -- Class, type, or 2d portrait icon
    if self.ClassIcon or self.Portrait_2d then
        local Tex
        local Coord
        if self.ClassIcon then
            if UnitIsPlayer(unit) then
                Coord = classIcons[self.CurrentUnitClass]
                Tex = ClassTexture
            else
                Tex = typeIcons[ UnitCreatureType(unit) ]
                Coord = Trimmed_TexCoord
            end
        end
        if self.Portrait_2d or not (Tex and Coord) then
            SetPortraitTexture(self.IconFrame.Portrait_2d, self.unit)
            Coord = Full_TexCoord
        else
            self.IconFrame.Portrait_2d:SetTexture(Tex)
        end
        self.IconFrame.Portrait_2d:SetTexCoord(unpack(Coord))
    end

    -- Name, Level colors
    SingleAdjustColors(self, true)
    local NameText
    local NameColor
    if self.NonGroupFrame then
        -- Name
        NameColor = FACTION_BAR_COLORS[UnitReaction(unit, 'player')] or RGB_White
        NameText = Str_Color(Name, NameColor.r, NameColor.g, NameColor.b)
        
        -- Classification
        local L_classification = classification[UnitClassification(unit)] or ''
       
        -- Level
        if Level < 1 then 
            Level = '??' 
        end         
        Level = Str_Color(Level, getDifficultyColor(Level))
        if self.frameType == "oUF_Lunar_Target" then
            NameText = Level..L_classification..' '..NameText
        end
    else
        if self.NameColor_Class then
            local cc = self.colors.class[self.CurrentUnitClass]
            if cc then
                NameText = Str_Color(Name, unpack(cc))
            else
                NameText = Str_Color(Name, RGB_White.r, RGB_White.g, RGB_White.b)
            end            
        elseif self.NameColor_Level then
            if UnitLevel(unit) then
                NameText = Str_Color(Name, getDifficultyColor(UnitLevel(unit)))
            else
                NameText = Str_Color(Name, RGB_White.r, RGB_White.g, RGB_White.b)
            end
        end
    end
    
    self.Name:SetText(NameText or "")    
    if Server then Name = strjoin("-",Name,Server) end
    self.LookupName = Name
    if self.Lunar_HealCommBar then
        self.Lunar_HealCommBar:SetValue(0)
    end
end 
---------------------------------------------------- Health
local function UpdateHealth(self, event, unit, bar, min, max) 
    -- Name Update if needed    
    if event~='OnHealthUpdate' then
--~         debug('event= '..(event or 'event'))
        self:UNIT_NAME_UPDATE(event, unit)
        if not self.Unit_Name then return end
    end
    -- OffStatus
    if(UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) and unit~='pet' and not UnitIsPlayer(unit)) then 
        bar.value:SetText('Tapped')
        bar:SetStatusBarColor(unpack(bar.ClassColor or OffStatus))
        return
    elseif not UnitIsConnected(unit) then 
        bar:SetValue(0)       
        if UnitIsPlayer(unit) then
            bar.value:SetText('Offline')
        else
            bar.value:SetText('')
        end
        bar.total:SetText('')
        bar:SetStatusBarColor(unpack(bar.ClassColor or OffStatus))
        return
    elseif(UnitIsGhost(unit)) then 
        bar:SetValue(0) 
        bar.value:SetText('Ghost')
        bar.total:SetText('')
        bar:SetStatusBarColor(unpack(bar.ClassColor or OffStatus))
        return
    elseif(UnitIsDead(unit)) then 
        bar:SetValue(0) 
        bar.value:SetText('Dead')
        bar.total:SetText('')
        bar:SetStatusBarColor(unpack(bar.ClassColor or OffStatus))
        return
    -- Health numbers
    elseif min<max or (min==max and not bar.HideTextWhenFull) then
        bar.value:SetFormattedText('%d%%', (min/max)*100)
        if not bar.Deficit then
            bar.total:SetFormattedText('%s/%s', letter(min), letter(max))
        else
            bar.total:SetFormattedText('-%s', letter(max-min))
        end        
    else
        bar.value:SetText('')
        bar.total:SetText('')
    end
    
    -- Colors
    if bar.ClassColor then
        return
    elseif min==max then
        bar:SetStatusBarColor(select(17, unpack(bar.smoothGradient)))
    else
        bar:SetStatusBarColor(bar.ColorAlphaGradient(min/max, unpack(bar.smoothGradient)))
    end
end 

---------------------------------------------------- Power 
local function UpdatePower(self, event, unit, bar, min, max)    
    if ((max == 0 or UnitIsDead(unit) or UnitIsGhost(unit) or not UnitIsConnected(unit)) and bar.value) then 
        bar.value:SetText() 
        if bar.InvertBars then
            bar:SetValue(max) 
        else
            bar:SetValue(0) 
        end
        bar.total:SetText('')
        bar.value:SetText('')
        return;
    end

    local PowerType = UnitPowerType(unit)
    if (PowerType==1 or PowerType==6) and bar.InvertBars then 
        bar:SetValue(max-min)
    end

    if (min>0 and min<max) or (not bar.HideTextWhenFull and min>=0 and min<=max) then
        bar.value:SetFormattedText('%d%%', (min/max)*100)
        if not bar.Deficit then
            bar.total:SetFormattedText('%s/%s', letter(min), letter(max))
        else
            bar.total:SetFormattedText('-%s', letter(max-min))
        end        
    else
        bar.value:SetText('')
        bar.total:SetText('')
    end    
   
    if not bar.ClassColor then
        bar:SetStatusBarColor(bar.ColorAlphaGradient(min/max, unpack(bar.smoothGradient)))
    end
end 

---------------------------------------------------- Pet Happiness
local function PostUpdateHappiness(self, event, unit, happiness)
    if ( happiness == 1 or happiness ==2 ) then
        self.Happiness:Show();
    elseif happiness == 3 then

        self.Happiness:Hide();
    end    
end
---------------------------------------------------- Druid Mana - (Thanks Caellian for the code)
oUF_Lunar.UpdateDruidMana = function(self)
	local pType = UnitPowerType('player')
	if(pType ~= 0) then
        local manaMin = UnitPower('player', 0)
		local manaMax = UnitPowerMax('player', 0)

		if(manaMin ~= manaMax) then
			self:SetFormattedText('%d%%', (manaMin/manaMax)*100)
		else
			self:SetText()
		end

		self:SetAlpha(1)
	else
		self:SetAlpha(0)
	end
end
---------------------------------------------------- Vehicle
local function Register_Pet_Delay(petframe)
	local f = CreateFrame("Frame")
	f:RegisterEvent("UNIT_ENTERED_VEHICLE")
	f:RegisterEvent("UNIT_EXITED_VEHICLE")	
    f:RegisterEvent("UNIT_PET")	
	local delay = 1
	local function OnUpdate(self, elapsed)
		self.elapsed = (self.elapsed or delay) - elapsed
		if self.elapsed < 0 then
			petframe:PLAYER_ENTERING_WORLD()
			self:SetScript("OnUpdate", nil)
		end
	end

	f:SetScript("OnEvent", function(self, event, unit)
		if unit == "player" then
			f.elapsed = delay
			f:SetScript("OnUpdate", OnUpdate)
		end
	end)
end

--[[****************************************************************************************************************************************************]]--
--                                                   AURA HANDLING
--[[****************************************************************************************************************************************************]]--
---------------------------------------------------- Button Facade
local function SetAuraOverlay(button)
    button.Original_Overlay:Hide()

    if button.__bf_normaltexture then
        button.overlay = button.__bf_normaltexture
        button.__bf_normaltexture = nil
        DebuffTypeColor.none.r, DebuffTypeColor.none.g, DebuffTypeColor.none.b = button.overlay:GetVertexColor()
        button:GetParent().showType = true
        button:GetParent().showDebuffType = true
    else
        button.overlay = button.Original_Overlay
        button:GetParent().showType = false
        button:GetParent().showDebuffType = false
    end
    button:SetFrameLevel(21)
end
local function OnSkin(self, skin, glossAlpha, gloss, group, _, colors)
	local styleDB
	if group == 'Auras' then
		styleDB = Profile.ButtonFacade_Style[group]
	end
    if not styleDB then return end
    styleDB[1] = skin
    styleDB[2] = glossAlpha
    styleDB[3] = gloss
    styleDB[4] = colors
    for _, button in ipairs(AuraIcons) do
        SetAuraOverlay(button)            
    end
    RefreshFrames("Aura")
end
local function GetButtonName(self)
    return self.Name
end
local function ButtonFacade_Button_Setup(group, button)
    if not LBF_Groups[group] then return end
    local ButtonName = "LUF_AURA_"..AuraButtonNameIndex
    AuraButtonNameIndex = AuraButtonNameIndex +1
    _G[ButtonName] = button
    _G[ButtonName..'Icon'] = button.icon
    _G[ButtonName..'Cooldown'] = button.cd
--~     _G[ButtonName..'Count'] = button.count
    button.Name = ButtonName
    button.GetName = GetButtonName
	LBF_Groups[group]:AddButton(button)
    SetAuraOverlay(button)
end
---------------------------------------------------- Aura Monitoring
local function ToggleAuraMonitoring(enabled)
    KA_update = enabled and OptionsMenu.KA_Update_Via_UnitAura;
    Profile.GatherNewAuras.enabled = enabled
    if enabled then
        OptionsMenu.CombatLogParser:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
        if not Spell_ID_Database_Exists then
            OptionsMenu.Build_Spell_ID_Database()
            Spell_ID_Database_Exists = true
        end
    else
        OptionsMenu.CombatLogParser:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    end
end
---------------------------------------------------- Modify Custom Aura List
local function ModifyCustomAuraList(frameType, AuraName, state, list, announce)
    local ScreenAuraName = gsub(AuraName, "_", " ")
    local ScreenFrameName = gsub(strformat("%s", frameType), "oUF_Lunar_", "")
    ScreenFrameName = gsub(ScreenFrameName, "oUF_", "")
    ScreenFrameName = gsub(ScreenFrameName, "_", " ")
    
    if state then
        if announce then message('Lunar Unit Frames: "'..ScreenAuraName..'" on "'..ScreenFrameName..'" added to "'..list..'"') end
        Profile[list][frameType][AuraName] = true
    else
        Profile[list][frameType][AuraName]=nil  
        if announce then message('Lunar Unit Frames: "'..ScreenAuraName..'" on "'..ScreenFrameName..'" removed from "'..list..'"') end
    end  
end
---------------------------------------------------- Aura OnClick Script
local function Aura_OnClick(self, mouseButton, MainSelf)
--~     if not Profile then return end
--~     local MainSelf = self:GetParent():GetParent()

    local frameType =  MainSelf.frameType 
--~     gsub(MainSelf:GetName(), "%d", "")
    local filter = self.filter
    local index = self:GetID()
    local unit = MainSelf.unit

    local name = UnitAura(unit, index, filter)
    if not name then return end
    local IndexName = gsub(name, " ", "_")
    
    if IsAltKeyDown() then
        if mouseButton == 'RightButton' then
            if Profile.PriorityAuras[frameType][IndexName] then
                ModifyCustomAuraList(frameType, IndexName, false, 'PriorityAuras', "announce")
            else
                ModifyCustomAuraList(frameType, IndexName, true, 'PriorityAuras', "announce")
            end
        elseif mouseButton == 'LeftButton' then
            ModifyCustomAuraList(frameType, IndexName, true, 'HiddenAuras', "announce")
        elseif mouseButton == 'MiddleButton' then
            if Profile.SelfCastPriority[frameType][IndexName] then
                ModifyCustomAuraList(frameType, IndexName, false, 'SelfCastPriority', "announce")
            else
                ModifyCustomAuraList(frameType, IndexName, true, 'SelfCastPriority', "announce")
            end
        end
    elseif mouseButton == 'RightButton' then
        CancelUnitBuff('player', index)
    end

    
    RefreshFrames("Aura")
    if OptionsMenu then
        OptionsMenu.SubMenu_2.AurasScrollFrame_Update()
    end
    
end

---------------------------------------------------- Debuff Animations
local function SetUpAnimGroup(self)
    -- Zoom
    do
        self.Animate_Icon = self:CreateAnimationGroup("Zoom")
        local Grow = 4
        local Dur = 0.3
        
        -- Step 1
        self.Animate_Icon.ZoomIn = self.Animate_Icon:CreateAnimation("SCALE", "ZoomIn")
        self.Animate_Icon.ZoomIn:SetScale(Grow, Grow) 
        self.Animate_Icon.ZoomIn:SetDuration(Dur)
        self.Animate_Icon.ZoomIn:SetOrder(1)
        
        self.Animate_Icon.Spin = self.Animate_Icon:CreateAnimation("ROTATION", "Spin")
        self.Animate_Icon.Spin:SetDegrees(360) 
        self.Animate_Icon.Spin:SetDuration(Dur)
        self.Animate_Icon.Spin:SetOrder(1)
        
        -- Step 2
        self.Animate_Icon.ZoomOut = self.Animate_Icon:CreateAnimation("SCALE", "ZoomOut")
        self.Animate_Icon.ZoomOut:SetScale(1/Grow, 1/Grow) 
        self.Animate_Icon.ZoomOut:SetOrder(2)
        self.Animate_Icon.ZoomOut:SetDuration(Dur)
                
--~         self.Animate_Icon.Spin2 = self.Animate_Icon:CreateAnimation("ROTATION", "Spin")
--~         self.Animate_Icon.Spin2:SetDegrees(180) 
--~         self.Animate_Icon.Spin2:SetDuration(Dur)
--~         self.Animate_Icon.Spin2:SetOrder(2)
    end
end


local function Animate()
    if GetTime()-0.1 > this.Age then
        this:SetScript("OnUpdate", nil)
        if not this.Animate_Icon:IsPlaying() then
            this.Animate_Icon:Play()
        end
    end
end
---------------------------------------------------- PostCreateAuraIcon
local function PostCreateAuraIcon(self, button, icons)
    icons.showDebuffType = true
--~     icons.showType = true
    button.cd:SetReverse() 
    local frame = self
    button:SetScript('OnClick', nil) 
    button:SetScript('OnMouseDown', function(self, mouseButton)
        Aura_OnClick(self, mouseButton, frame)
    end) 

    -- Animation setup
    if not button.Animate_Icon then
        SetUpAnimGroup(button)
	end
    
    -- Button Skinning
    if ButtonFacade then
        button.Original_Overlay = button.overlay
        tinsert(AuraIcons, button)
        ButtonFacade_Button_Setup('Auras', button)
    else
        button.icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        button.overlay:ClearAllPoints()
        button.overlay:SetPoint('TOPLEFT', -1, 1)
        button.overlay:SetPoint('BOTTOMRIGHT', 1, -1)
    end
end
---------------------------------------------------- Aura Post Update
local function PostUpdateAuraIcon(CustomFrameData, self, icons, unit, icon, index, offset, filter, isDebuff)
    if not icon.Showing then return end
    local duration, expireTime = select(6, UnitAura(self.unit, index, filter))
    icon.IsNew  = (duration + GetTime() - expireTime) < 0.1
    
    --update database
    if KA_update and (icon.IsNew or not (duration > 0)) then
        KA_update(self.unit, index, icon.filter)
    end
    
    -- Overlay coloring
    if not (DebuffTypeColor[icon.dType] or ButtonFacade) then
        icon.overlay:Hide()
    end 
    
    -- Cooldown spirals
    if duration > 0 and ((icon.IsPriority and CustomFrameData.CooldownSpiralPriority) or CustomFrameData.CooldownSpiral) then
        icon.cd:SetCooldown(expireTime - duration, duration)
        icon.cd:Show()
    else
        icon.cd:SetCooldown(0, 0)
        icon.cd:Hide()
    end
end

---------------------------------------------------- Aura Filter
local function Aura_Filter(SelfCastPriority, Priority, Hidden, Auras, unit, icon, name, rank, texture, count, dtype, duration, timeLeft, caster)
    if not name then return end
    local isPlayer
    local isFriendly
    if caster then 
        isPlayer = UnitIsUnit(caster, 'player') or UnitIsUnit(caster, 'vehicle')
        isFriendly = UnitIsFriend("player",caster)
    end
    icon.owner = caster
    icon.dType = dtype
    --filter if hidden
    local IndexName = gsub(name, " ", "_")
    -- Check if priority
    if (Priority[IndexName] or (SelfCastPriority[IndexName] and isPlayer)) then
        icon.IsPriority = true
        Auras.Vis = Auras.Vis +1
        icon.Showing = true
        return true;
    -- Check if casted by group or hidden
    elseif Auras.MouseOverMode or (Auras.onlyShowPlayer and isFriendly and not isPlayer) or Hidden[IndexName] then    
        return
    else
        -- process as normal
        icon.IsPriority = false
        icon.Showing = true
        Auras.Vis = Auras.Vis +1
        return true;
    end
end
    
---------------------------------------------------- Set Aura Position
local function SetAuraPosition(self, CustomFrameData, Low_Buffs, Low_Debuffs, Auras)
	if Auras.Vis==0 then return end
    local IsDebuff
    local P_col, P_row, B_col, B_row, D_col, D_row = 0,0,0,0,0,0;
    local P_gap = true
    local P_spacing, P_size, P_anchor, P_growth_X, P_growth_Y, P_cols, P_Aspect = unpack(Auras.Position_Info);
    local B_spacing, B_size, B_anchor, B_growth_X, B_growth_Y, B_cols, B_Aspect = unpack(Low_Buffs.Position_Info);
    local D_spacing, D_size, D_anchor, D_growth_X, D_growth_Y, D_cols, D_Aspect = unpack(Low_Debuffs.Position_Info);
    
    -- Downscale if too many auras
    local VisibleAuras = Auras.Vis + 1
    if VisibleAuras > (B_cols * 2) then
        local B_rows = math_ceil( (VisibleAuras / B_Aspect) ^ 0.5)
        B_cols = math_floor(B_rows * B_Aspect)
        B_size = B_size * (2 / B_rows)
        D_size, D_cols = B_size, B_cols;
    end
    Auras.Vis = 0
    -- Process each icon
    for _, icon in ipairs(Auras) do
        if icon.Showing then
            icon.Showing = false
            IsDebuff = icon.debuff
            icon:ClearAllPoints()
            -- Priority Auras
            if icon.IsPriority then
                --Set size
                icon:SetScale(P_size)
                --Set in priority position
                if(P_gap and IsDebuff) then
                    if(P_col > 0) then
                        P_col = P_col + 0.5
                    end
                    P_gap = false
                end
                if(P_col >= P_cols) then
                    P_col = 0
                    P_row = P_row + 1
                end
                icon:SetPoint(P_anchor, Auras, P_anchor, (P_col * P_spacing * P_growth_X), (P_row * P_spacing * P_growth_Y))
                P_col = P_col + 1
            else
                -- Normal Debuffs
                if IsDebuff then
                    --Set size
                    icon:SetScale(D_size)
                    --Set in low_debuff position
                    if(D_col >= D_cols) then
                        D_col = 0
                        D_row = D_row + 1
                    end
                    icon:SetPoint(D_anchor, Low_Debuffs, D_anchor, (D_col * D_spacing * D_growth_X), (D_row * D_spacing * D_growth_Y))
                    D_col = D_col + 1
                -- Normal Buffs
                else
                    --Set size
                    icon:SetScale(B_size)
                    --Set in low_buff position
                    if(B_col >= B_cols) then
                        B_col = 0
                        B_row = B_row + 1
                    end
                    icon:SetPoint(B_anchor, Low_Buffs, B_anchor, (B_col * B_spacing * B_growth_X), (B_row * B_spacing * B_growth_Y))
                    B_col = B_col + 1
                end
            end
            --Set Animation
            if icon.IsNew and IsDebuff and CustomFrameData.Animations then
                icon.Age = GetTime()
                icon:SetScript("OnUpdate", Animate)
            end
        end
    end
end


--[[****************************************************************************************************************************************************]]--
--                                                   LOADING FUNCTIONS
--[[****************************************************************************************************************************************************]]--

local function Load_SharedMedia(Lunar_Media, MediaType)

    -- Grizzly Shared Media
    if Media and Media.registry[MediaType] then
        local Grizzly = Media.registry[MediaType]
        if Grizzly then
            for _, entry in ipairs(Grizzly) do
                key, data = unpack(entry)
                Lunar_Media[key] = data
            end
        end
    end
    
    -- Shared Media
    local LibStub = _G['LibStub']    
    if not LibStub then return end
    local SharedMedia, SharedMedia_MediaType, key, data;
    
    for i=1, 10 do
        SharedMedia = LibStub:GetLibrary("LibSharedMedia-"..i..'.0', true)
        if SharedMedia then
            SharedMedia_MediaType = SharedMedia.MediaTable and SharedMedia.MediaTable[MediaType]
            if SharedMedia_MediaType then
                for key in pairs(SharedMedia_MediaType) do
                    Lunar_Media[key] = SharedMedia_MediaType[key]
                end
            end
        end
    end
end
---------------------------------------------------- Build unit frames
-- Spawning template to for unit frames
local function Lunar_Spawn(unit, oUF_Name, HeaderName, fType, template)
    HeaderName = HeaderName or oUF_Name
    local MasterUnitType
    if (fType or oUF_Name) == 'oUF_Lunar_X_Target' then
        MasterUnitType =  gsub(gsub(HeaderName, 'Target', '', 1), '%d', '')
    end
    if not (Profile.LoadFrameTypes[MasterUnitType or fType or oUF_Name] and Profile.LoadFrameTypes[fType or oUF_Name]) then 
        return
    end
    if not DragableFrames[HeaderName] then
        DragableFrames[HeaderName] =  CreateFrame('Frame', HeaderName, UIParent)
    end
    local Header = DragableFrames[HeaderName]
    Header.DisplayName = DisplayNames[HeaderName]
    unit = strlower(unit)
    unit = conv[unit] or unit
    local self = oUF:Spawn(unit, oUF_Name, template)
    self.MasterUnitType = MasterUnitType
    self:SetParent(Header)
    self:SetPoint('TOPLEFT', Header, 'TOPLEFT')
    self:SetPoint('RIGHT', Header, 'RIGHT')
    if unit == 'header' then
        self:SetPoint('BOTTOM', Header, 'BOTTOM')
    end
    self.HeaderName = HeaderName
    self.frameType = fType or oUF_Name
    for i, fType in ipairs(oUF_Lunar_FrameTypes) do
        if fType == self.frameType then
            return(self)
        end
    end
    tinsert(oUF_Lunar_FrameTypes, self.frameType)
    return (self)
end

local function BuildUnitFrames()
    oUF:SetActiveStyle('Lunar')
    local Header
    local uType

    -- Spawn single unit frames 
    for _, uType in ipairs(SingleUnitFrames) do
        Lunar_Spawn(uType, 'oUF_Lunar_'..uType)
        if uType~='Player' then
            TargetOf[uType] = Lunar_Spawn(uType..'target', 'oUF_Lunar_X_Target', 'oUF_Lunar_'..uType..'Target')
        end
    end
    
    -- Party
    Header = Lunar_Spawn('header', 'oUF_Lunar_Party')
    if Header then
        Header:Show()
        Header:SetAttribute('unitsPerColumn', 4)
        OpenUnitHeaders['oUF_Lunar_Party'] = Header
        OpenUnitHeaders[1] = Header
    end
    
    -- Party Targets
    Header = Lunar_Spawn('header', 'oUF_Lunar_Party', 'oUF_Lunar_PartyTarget', 'oUF_Lunar_X_Target')
    if Header then
--~         Header:SetAttribute('showParty', true)
        Header:SetAttribute('template', 'oUF_Lunar_Party')
        Header:SetAttribute('unitsPerColumn', 4)
        Header:Show()
        TargetOf['Party'] = Header
        OpenUnitHeaders[2] = Header
    end
            
    -- PartyPets
    Header = Lunar_Spawn('header', 'oUF_Lunar_PartyPet','oUF_Lunar_PartyPet','oUF_Lunar_PartyPet','SecureGroupPetHeaderTemplate')
    if Header then
        Header:Show()
        Header:SetAttribute('unitsPerColumn', 4)
        OpenUnitHeaders['oUF_Lunar_PartyPet'] = Header
        OpenUnitHeaders[3] = Header
    end
    
    -- Raid
    local RaidNumber = 0
    for i = 1, RaidGroupCount do
        Header = Lunar_Spawn('header', 'oUF_Lunar_Raid','oUF_Lunar_Raid'..i)
        if Header then
            Header:SetAttribute('groupFilter', tostring(i))
--~             Header:SetAttribute('showRaid', true)
            Header:Show()
            Header.RaidMode = true
            OpenUnitHeaders['oUF_Lunar_Raid'..i] = Header
        end
    end
    
    -- Arena
    for i = 1, 5 do
        uType = 'Arena'..i
        Lunar_Spawn(uType, 'oUF_Lunar_'..uType,'oUF_Lunar_'..uType, 'oUF_Lunar_Arena')
        Lunar_Spawn('arenapet'..i, 'oUF_Lunar_'..uType..'Pet','oUF_Lunar_'..uType..'Pet', 'oUF_Lunar_ArenaPet')
        TargetOf[uType] = Lunar_Spawn('arena'..i..'target', 'oUF_Lunar_X_Target', 'oUF_Lunar_'..uType..'Target')
    end
    
    -- Set monitors
    PartyMonitor = CreateFrame('Frame')
    PartyMonitor:SetScript('OnEvent', PartyUpdate)
    PartyMonitor:RegisterEvent('RAID_ROSTER_UPDATE')
    PartyMonitor:RegisterEvent('PARTY_LEADER_CHANGED')
    PartyMonitor:RegisterEvent('PARTY_MEMBER_CHANGED')
    PartyMonitor:RegisterEvent('GUILD_ROSTER_UPDATE')
    PartyMonitor:RegisterEvent('FRIENDLIST_UPDATE')
end

---------------------------------------------------- Profile Specific Style
local function ProfileSpecificStyle(self)
    local frameName = self:GetName()
    local frameType = self.frameType
    
    if not Profile and frameType then return; end
    
    SingleAdjustBarHeight(self)
    SingleAdjustTextVisibility(self)
    SingleAdjustAuraSize(self)
    SingleAdjustFrame(self) 
    SingleAdjustFrameWidth(self)
    SingleAdjustRangeAlpha(self)
    SingleAdjustScale(self)
    SingleAdjustTextures(self)
    SingleAdjustColors(self)
    SingleAdjustFont(self)
    SingleAdjustFrameHeight(self)
    
    local CustomFrameData = Profile.CustomFrames[frameType]
    
    if CustomFrameData.RuneFrame and UnitClass('player')=='Death Knight' then
        RuneFrame:Show()
    else
        RuneFrame:Hide()
    end
    
--~     self:SetScript("OnUpdate", nil)
    
    -- Custom Aura
    local Priority = Profile.PriorityAuras[frameType]
    local Hidden = Profile.HiddenAuras[frameType]
    local SelfCastPriority = Profile.SelfCastPriority[frameType]
    local Auras = self.Auras
    local Low_Buffs = self.Low_Buffs
    local Low_Debuffs = self.Low_Debuffs
    
    self.PostUpdateAuraIcon = function(...)
        PostUpdateAuraIcon(CustomFrameData, ...)
    end
    
    self.SetAuraPosition = function()            
        SetAuraPosition(self, CustomFrameData, Low_Buffs, Low_Debuffs, Auras)
    end

    self.CustomAuraFilter = function(...)
        return Aura_Filter(SelfCastPriority, Priority, Hidden, ...) and true
    end
end
---------------------------------------------------- Update Profile
local function Update_Profile(ProfileGettingUpdated, ProfileToUpdateTo)
    local function Scan(OldTable, NewTable)
        for index, value in pairs(NewTable) do
            if OldTable[index]==nil and type(NewTable[index])~='boolean' then -- This is to prevent adding values to aura tables
                -- Copy defaults if needed
                OldTable[index] = deepcopy(NewTable[index])
            elseif type(NewTable[index])=='table' then
                -- Scan sub-tables if needed
                Scan(OldTable[index], NewTable[index])
            end
        end
    end
    Scan(ProfileGettingUpdated, ProfileToUpdateTo)
end
---------------------------------------------------- Load Profile
function oUF_Lunar_SetProfile(NewProfile, first)    
    
    oUF_Lunar_SelectedProfile = NewProfile or oUF_Lunar_SelectedProfile or "default";
    
    if not oUF_Lunar_Profiles then
        oUF_Lunar_Profiles = {}
    end
    
    Profile = oUF_Lunar_Profiles[oUF_Lunar_SelectedProfile]
    
    -- Set to defaults if needed
    if not (Profile and Profile.version) then
        oUF_Lunar_Profiles[oUF_Lunar_SelectedProfile] = deepcopy(Lunar_Default_Options)
        Profile = oUF_Lunar_Profiles[oUF_Lunar_SelectedProfile]
        Profile.Name = oUF_Lunar_SelectedProfile
        Profile.version = oUF_Lunar_Version and oUF_Lunar_Version or Profile.version
    elseif Profile.version < oUF_Lunar_Version then
        message('Lunar Unit Frames: "'..oUF_Lunar_SelectedProfile..'" profile was version '..Profile.version..', updated to '..oUF_Lunar_Version..'. Please look at the new configuration options.')
        Update_Profile(Profile, Lunar_Default_Options)
        Profile.version = oUF_Lunar_Version and oUF_Lunar_Version or Profile.version
    end
    Profile.timesloaded = Profile.timesloaded + 1
        
    if first then
        -- Build things
        RaidGroupCount = Profile.RaidGroupCount or RaidGroupCount
        BuildUnitFrames()
    else
        -- Notify of new profile load
        message('Lunar Unit Frames: "'..oUF_Lunar_SelectedProfile..'" Profile loaded ('..Profile.timesloaded..').')
    end
    
    --Move frames
    for HeaderName in pairs(DragableFrames) do
        local self = DragableFrames[HeaderName]
        local P = Profile.FramePosition[HeaderName]
        self:ClearAllPoints()
        if not P then
            message('Lunar: No position for '..HeaderName..' moving to center.')
            self:SetPoint('CENTER')
        else
            self:SetPoint(P.FromPoint, getglobal(P.Parent) and P.Parent or 'UIParent', P.ToPoint, P.X, P.Y)
        end
    end
    
    for _, frameType in ipairs(oUF_Lunar_FrameTypes) do
        AdjustFrameWidth(frameType)
        AdjustSpacers(frameType)
    end
    
    --Profile specific  style
    for _, self in ipairs(OpenUnitFrames) do
        ProfileSpecificStyle(self)
    end
    
    ShowHideFrames()
    ShowHideBlizzardFrames('Global')
    
    -- ButtonFacade
    if LBF_Groups['Auras'] then
        LBF_Groups['Auras']:Skin(unpack(Profile.ButtonFacade_Style['Auras']))
    end
    
    --Refresh
    RefreshFrames()
    AdjustFont('Global')  
    if OptionsMenu then
        ToggleAuraMonitoring(Profile.GatherNewAuras.enabled)
        OptionsMenu.SubMenu_4.refresh()
        UpdateMinimapButton(Profile.Minimap.enabled==1)
    end   
    PartyUpdate('FRIENDLIST_UPDATE')
    
    --Summary
    return (oUF_Lunar_Profiles[oUF_Lunar_SelectedProfile])
end
---------------------------------------------------- oUF_Lunar style
oUF:RegisterStyle('Lunar', function(self, unit)
    -- Labels
    local frameName = self:GetName()
    local frameType = gsub(gsub(frameName, "%d", ''), 'UnitButton', '')
    local frameParent = self:GetParent()
    self.MasterUnitType = self.MasterUnitType or frameParent.MasterUnitType
    self.HeaderName = self.HeaderName or frameParent.HeaderName
    if self.MasterUnitType then
        -- Handle TargetOf frames
        frameType = 'oUF_Lunar_X_Target'
        if strmatch(frameName, 'Button') then
            -- This is for placeholder frames
            self:SetAttribute('initial-width', 1)
            self:SetAttribute('initial-height', 32)
            -- Handle TargetOf frames in party/raid
            if strmatch(frameName, 'Target') then
                local index = strmatch(frameName, '%d')
                self:SetPoint('CENTER')
            else
                -- This is for placeholder frames
                self.ignoreHealComm = true
                self:EnableMouse(false)
                return self
            end
        end
    end

    self.frameType = frameType
    -- add to frame databases
    tinsert(OpenUnitFrames, self)
    
    -- run static layout
    oUF_Lunar_Static_Style(self, unit)
    
    -- set functions
    self.menu = RightClickMenu
    self.UNIT_NAME_UPDATE = UpdateName
    self.OverrideUpdateHealth = UpdateHealth
    self.OverrideUpdatePower = UpdatePower
    self.PostCreateAuraIcon = PostCreateAuraIcon
    self.PostUpdateHappiness = PostUpdateHappiness
    self.PostUpdateHealth = Lunar_Update_HealCommBar
    -- PostUpdateHealth is used for HealComm
    if unit=='pet' then
        Register_Pet_Delay(self)
    end    
    -- Profile specific settings
    ProfileSpecificStyle(self)
    return self;
end)

---------------------------------------------------- Event Manager
local EventManager = CreateFrame("FRAME"); -- Need a frame to respond to events
EventManager:RegisterEvent("VARIABLES_LOADED");
EventManager:RegisterEvent("PLAYER_ENTERING_WORLD");

function EventManager:OnEvent(event)
    if event == "VARIABLES_LOADED" then
        EventManager:UnregisterEvent("VARIABLES_LOADED");
        Load_SharedMedia(Textures_Hash, 'statusbar')
        Load_SharedMedia(Font_Hash, 'font')
        
        ShowHideBlizzardFrames()

        -- Set Profile
        oUF_Lunar_SetProfile(nil, true)
        
        -- Build options menu
        if oUF_Lunar_BuildOptionsMenu then
            local Functions_To_Export = {
                DragableFrames, 
                OpenUnitFrames, 
                ShowHideFrames, 
                PartyUpdate, 
                AdjustRangeAlpha, 
                AdjustFrame, 
                AdjustAuraSize, 
                AdjustSpacers, 
                AdjustScale, 
                RefreshFrames, 
                Textures_Hash,
                Font_Hash,
                AdjustTextures,
                AdjustColors,
                oUF_Lunar_FrameTypes,
                Lunar_Default_Options,
                AdjustFrameWidth,
                PartyUpdate,
                ToggleAuraMonitoring,
                ShowHideBlizzardFrames,
                AdjustBarHeight,
                AdjustFont,
                AdjustTextVisibility,
                AdjustFrameHeight
            }
            OptionsMenu, UpdateMinimapButton = oUF_Lunar_BuildOptionsMenu(Functions_To_Export)
        end
        Player_Has_Loaded = true
        
    elseif event == "PLAYER_ENTERING_WORLD" then
        EventManager:UnregisterEvent("PLAYER_ENTERING_WORLD");
        PartyUpdate('GUILD_ROSTER_UPDATE')
        if OptionsMenu then
            if Profile.GatherNewAuras.enabled then
                ToggleAuraMonitoring(Profile.GatherNewAuras.enabled)
            end
            UpdateMinimapButton(Profile.Minimap.enabled==1)
            OptionsMenu.SubMenu_4.refresh()
        end
        -- This is done to set active oUF elements
        AdjustBarHeight('Global')  
        AdjustFrameElements('Global')        
        -- Button facade support
        local bf = LibStub('LibButtonFacade', true)
        if bf then
            LBF_Groups['Auras'] = bf:Group('Lunar Unit Frames', 'Auras')
            LBF_Groups['Auras']:Skin(unpack(Profile.ButtonFacade_Style['Auras']))
            bf:RegisterSkinCallback('Lunar Unit Frames', OnSkin, LBF_Groups['Auras'])
            for _, icon in ipairs(AuraIcons) do
                if not icon:GetName() then
                    ButtonFacade_Button_Setup("Auras", icon)
                end
            end
        end
        
        -- Register right click menu entrys
        hooksecurefunc("UnitPopup_OnClick", UnitPopup_OnClick);
        RegisterRightClick("Lunar Unit Frames: Add to priority raid", function(name)
            Modify_PriorityRaid(name, true)
        end,
        true
        )
        RegisterRightClick("Lunar Unit Frames: Remove from priority raid", function(name)
            Modify_PriorityRaid(name, false)
        end)
        local Old_RaidGroupButton_ShowMenu = RaidGroupButton_ShowMenu
        RaidGroupButton_ShowMenu = function(self) 
            RightClickMenu(self)
            Old_RaidGroupButton_ShowMenu(self)
        end
        
        -- Collect Garbage
        collectgarbage("collect")
    end
end
EventManager:SetScript("OnEvent", EventManager.OnEvent);
