local function debug (message)
    DEFAULT_CHAT_FRAME:AddMessage(message)
end
-- These can be changed
local DefaultWidth = 150
local DefaultHeight = 32
local DefaultSpacerY = 36
local DefaultSpacerY2 = 6
local DefaultSpacerX = 65
local RaidGroupCount = 8
local BarWidth = 127
local DefaultSpacerYGroup = 6
local X_Target_Scale = 0.70
local Focus_Scale = 0.80
local Pets_Scale = 0.8
local Arena_Scale = 1
local Arena_Height = 36
-- Frame names
local oUF_Lunar_FrameNames = { 
    "oUF_Lunar_Player", 
    "oUF_Lunar_PlayerPet", 
    "oUF_Lunar_Party", 
    "oUF_Lunar_PartyPet", 
    "oUF_Lunar_Raid", 
    "oUF_Lunar_Focus", 
    "oUF_Lunar_Target", 
    "oUF_Lunar_Arena",
    "oUF_Lunar_ArenaPet",
    "oUF_Lunar_X_Target",
}

-- Create default options
Lunar_Default_Options = {
    Version = 435,
    GlobalFont = 'Friz Quadrata TT',
    GlobalFontSizeModify = 0,
    timesloaded = 0,
    sortMethod = "Class",
    HiddenAuras = {},
    PriorityAuras = {},
    SelfCastPriority = {},
    CustomFrames = {},
    Tutorial_Tooltips = true,
    MOB_key = 'Is Alt Key Down',
    HealComm = true,
    MinimumRaid = 5,
    
    Minimap = {
        ["enabled"] = 1,
        ["drag"] = "CIRCLE",
        ["position"] = -100,
    },
    GatherNewAuras = { enabled = false },
    BlizzardFrames = {
        ['BuffFrame'] = false,
        ['Castbars'] = true,
    },
    PartyFilter = {
        Enabled = true,
        PartyFilterAllowGuild = false,
        PartyFilterAllowFriends = false,
    },
    LoadFrameTypes = {
        "oUF_Lunar_Target", 
        "oUF_Lunar_X_Target",
        "oUF_Lunar_Player", 
        "oUF_Lunar_PlayerPet", 
        "oUF_Lunar_Focus",
        "oUF_Lunar_Party", 
        "oUF_Lunar_PartyPet", 
        "oUF_Lunar_Raid", 
        "oUF_Lunar_Arena", 
        "oUF_Lunar_ArenaPet",
        oUF_Lunar_Target = 1, 
        oUF_Lunar_X_Target = 1,
        oUF_Lunar_Player = 1, 
        oUF_Lunar_PlayerPet = 1, 
        oUF_Lunar_Focus = 1, 
        oUF_Lunar_Party = 1, 
        oUF_Lunar_PartyPet = 1, 
        oUF_Lunar_Raid = 1, 
        oUF_Lunar_Arena = 1, 
        oUF_Lunar_ArenaPet = 1,
    },
    RaidGroupCount = 8,
    -- Button Facade
    ButtonFacade_Style = {
        Auras = {
            'Zoomed', -- [1]
            nil, -- [2]
            nil, -- [3]
            {
            }, -- [4]
        },
    },
}
-- Frame positions
do
    Lunar_Default_Options.FramePosition = {
        ["oUF_Lunar_Player"] = {
            ["Y"] = -DefaultSpacerY/2,
            ["X"] = DefaultSpacerX*2 + DefaultWidth -20,
            ["ToPoint"] = "LEFT",
            ["FromPoint"] = "LEFT",
            ["Parent"] = "UIParent",
        },
        ["oUF_Lunar_PlayerPet"] = {
            ["Y"] = -DefaultSpacerY2/2 / Pets_Scale,
            ["X"] = DefaultSpacerX+20,
            ["ToPoint"] = "LEFT",
            ["FromPoint"] = "LEFT",
            ["Parent"] = "UIParent",
        },
        ["oUF_Lunar_Party"] = {
            ["Y"] = -(DefaultHeight + DefaultSpacerY),
            ["X"] = DefaultSpacerX*2 + DefaultWidth -20,
            ["ToPoint"] = "LEFT",
            ["FromPoint"] = "TOPLEFT",
            ["Parent"] = "UIParent",
        },
        ["oUF_Lunar_PartyPet"] = {
            ["Y"] = -(DefaultHeight + DefaultSpacerY) / Pets_Scale,
            ["X"] = DefaultSpacerX+20,
            ["ToPoint"] = "LEFT",
            ["FromPoint"] = "TOPLEFT",
            ["Parent"] = "UIParent",
        },
        ["oUF_Lunar_Target"] = {
            ["Y"] = -DefaultSpacerY/2,
            ["X"] = -(DefaultWidth + DefaultSpacerX*2) +20,
            ["ToPoint"] = "RIGHT",
            ["FromPoint"] = "RIGHT",
            ["Parent"] = "UIParent",
        },
        ["oUF_Lunar_Focus"] = {
            ["Y"] = (85) / Focus_Scale  - DefaultSpacerY,
            ["X"] = -(DefaultWidth + DefaultSpacerX*2) / Focus_Scale,
            ["ToPoint"] = "RIGHT",
            ["FromPoint"] = "BOTTOMRIGHT",
            ["Parent"] = "UIParent",
        },
    }
    
    -- Target of *
    local function AssignPosition(name, RightSide)
        local Position = {
            Y = -5 / X_Target_Scale,
            X = (RightSide and 1 or -1) *5,
            ToPoint = RightSide and "TOPRIGHT" or "TOPLEFT",
            FromPoint = RightSide and "TOPLEFT" or "TOPRIGHT",
        }
        Position.Parent = "oUF_Lunar_"..name
        Lunar_Default_Options.FramePosition["oUF_Lunar_"..name.."Target"] = Position
        return Position
    end
    
    AssignPosition("Target", 'right')
    AssignPosition("Focus", 'right')
    AssignPosition("PlayerPet")
    AssignPosition("Party")

    -- Raid
    local floor = math.floor
    local StartX = 10
    local StartY = -10
    local RowCount = 3
    
    for i=1, RaidGroupCount do
        -- Find position based on i     
        local RowFinder = (i+RowCount) % RowCount
        local Row = RowFinder > 0 and RowFinder or RowCount
        local Col = floor ( (i+RowCount-1) / RowCount)
        
        local X = StartX + (DefaultWidth + 45)*(Col-1)
        local Y = StartY + (((DefaultHeight + DefaultSpacerYGroup)*5) + DefaultSpacerYGroup) * -(Row-1)

        -- Save values
        Lunar_Default_Options.FramePosition["oUF_Lunar_Raid"..i]= {
            ["X"] = X,
            ["Y"] = Y,
            ["ToPoint"] = "TOPLEFT",
            ["FromPoint"] = "TOPLEFT",
            ["Parent"] = "UIParent",
        }
    end
    
    -- Arena
    StartX = -(DefaultWidth + DefaultSpacerX*2) +20
    StartY = -(DefaultHeight + DefaultSpacerY)
    RowCount = 5
    for i=1, 10 do
        -- Find position based on i     
        local RowFinder = (i+RowCount) % RowCount
        local Row = RowFinder > 0 and RowFinder or RowCount
        local Col = floor ( (i+RowCount-1) / RowCount)
        
        local X = (StartX + (DefaultWidth + 40)*(Col-1))/Arena_Scale
        local Y = (StartY + (Arena_Height + DefaultSpacerY2 +2) * -(Row-1))

        -- Save values
        if i<=5 then
            Lunar_Default_Options.FramePosition["oUF_Lunar_Arena"..i]= {
                ["X"] = X,
                ["Y"] = Y,
                ["ToPoint"] = "RIGHT",
                ["FromPoint"] = "TOPRIGHT",
                ["Parent"] = "UIParent",
            }
            AssignPosition("Arena"..i, 'right')
        else
            Lunar_Default_Options.FramePosition["oUF_Lunar_Arena"..(i-5)..'Pet']= {
                ["X"] = X,
                ["Y"] = Y/Pets_Scale-32,
                ["ToPoint"] = "RIGHT",
                ["FromPoint"] = "RIGHT",
                ["Parent"] = "UIParent",
            }
        end
    end
end

-- Per frame type settings
do
    local Default_CF = Lunar_Default_Options.CustomFrames
    for _, name in ipairs(oUF_Lunar_FrameNames) do        
        Lunar_Default_Options.HiddenAuras[name] = {}
        Lunar_Default_Options.PriorityAuras[name] = {}
        Lunar_Default_Options.SelfCastPriority[name] = {}
        
        -- Custom Frame options
        Default_CF[name] = {
            -- Enabling
            Enabled = true,
            ShowWhileRaid = true,
            ShowTargetOf = true,
            
            -- Frame sizes
            FrameScale = 100,
            FrameWidth = 160,
            FrameHeight = 32,
            DefaultSpacerY = DefaultSpacerY2,
            PowerHeight = 9,
            CastBarHeight = 3,
            
            -- Bar Direction
            InvertBars = true,
            ReverseBars = false,
            
            -- Bar Text
            TotalText = false,
            ValueText = true,
            HideTextWhenFull = true,
            NameColor_Class = true,
            NameColor_Level = false,
            Deficit = false,

            -- Auras
            AuraLoc = 1,
            AuraSize = 32,
            AuraOffset = 0,
            AuraSpacing = 0.5,
            Animations = true,
            ShowCastable = false,
            CooldownSpiral = false,
            CooldownSpiralPriority = true,
            MouseOverAuras = true,

            -- Portrait
            PortraitLoc = -1,
            ModelPortrait = false,
            ClassPortrait = true,
            
            -- Textures
            HealthTexture = 'Comet',
            PowerTexture = 'Comet',
            CastTexture = 'Melli',
            
            -- Colors and alpha
            HealthClassColor = 0,
            HealthColor1 = {1,0,0,1}, -- ColorGradient
            HealthColor2 = {1,0.2,0,1}, -- ColorGradient
            HealthColor3 = {1,1,0,1}, -- ColorGradient
            HealthColor4 = {0.2,1,0,1}, -- ColorGradient
            HealthColor5 = {0,1,0,1}, -- ColorGradient
            HealthColor6 = {0.77,0.12,0.23,1}, -- DeathKnight
            HealthColor7 = {1,0.49,0.04,1}, -- Druid
            HealthColor8 = {0.67,0.83,0.45,1}, -- Hunter
            HealthColor9 = {0.41,0.8,0.94,1}, -- Mage
            HealthColor10 = {0.96,0.55,0.73,1}, -- Paladin
            HealthColor11 = {1,1,1,1}, -- Priest
            HealthColor12 = {1,0.96,0.41,1}, -- Rogue
            HealthColor13 = {0.14,0.35,1,1}, -- Shaman
            HealthColor14 = {0.58,0.51,0.79,1}, -- Warlock
            HealthColor15 = {0.78,0.61,0.43,1}, -- Warrior
            
            PowerClassColor = 1,
            PowerColor1 = {1,1,1,1}, -- ColorGradient
            PowerColor2 = {1,1,1,1}, -- ColorGradient
            PowerColor3 = {1,1,1,1}, -- ColorGradient
            PowerColor4 = {1,1,1,1}, -- ColorGradient
            PowerColor5 = {1,1,1,1}, -- ColorGradient
            PowerColor6 = {0.8,0.8,0.8,1}, -- DeathKnight
            PowerColor7 = {0,0,1,1}, -- Druid
            PowerColor8 = {0,0,1,1}, -- Hunter
            PowerColor9 = {0,0,1,1}, -- Mage
            PowerColor10 = {0,0,1,1}, -- Paladin
            PowerColor11 = {0,0,1,1}, -- Priest
            PowerColor12 = {1,1,0,1}, -- Rogue
            PowerColor13 = {0,0,1,1}, -- Shaman
            PowerColor14 = {0,0,1,1}, -- Warlock
            PowerColor15 = {1,0,0,1}, -- Warrior
            
            CastClassColor = 0,
            CastColor1 = {1,1,1,1}, -- ColorGradient
            CastColor2 = {1,1,1,1}, -- ColorGradient
            CastColor3 = {1,1,1,1}, -- ColorGradient
            CastColor4 = {1,1,1,1}, -- ColorGradient
            CastColor5 = {1,1,1,1}, -- ColorGradient
            CastColor6 = {0.77,0.12,0.23,1}, -- DeathKnight
            CastColor7 = {1,0.49,0.04,1}, -- Druid
            CastColor8 = {0.67,0.83,0.45,1}, -- Hunter
            CastColor9 = {0.41,0.8,0.94,1}, -- Mage
            CastColor10 = {0.96,0.55,0.73,1}, -- Paladin
            CastColor11 = {1,1,1,1}, -- Priest
            CastColor12 = {1,0.96,0.41,1}, -- Rogue
            CastColor13 = {0.14,0.35,1,1}, -- Shaman
            CastColor14 = {0.58,0.51,0.79,1}, -- Warlock
            CastColor15 = {0.78,0.61,0.43,1}, -- Warrior
            
            -- Background Color and alpha
            HealthColor99 = {0.1,0.1,0.1,0}, 
            PowerColor99 = {0.1,0.1,0.1,0},
            CastColor99 = {0,0,0,0},
            
            -- Misc
            RangeAlpha = 66,
            RuneFrame = true,
            ComboPoints = false,
            FrameBackDrop = true,
            SmoothBars = true,
            RestIcon = true,
            PvpIcon = true,
        } 
         -- Exceptions
        do
            local Default_CFN = Default_CF[name]
            -- Non-group frame settings
            if name == "oUF_Lunar_Target" or name == "oUF_Lunar_Focus" or name == "oUF_Lunar_X_Target" or name == "oUF_Lunar_Arena" or name == "oUF_Lunar_ArenaPet" then
                Default_CFN.InvertBars = false
                Default_CFN.Animations = false
                Default_CFN.AuraLoc = -1
                Default_CFN.PortraitLoc = 1
                Default_CFN.HealthColor99[4] = 1
                Default_CFN.PowerColor99[4] = 1
                Default_CFN.HealthTexture = 'Comet-Reversed'
                Default_CFN.PowerTexture = 'Comet-Reversed'
                Default_CFN.SmoothBars = false
                Default_CFN.HideTextWhenFull = false
            end
            
            -- Other exeptions
            if name == "oUF_Lunar_PlayerPet" then
                Default_CFN.ClassPortrait = false
                Default_CFN.ModelPortrait = true
                Default_CFN.FrameScale = 80
                Default_CFN.FrameWidth = 120
                Default_CFN.AuraSize = 20
            end
            
            if name == "oUF_Lunar_PartyPet" or name == "oUF_Lunar_ArenaPet" then
                Default_CFN.ClassPortrait = false
                Default_CFN.ModelPortrait = true
                Default_CFN.FrameScale = 80
                Default_CFN.DefaultSpacerY = DefaultSpacerY2 / Pets_Scale
                Default_CFN.AuraSize = 20
                Default_CFN.FrameWidth = 120
                Default_CFN.PowerHeight = 0
            end
                
            if name == "oUF_Lunar_Raid" then
                Default_CFN.Animations = false
                Default_CFN.RangeAlpha = 20
                Default_CFN.FrameScale = 80
                Default_CFN.CooldownSpiral = false
                Default_CFN.FrameWidth = 100
                Default_CFN.ClassPortrait = false
                Default_CFN.PowerHeight = 4
                Default_CFN.CastBarHeight = 0
                Default_CFN.Enabled = false
                Default_CFN.AuraSize = 24
                
            elseif name == "oUF_Lunar_Target" then
                Default_CFN.MouseOverAuras = false
                Default_CFN.DefaultSpacerY = DefaultSpacerY
                Default_CFN.ComboPoints = true
                Default_CFN.CooldownSpiral = true
                Default_CFN.TotalText = true
                Default_CFN.FrameHeight = 38
                Default_CFN.PowerHeight = 10
                Default_CFN.CastBarHeight = 4
                
            elseif name == "oUF_Lunar_X_Target" then
                Default_CFN.FrameScale = X_Target_Scale*100
                Default_CFN.FrameWidth = 72
                Default_CFN.FrameHeight = 26
                Default_CFN.DefaultSpacerY = 26
                Default_CFN.PowerHeight = 0
                Default_CFN.CastBarHeight = 0
                Default_CFN.HealthColor1 = {0.8,0.8,0.8,1} -- ColorGradient
                Default_CFN.HealthColor2 = {0.8,0.8,0.8,1} -- ColorGradient
                Default_CFN.HealthColor3 = {0.8,0.8,0.8,1} -- ColorGradient
                Default_CFN.HealthColor4 = {0.8,0.8,0.8,1} -- ColorGradient
                Default_CFN.HealthColor5 = {0.8,0.8,0.8,1} -- ColorGradient
                
            elseif name == "oUF_Lunar_Focus" then
                Default_CFN.FrameScale = Focus_Scale*100
                Default_CFN.FrameWidth = 120
                
            elseif name == "oUF_Lunar_Player" then
                Default_CFN.RangeAlpha = 100
                Default_CFN.MouseOverAuras = false
                Default_CFN.DefaultSpacerY = DefaultSpacerY
                Default_CFN.CooldownSpiral = true
                Default_CFN.ClassPortrait = false
                Default_CFN.ModelPortrait = true
                Default_CFN.FrameHeight = 38
                Default_CFN.PowerHeight = 10
                Default_CFN.CastBarHeight = 4
            elseif name == "oUF_Lunar_Party" then
                Default_CFN.DefaultSpacerY = 10
                Default_CFN.PowerHeight = 5
            elseif name == "oUF_Lunar_Arena" then
                Default_CFN.TotalText = true
                Default_CFN.FrameHeight = Arena_Height
            end
        end
    end
end

-- Assign Aura Priorities
do
    Lunar_Default_Options["SelfCastPriority"] = {
        ["oUF_Lunar_Player"] = {
            ["Master's_Call"] = true,
            ["Deterrence"] = true,
            ["Arcane_Power"] = true,
            ["Aspect_of_the_Dragonhawk"] = true,
            ["Wild_Growth"] = true,
            ["Hopped_Up"] = true,
            ["Fade"] = true,
            ["Lock_and_Load"] = true,
            ["Dark_Iron_Pipeweed"] = true,
            ["Earth_Shield"] = true,
            ["Aspect_of_the_Pack"] = true,
            ["Divine_Hymn"] = true,
            ["Missile_Barrage"] = true,
            ["Rapid_Fire"] = true,
            ["Hand_of_Salvation"] = true,
            ["Lightning_Speed"] = true,
            ["The_Beast_Within"] = true,
            ["Presence_of_Mind"] = true,
            ["Sacred_Shield"] = true,
            ["Healing_Way"] = true,
            ["Gift_of_the_Naaru"] = true,
            ["Abolish_Poison"] = true,
            ["Regrowth"] = true,
            ["Savage_Roar"] = true,
            ["Wing_Clip"] = true,
            ["Lifeblood"] = true,
            ["Prayer_of_Mending"] = true,
            ["Aspect_of_the_Viper"] = true,
            ["Renew"] = true,
            ["Tiger's_Fury"] = true,
            ["Restoration"] = true,
            ["Living_Seed"] = true,
            ["Reflection_of_Torment"] = true,
            ["Hand_of_Sacrifice"] = true,
            ["Healing_Trance"] = true,
            ["Berserk"] = true,
            ["Earthliving"] = true,
            ["Argent_Heroism"] = true,
            ["Barkskin"] = true,
            ["Deathchill"] = true,
            ["Lifebloom"] = true,
            ["Inner_Focus"] = true,
            ["Power_Word:_Shield"] = true,
            ["Rejuvenation"] = true,
            ["Master_Tactician"] = true,
            ["Invisibility"] = true,
        },
        ["oUF_Lunar_Party"] = {
            ["Renew"] = true,
            ["Wing_Clip"] = true,
            ["Regrowth"] = true,
            ["Gift_of_the_Naaru"] = true,
            ["Prayer_of_Mending"] = true,
            ["Lifebloom"] = true,
            ["Wild_Growth"] = true,
            ["Hand_of_Sacrifice"] = true,
            ["Restoration"] = true,
            ["Aspect_of_the_Pack"] = true,
            ["Divine_Hymn"] = true,
            ["Healing_Way"] = true,
            ["Earthliving"] = true,
            ["Hand_of_Salvation"] = true,
            ["Power_Word:_Shield"] = true,
            ["Abolish_Poison"] = true,
            ["Earth_Shield"] = true,
            ["Rejuvenation"] = true,
            ["Living_Seed"] = true,
            ["Sacred_Shield"] = true,
        },
        ["oUF_Lunar_Target"] = {
            ["Spell_Lock"] = true,
            ["Pounce_Bleed"] = true,
            ["Death_Grip"] = true,
            ["Feral_Charge_Effect"] = true,
            ["Touch_the_Nightmare"] = true,
            ["End_of_Round"] = true,
            ["Corrosive_Vial"] = true,
            ["Web_Shot"] = true,
            ["Acid_Volley"] = true,
            ["Hibernate"] = true,
            ["Widow's_Embrace"] = true,
            ["Blast_Wave"] = true,
            ["Serpent_Sting"] = true,
            ["Pounce"] = true,
            ["Insect_Swarm"] = true,
            ["Bonk"] = true,
            ["Fireball"] = true,
            ["Necrotic_Poison"] = true,
            ["Earth_Shield"] = true,
            ["Aftermath"] = true,
            ["Psychic_Scream"] = true,
            ["Faerie_Fire_(Feral)"] = true,
            ["Living_Bomb"] = true,
            ["Curse_of_Weakness"] = true,
            ["Bested_the_Undercity"] = true,
            ["Heart_of_the_Crusader"] = true,
            ["Restoration"] = true,
            ["Deathbloom"] = true,
            ["Curse_of_Doom"] = true,
            ["Screams_of_the_Dead"] = true,
            ["Volatile_Infection"] = true,
            ["Challenging_Shout"] = true,
            ["Piercing_Howl"] = true,
            ["Power_Word:_Shield"] = true,
            ["Mutating_Injection"] = true,
            ["Leeching_Poison"] = true,
            ["Froststorm_Breath"] = true,
            ["Acid_Splash"] = true,
            ["Death_Plague"] = true,
            ["Arcing_Burn"] = true,
            ["Mark_of_Darkness"] = true,
            ["Tail_Lash"] = true,
            ["Poison"] = true,
            ["Frostfire_Bolt"] = true,
            ["Mind_Sear"] = true,
            ["Poisoned_Spear"] = true,
            ["Hooked_Net"] = true,
            ["Bested_Ironforge"] = true,
            ["Aspect_of_the_Pack"] = true,
            ["Infected_Wound"] = true,
            ["Hand_of_Salvation"] = true,
            ["Engine_on_Fire"] = true,
            ["Mind-numbing_Poison"] = true,
            ["Winter's_Chill"] = true,
            ["Cone_of_Cold"] = true,
            ["Unrelenting_Assault"] = true,
            ["Necrotic_Aura"] = true,
            ["Deathfrost"] = true,
            ["Lacerate"] = true,
            ["Blood_Corruption"] = true,
            ["Silenced_-_Improved_Counterspell"] = true,
            ["Prayer_of_Mending"] = true,
            ["Electro_Shock"] = true,
            ["Hamstring"] = true,
            ["Agonizing_Strike"] = true,
            ["Demoralizing_Shout"] = true,
            ["Bested_Thunder_Bluff"] = true,
            ["Arcane_Stream"] = true,
            ["Unstable_Affliction"] = true,
            ["Poison_Spit"] = true,
            ["Charge"] = true,
            ["Hydra_Sputum"] = true,
            ["Blizzard"] = true,
            ["Barbed_Net"] = true,
            ["Detonate_Mana"] = true,
            ["Icebolt"] = true,
            ["Infected_Wounds"] = true,
            ["Freeze"] = true,
            ["Thundering_Stomp"] = true,
            ["Grievous_Wound"] = true,
            ["Forbearance"] = true,
            ["Shield_Slam"] = true,
            ["Bested_Darnassus"] = true,
            ["Immolation_Strike"] = true,
            ["War_Stomp"] = true,
            ["Crush_Armor"] = true,
            ["Disarm"] = true,
            ["Stampede"] = true,
            ["Frost_Fever"] = true,
            ["Chains_of_Ice"] = true,
            ["Frost_Breath"] = true,
            ["Deadly_Throw"] = true,
            ["Impale"] = true,
            ["Deadly_Poison_VIII"] = true,
            ["Recently_Dropped_Flag"] = true,
            ["Poison_Stinger"] = true,
            ["Bested_Orgrimmar"] = true,
            ["Rake"] = true,
            ["Rip"] = true,
            ["Crazed"] = true,
            ["Serinar's_Vision"] = true,
            ["Sated"] = true,
            ["Focus_Beam"] = true,
            ["Pulsing_Shockwave_Aura"] = true,
            ["Shadow_Embrace"] = true,
            ["Healing_Way"] = true,
            ["Deadly_Poison_VI"] = true,
            ["Demon_Charge"] = true,
            ["Fiery_Payback"] = true,
            ["Sleep"] = true,
            ["Curse_of_Exhaustion"] = true,
            ["Renew"] = true,
            ["Putrid_Bite"] = true,
            ["Mangle_(Bear)"] = true,
            ["Vicious_Roar"] = true,
            ["Uncontrollable_Energy"] = true,
            ["Skewer"] = true,
            ["Jagged_Knife"] = true,
            ["Concussion_Blow"] = true,
            ["Righteous_Vengeance"] = true,
            ["Wither"] = true,
            ["Silence"] = true,
            ["Lock_and_Load"] = true,
            ["Sunder_Armor"] = true,
            ["Hand_of_Reckoning"] = true,
            ["Snatch"] = true,
            ["Twilight_Shift"] = true,
            ["Seed_of_Corruption"] = true,
            ["Poultryized!"] = true,
            ["Typhoon"] = true,
            ["Resurrection_Sickness"] = true,
            ["Chill"] = true,
            ["Seduction"] = true,
            ["Silenced_-_Gag_Order"] = true,
            ["Rocket_Launch"] = true,
            ["Drain_Power"] = true,
            ["Gift_of_Arthas"] = true,
            ["Arcane_Blast"] = true,
            ["Blinding_Swarm"] = true,
            ["Frost_Vulnerability"] = true,
            ["Powerful_Bite"] = true,
            ["Unholy_Blight"] = true,
            ["Shadowfury"] = true,
            ["Glyph_of_Shadowflame"] = true,
            ["Focused"] = true,
            ["Time_Bomb"] = true,
            ["Trespasser!"] = true,
            ["Lifeblood"] = true,
            ["Warped_Armor"] = true,
            ["Veil_of_Shadow"] = true,
            ["Bested_Silvermoon_City"] = true,
            ["Revenge_Stun"] = true,
            ["Flamestrike"] = true,
            ["Restricted_Flight_Area"] = true,
            ["Frostbrand_Attack"] = true,
            ["Stoneclaw_Stun"] = true,
            ["Exhaustion"] = true,
            ["Misery"] = true,
            ["Slime_Burst"] = true,
            ["Mark_of_Rivendare"] = true,
            ["Bash"] = true,
            ["Hemorrhage"] = true,
            ["Hammer_of_Injustice"] = true,
            ["Lifebloom"] = true,
            ["Thunderclap"] = true,
            ["Poison_Breath"] = true,
            ["Glyph_of_Rake"] = true,
            ["Withered_Touch"] = true,
            ["Leech_Poison"] = true,
            ["Deadly_Poison_V"] = true,
            ["Mark_of_Zeliek"] = true,
            ["Mark_of_Korth'azz"] = true,
            ["Mace_Smash"] = true,
            ["Weakened_Resolve"] = true,
            ["Puncture_Wound"] = true,
            ["Silenced_-_Shield_of_the_Templar"] = true,
            ["Shadow_Mastery"] = true,
            ["Immolate"] = true,
            ["Acidic_Sludge"] = true,
            ["Retching_Plague"] = true,
            ["Mind_Flay"] = true,
            ["Gushing_Wound"] = true,
            ["Terrifying_Roar"] = true,
            ["Savage_Combat"] = true,
            ["Weakened_Soul"] = true,
            ["Withering_Strike"] = true,
            ["Waylay"] = true,
            ["Welding_Beam"] = true,
            ["Crippling_Poison"] = true,
            ["Glyph_of_Hamstring"] = true,
            ["Counterattack"] = true,
            ["Righteous_Defense"] = true,
            ["Furious_Attacks"] = true,
            ["Frost_Blast"] = true,
            ["Withering_Roar"] = true,
            ["Psychic_Horror"] = true,
            ["Shadowburn"] = true,
            ["Flame_Tsunami"] = true,
            ["Curse_of_Agony"] = true,
            ["Carnivorous_Bite"] = true,
            ["Taunt"] = true,
            ["Shred"] = true,
            ["Virulent_Poison"] = true,
            ["Stormstrike"] = true,
            ["Holy_Wrath"] = true,
            ["Head_Crack"] = true,
            ["Desecration"] = true,
            ["Expose_Armor"] = true,
            ["Twilight_Torment"] = true,
            ["Acid_Cloud"] = true,
            ["Diminish_Soul"] = true,
            ["Dazed"] = true,
            ["Curse_of_the_Plaguebringer"] = true,
            ["Stormbolt"] = true,
            ["Consecration"] = true,
            ["Slow"] = true,
            ["Plague_Splash"] = true,
            ["Blade_Twisting"] = true,
            ["Frost_Aura"] = true,
            ["Venom_Web_Spray"] = true,
            ["Decrepit_Fever"] = true,
            ["Rejuvenation"] = true,
            ["Maim"] = true,
            ["Fungal_Creep"] = true,
            ["Demoralizing_Roar"] = true,
            ["Silencing_Shot"] = true,
            ["Shrink"] = true,
            ["Earth_and_Moon"] = true,
            ["Devouring_Plague"] = true,
            ["Hex"] = true,
            ["Spell_Disruption"] = true,
            ["Recently_Bandaged"] = true,
            ["Spider_Web"] = true,
            ["Strangulate"] = true,
            ["Frost_Shock"] = true,
            ["Wing_Clip"] = true,
            ["Curse_of_Mending"] = true,
            ["Bested_the_Exodar"] = true,
            ["Gift_of_the_Naaru"] = true,
            ["Frostbite"] = true,
            ["Vile_Vomit"] = true,
            ["Inevitable_Doom"] = true,
            ["Charge_Stun"] = true,
            ["Rupture"] = true,
            ["Paralyze"] = true,
            ["Haunt"] = true,
            ["Arc_Weld"] = true,
            ["Hurl_Weapon"] = true,
            ["Ticking_Bomb"] = true,
            ["Dark_Command"] = true,
            ["Stomp"] = true,
            ["Divine_Hymn"] = true,
            ["Savage_Rend"] = true,
            ["Demoralizing_Screech"] = true,
            ["Chicken_Net"] = true,
            ["Mind_Control"] = true,
            ["Scare_Beast"] = true,
            ["Vindication"] = true,
            ["Beast's_Mark"] = true,
            ["Shattered_Barrier"] = true,
            ["Terrify"] = true,
            ["Crushing_Webs"] = true,
            ["Grave_Strike"] = true,
            ["Faerie_Fire"] = true,
            ["Explosive_Shot"] = true,
            ["Abolish_Poison"] = true,
            ["Challenging_Roar"] = true,
            ["Water_Tomb"] = true,
            ["Frostbolt"] = true,
            ["Ritual_Strike"] = true,
            ["Flame_Shock"] = true,
            ["Bile_Vomit"] = true,
            ["Avenger's_Shield"] = true,
            ["Mark_of_Blaumeux"] = true,
            ["Kidney_Shot"] = true,
            ["Flesh_Rot"] = true,
            ["Sonic_Blast"] = true,
            ["Pound"] = true,
            ["Net"] = true,
            ["Dart"] = true,
            ["Turn_Evil"] = true,
            ["Fetid_Rot"] = true,
            ["Stop_Time"] = true,
            ["Gouge"] = true,
            ["Earthliving"] = true,
            ["Ebon_Plague"] = true,
            ["Hammer_Drop"] = true,
            ["Open_Wound"] = true,
            ["Melt_Armor"] = true,
            ["Blinding_Webs"] = true,
            ["Unholy_Shadow"] = true,
            ["Blind"] = true,
            ["Infected_Bite"] = true,
            ["Deadly_Poison_VII"] = true,
            ["Hunter's_Mark"] = true,
            ["Corruption"] = true,
            ["Howling_Screech"] = true,
            ["Rune_Punch"] = true,
            ["Shadowflame"] = true,
            ["Icy_Clutch"] = true,
            ["Shadow_Word:_Pain"] = true,
            ["Riposte"] = true,
            ["Drain_Soul"] = true,
            ["Earth's_Fury"] = true,
            ["Poison_Tipped_Spear"] = true,
            ["Poison_Spray"] = true,
            ["Harvest_Soul"] = true,
            ["Mind_Soothe"] = true,
            ["Bested_Sen'jin"] = true,
            ["Improved_Fire_Nova_Totem"] = true,
            ["Blood_Frenzy"] = true,
            ["Toxic_Spit"] = true,
            ["Deadly_Poison_IX"] = true,
            ["Judgement_of_Wisdom"] = true,
            ["Teleport"] = true,
            ["Immobilizing_Field"] = true,
            ["Curse_of_the_Elements"] = true,
            ["Frost_Buffet"] = true,
            ["Judgement_of_Light"] = true,
            ["Ice_Beam"] = true,
            ["Envenomed_Shot"] = true,
            ["Improved_Scorch"] = true,
            ["Cheap_Shot"] = true,
            ["Animal_Blood"] = true,
            ["Judgement_of_Justice"] = true,
            ["Frostweave_Net"] = true,
            ["Idle"] = true,
            ["Stampy's_Stompy-Stomp"] = true,
            ["Shadow_Mark"] = true,
            ["Sacred_Shield"] = true,
            ["Regrowth"] = true,
            ["Polymorph"] = true,
            ["Runic_Focus"] = true,
            ["Frostbolt_Volley"] = true,
            ["Repentance"] = true,
            ["Black_Arrow"] = true,
            ["Poison_Bolt"] = true,
            ["Moonfire"] = true,
            ["Pyroblast"] = true,
            ["Vampiric_Touch"] = true,
            ["Web_Wrap"] = true,
            ["Intimidating_Shout"] = true,
            ["Bested_Stormwind"] = true,
            ["Suffering"] = true,
            ["Silenced_-_Improved_Kick"] = true,
            ["Unbalancing_Strike"] = true,
            ["Crush"] = true,
            ["Death_and_Decay"] = true,
            ["Immolation_Trap_Effect"] = true,
            ["Curse_of_Tongues"] = true,
            ["Growl"] = true,
            ["Inactive"] = true,
            ["Life_Drain"] = true,
            ["Drain_Life"] = true,
            ["Soothing_Kiss"] = true,
            ["Mangle_(Cat)"] = true,
            ["Viper_Sting"] = true,
            ["Hypothermia"] = true,
            ["Curse_of_Fatigue"] = true,
            ["Thunder_Clap"] = true,
            ["Azure_Bindings"] = true,
            ["Conversion_Beam"] = true,
            ["Stun"] = true,
            ["Magma"] = true,
            ["Earthbind"] = true,
            ["Concussive_Shot"] = true,
            ["Entangling_Roots"] = true,
            ["Wild_Growth"] = true,
            ["Shackle_Undead"] = true,
            ["Improved_Hamstring"] = true,
            ["Hand_of_Sacrifice"] = true,
            ["Fade_Armor"] = true,
            ["Static_Overload"] = true,
            ["Glyph_of_Heart_Strike"] = true,
            ["Living_Seed"] = true,
            ["Pierce_Armor"] = true,
            ["Shockwave"] = true,
            ["Dull_Weapons"] = true,
            ["Dragon's_Breath"] = true,
            ["Gnaw"] = true,
            ["Mind_Vision"] = true,
            ["Chilled"] = true,
            ["Temporal_Rift"] = true,
            ["Poison_Charge"] = true,
            ["Poison_Bolt_Volley"] = true,
            ["Transporter_Malfunction"] = true,
            ["Positive_Charge"] = true,
            ["Sandfern_Disguise"] = true,
            ["Gore"] = true,
            ["Web"] = true,
            ["Nightmare"] = true,
            ["Trauma"] = true,
            ["Ghost"] = true,
            ["Frost_Nova"] = true,
            ["Rend"] = true,
            ["Cripple"] = true,
            ["Frightening_Shout"] = true,
            ["Dismantle"] = true,
            ["Banish"] = true,
            ["Hammer_Blow"] = true,
            ["Arcane_Torrent"] = true,
            ["Icy_Touch"] = true,
            ["Crypt_Fever"] = true,
            ["Frostbomb"] = true,
            ["Deep_Freeze"] = true,
            ["Holy_Vengeance"] = true,
            ["Web_Spray"] = true,
            ["Blood_Plague"] = true,
            ["Bested_Gnomeregan"] = true,
            ["Deep_Wounds"] = true,
            ["Earthgrab"] = true,
            ["Intercept"] = true,
            ["Shattering_Throw"] = true,
            ["Inferno_Effect"] = true,
            ["Holy_Fire"] = true,
            ["Ignite"] = true,
            ["Vampiric_Embrace"] = true,
            ["Garrote"] = true,
            ["Hammer_of_Justice"] = true,
            ["Concussive_Barrage"] = true,
        },
        ["oUF_Lunar_Raid"] = {
            ["Renew"] = true,
            ["Wing_Clip"] = true,
            ["Regrowth"] = true,
            ["Gift_of_the_Naaru"] = true,
            ["Prayer_of_Mending"] = true,
            ["Lifebloom"] = true,
            ["Wild_Growth"] = true,
            ["Hand_of_Sacrifice"] = true,
            ["Restoration"] = true,
            ["Living_Seed"] = true,
            ["Divine_Hymn"] = true,
            ["Healing_Way"] = true,
            ["Earthliving"] = true,
            ["Hand_of_Salvation"] = true,
            ["Power_Word:_Shield"] = true,
            ["Abolish_Poison"] = true,
            ["Earth_Shield"] = true,
            ["Rejuvenation"] = true,
            ["Aspect_of_the_Pack"] = true,
            ["Sacred_Shield"] = true,
        },
        ["oUF_Lunar_Focus"] = {
            ["Renew"] = true,
            ["Wing_Clip"] = true,
            ["Gift_of_the_Naaru"] = true,
            ["Frostfire_Bolt"] = true,
            ["Polymorph"] = true,
            ["Wild_Growth"] = true,
            ["Hand_of_Sacrifice"] = true,
            ["Aspect_of_the_Pack"] = true,
            ["Divine_Hymn"] = true,
            ["Serpent_Sting"] = true,
            ["Hand_of_Salvation"] = true,
            ["Frostbolt"] = true,
            ["Abolish_Poison"] = true,
            ["Regrowth"] = true,
            ["Psychic_Horror"] = true,
            ["Sacred_Shield"] = true,
            ["Prayer_of_Mending"] = true,
            ["Lifebloom"] = true,
            ["Lock_and_Load"] = true,
            ["Healing_Way"] = true,
            ["Restoration"] = true,
            ["Explosive_Shot"] = true,
            ["Hamstring"] = true,
            ["Earth_Shield"] = true,
            ["Earthliving"] = true,
            ["Blood_Plague"] = true,
            ["Power_Word:_Shield"] = true,
            ["Silencing_Shot"] = true,
            ["Viper_Sting"] = true,
            ["Living_Seed"] = true,
            ["Black_Arrow"] = true,
            ["Rejuvenation"] = true,
            ["Shattering_Throw"] = true,
            ["Hunter's_Mark"] = true,
        },
        ["oUF_Lunar_Arena"] = {
            ["Aspect_of_the_Pack"] = true,
            ["Glyph_of_Hamstring"] = true,
            ["Silenced_-_Gag_Order"] = true,
            ["Rend"] = true,
            ["Chains_of_Ice"] = true,
            ["Black_Arrow"] = true,
            ["Entangling_Roots"] = true,
            ["Explosive_Shot"] = true,
            ["Improved_Hamstring"] = true,
            ["Blood_Plague"] = true,
            ["Sunder_Armor"] = true,
        },
        ["oUF_Lunar_X_Target"] = {
        },
        ["oUF_Lunar_ArenaPet"] = {
            ["Glyph_of_Hamstring"] = true,
            ["Silenced_-_Gag_Order"] = true,
            ["Rend"] = true,
            ["Chains_of_Ice"] = true,
            ["Black_Arrow"] = true,
            ["Improved_Hamstring"] = true,
            ["Explosive_Shot"] = true,
            ["Blood_Plague"] = true,
            ["Sunder_Armor"] = true,
        },
        ["oUF_Lunar_PartyPet"] = {
            ["Renew"] = true,
            ["Wing_Clip"] = true,
            ["Regrowth"] = true,
            ["Gift_of_the_Naaru"] = true,
            ["Prayer_of_Mending"] = true,
            ["Lifebloom"] = true,
            ["Wild_Growth"] = true,
            ["Hand_of_Sacrifice"] = true,
            ["Restoration"] = true,
            ["Living_Seed"] = true,
            ["Divine_Hymn"] = true,
            ["Healing_Way"] = true,
            ["Earthliving"] = true,
            ["Hand_of_Salvation"] = true,
            ["Power_Word:_Shield"] = true,
            ["Abolish_Poison"] = true,
            ["Earth_Shield"] = true,
            ["Rejuvenation"] = true,
            ["Aspect_of_the_Pack"] = true,
            ["Sacred_Shield"] = true,
        },
        ["oUF_Lunar_PlayerPet"] = {
            ["Renew"] = true,
            ["Abolish_Poison"] = true,
            ["Regrowth"] = true,
            ["Gift_of_the_Naaru"] = true,
            ["Prayer_of_Mending"] = true,
            ["Lifebloom"] = true,
            ["Wild_Growth"] = true,
            ["Hand_of_Sacrifice"] = true,
            ["Restoration"] = true,
            ["Living_Seed"] = true,
            ["Divine_Hymn"] = true,
            ["Healing_Way"] = true,
            ["Earthliving"] = true,
            ["Hand_of_Salvation"] = true,
            ["Power_Word:_Shield"] = true,
            ["Wing_Clip"] = true,
            ["Rejuvenation"] = true,
            ["Earth_Shield"] = true,
            ["Sacred_Shield"] = true,
        },
    }
            
    Lunar_Default_Options["PriorityAuras"] = {
        ["oUF_Lunar_Player"] = {
            ["Mortal_Wound"] = true,
            ["Mace_Smash"] = true,
            ["Pounce_Bleed"] = true,
            ["Blood_Fury"] = true,
            ["Howl_of_Terror"] = true,
            ["Roar_of_Recovery"] = true,
            ["Anti-Magic_Shell"] = true,
            ["Terrifying_Roar"] = true,
            ["Survival_Instincts"] = true,
            ["Ravage"] = true,
            ["Blind"] = true,
            ["Pounce"] = true,
            ["Adrenaline_Rush"] = true,
            ["Impact"] = true,
            ["Power_Infusion"] = true,
            ["Berserker_Rage"] = true,
            ["Cyclone"] = true,
            ["Aimed_Shot"] = true,
            ["Holy_Wrath"] = true,
            ["Divine_Illumination"] = true,
            ["Shield_Wall"] = true,
            ["Psychic_Scream"] = true,
            ["Dancing_Rune_Weapon"] = true,
            ["Tendon_Rip"] = true,
            ["Hex"] = true,
            ["Screams_of_the_Dead"] = true,
            ["Starfall"] = true,
            ["Maim"] = true,
            ["Berserk"] = true,
            ["Intimidation"] = true,
            ["Silencing_Shot"] = true,
            ["Silverwing_Flag"] = true,
            ["Shockwave"] = true,
            ["Second_Wind"] = true,
            ["Will_of_the_Forsaken"] = true,
            ["Freezing_Trap_Effect"] = true,
            ["Mortal_Strike"] = true,
            ["Icebound_Fortitude"] = true,
            ["Charge_Stun"] = true,
            ["Stun"] = true,
            ["Cloak_of_Shadows"] = true,
            ["Netherstorm_Flag"] = true,
            ["Drink"] = true,
            ["Clearcasting"] = true,
            ["Wound_Poison_VI"] = true,
            ["Tidal_Force"] = true,
            ["Scare_Beast"] = true,
            ["Divine_Sacrifice"] = true,
            ["Bladestorm"] = true,
            ["Shadow_Dance"] = true,
            ["Wyvern_Sting"] = true,
            ["Sprint"] = true,
            ["Garrote_-_Silence"] = true,
            ["Wound_Poison_IV"] = true,
            ["Killing_Spree"] = true,
            ["Stoneclaw_Stun"] = true,
            ["Metamorphosis"] = true,
            ["Kidney_Shot"] = true,
            ["Sonic_Blast"] = true,
            ["Unbreakable_Armor"] = true,
            ["Lichborne"] = true,
            ["Blood_of_the_Fallen_Cultist"] = true,
            ["Fear_Ward"] = true,
            ["Blade_Flurry"] = true,
            ["Demon_Charge"] = true,
            ["Innervate"] = true,
            ["Death_Coil"] = true,
            ["Repentance"] = true,
            ["Evasion"] = true,
            ["Polymorph"] = true,
            ["Pain_Suppression"] = true,
            ["Icebolt"] = true,
            ["Improved_Fire_Nova_Totem"] = true,
            ["Aspect_of_the_Beast"] = true,
            ["Hysteria"] = true,
            ["Shield_Slam"] = true,
            ["Hungering_Cold"] = true,
            ["Blink"] = true,
            ["Hand_of_Protection"] = true,
            ["Startling_Roar"] = true,
            ["Stampy's_Stompy-Stomp"] = true,
            ["Turn_Evil"] = true,
            ["Disarm"] = true,
            ["Misdirection"] = true,
            ["Impale"] = true,
            ["Web_Wrap"] = true,
            ["Intimidating_Shout"] = true,
            ["Spell_Reflection"] = true,
            ["Barkskin"] = true,
            ["Entangling_Roots"] = true,
            ["Mark_of_Blood"] = true,
            ["Sap"] = true,
            ["Icy_Veins"] = true,
            ["Judgement_of_Justice"] = true,
            ["Shield_Block"] = true,
            ["Bloodrage"] = true,
            ["Viper_Sting"] = true,
            ["Indestructible"] = true,
            ["Demonic_Empowerment"] = true,
            ["Astral_Shift"] = true,
            ["Soul_Strike"] = true,
            ["Gnaw"] = true,
            ["Warsong_Flag"] = true,
            ["Aura_Mastery"] = true,
            ["Wound_Poison_VII"] = true,
            ["Cheap_Shot"] = true,
            ["Frenzied_Regeneration"] = true,
            ["Shamanistic_Rage"] = true,
            ["Mirror_Image"] = true,
            ["Guardian_Spirit"] = true,
            ["Terrify"] = true,
            ["Fear"] = true,
            ["Strangulate"] = true,
            ["Wound_Poison"] = true,
            ["Hammer_of_Injustice"] = true,
            ["Tail_Lash"] = true,
            ["Seduction"] = true,
            ["Divine_Plea"] = true,
            ["Prowl"] = true,
            ["Sacrifice"] = true,
            ["Wound_Poison_V"] = true,
            ["Army_of_the_Dead"] = true,
            ["Recklessness"] = true,
            ["Concussion_Blow"] = true,
            ["Seal_of_Command"] = true,
            ["Summoning_Disorientation"] = true,
            ["Shadowfury"] = true,
            ["Avenging_Wrath"] = true,
            ["Tranquility"] = true,
            ["Bone_Shield"] = true,
            ["Vampiric_Blood"] = true,
            ["Call_of_the_Wild"] = true,
            ["Frightening_Shout"] = true,
            ["War_Stomp"] = true,
            ["Scatter_Shot"] = true,
            ["Divine_Shield"] = true,
            ["Freezing_Arrow_Effect"] = true,
            ["Divine_Protection"] = true,
            ["The_Beast_Within"] = true,
            ["Dispersion"] = true,
            ["Nature's_Grasp"] = true,
            ["Ice_Block"] = true,
            ["Web_Spray"] = true,
            ["Hammer_of_Justice"] = true,
            ["Fury_of_the_Fallen_Cultist"] = true,
            ["Nightmare"] = true,
            ["Revenge_Stun"] = true,
            ["Bloodlust"] = true,
            ["Intercept"] = true,
            ["Inferno_Effect"] = true,
            ["Deep_Freeze"] = true,
            ["Retaliation"] = true,
            ["Dash"] = true,
            ["Evocation"] = true,
            ["Bash"] = true,
            ["Nature's_Swiftness"] = true,
        },
        ["oUF_Lunar_ArenaPet"] = {
            ["Mortal_Wound"] = true,
            ["Mace_Smash"] = true,
            ["Pounce_Bleed"] = true,
            ["Howl_of_Terror"] = true,
            ["Soul_Strike"] = true,
            ["Mind_Flay"] = true,
            ["Terrifying_Roar"] = true,
            ["Ravage"] = true,
            ["Widow's_Embrace"] = true,
            ["Blast_Wave"] = true,
            ["Pounce"] = true,
            ["Cyclone"] = true,
            ["Aimed_Shot"] = true,
            ["Holy_Wrath"] = true,
            ["Aftermath"] = true,
            ["Psychic_Scream"] = true,
            ["Waylay"] = true,
            ["Slow"] = true,
            ["Fear"] = true,
            ["Hex"] = true,
            ["Screams_of_the_Dead"] = true,
            ["Challenging_Shout"] = true,
            ["Maim"] = true,
            ["Chilled"] = true,
            ["Froststorm_Breath"] = true,
            ["Spider_Web"] = true,
            ["Shockwave"] = true,
            ["Tail_Lash"] = true,
            ["Wing_Clip"] = true,
            ["Freezing_Trap_Effect"] = true,
            ["Mortal_Strike"] = true,
            ["Frostfire_Bolt"] = true,
            ["Vile_Vomit"] = true,
            ["Charge_Stun"] = true,
            ["Rupture"] = true,
            ["Stun"] = true,
            ["Savage_Rend"] = true,
            ["Wound_Poison_VI"] = true,
            ["Scare_Beast"] = true,
            ["Gnaw"] = true,
            ["Cone_of_Cold"] = true,
            ["Wound_Poison_IV"] = true,
            ["Water_Tomb"] = true,
            ["Psychic_Horror"] = true,
            ["Stoneclaw_Stun"] = true,
            ["Avenger's_Shield"] = true,
            ["Kidney_Shot"] = true,
            ["Sonic_Blast"] = true,
            ["Challenging_Roar"] = true,
            ["Hamstring"] = true,
            ["Stop_Time"] = true,
            ["Demon_Charge"] = true,
            ["Death_Coil"] = true,
            ["Repentance"] = true,
            ["Frostbolt_Volley"] = true,
            ["Hydra_Sputum"] = true,
            ["Riposte"] = true,
            ["Polymorph"] = true,
            ["Icebolt"] = true,
            ["Infected_Wounds"] = true,
            ["Improved_Fire_Nova_Totem"] = true,
            ["Curse_of_Fatigue"] = true,
            ["Shield_Slam"] = true,
            ["Envenomed_Shot"] = true,
            ["Hungering_Cold"] = true,
            ["Ice_Beam"] = true,
            ["War_Stomp"] = true,
            ["Entangling_Roots"] = true,
            ["Blind"] = true,
            ["Mark_of_Blood"] = true,
            ["Turn_Evil"] = true,
            ["Disarm"] = true,
            ["Stampede"] = true,
            ["Shield_Wall"] = true,
            ["Blade_Twisting"] = true,
            ["Stampy's_Stompy-Stomp"] = true,
            ["Deadly_Throw"] = true,
            ["Impale"] = true,
            ["Web_Wrap"] = true,
            ["Intimidating_Shout"] = true,
            ["Frostbomb"] = true,
            ["Rip"] = true,
            ["Crazed"] = true,
            ["Spell_Reflection"] = true,
            ["Sap"] = true,
            ["Frostbolt"] = true,
            ["Shield_Block"] = true,
            ["Fiery_Payback"] = true,
            ["Curse_of_Exhaustion"] = true,
            ["Crippling_Poison"] = true,
            ["Icy_Clutch"] = true,
            ["Frost_Blast"] = true,
            ["Web_Shot"] = true,
            ["Judgement_of_Justice"] = true,
            ["Earthbind"] = true,
            ["Venom_Web_Spray"] = true,
            ["Concussion_Blow"] = true,
            ["Shackle_Undead"] = true,
            ["Cheap_Shot"] = true,
            ["Desecration"] = true,
            ["Expose_Armor"] = true,
            ["Tendon_Rip"] = true,
            ["Curse_of_Doom"] = true,
            ["Slime_Burst"] = true,
            ["Wound_Poison"] = true,
            ["Curse_of_Agony"] = true,
            ["Piercing_Howl"] = true,
            ["Seduction"] = true,
            ["Silencing_Shot"] = true,
            ["Thunderclap"] = true,
            ["Wyvern_Sting"] = true,
            ["Wound_Poison_V"] = true,
            ["Strangulate"] = true,
            ["Cripple"] = true,
            ["Concussive_Shot"] = true,
            ["Typhoon"] = true,
            ["Hammer_of_Injustice"] = true,
            ["Shadowfury"] = true,
            ["Gushing_Wound"] = true,
            ["Howling_Screech"] = true,
            ["Unstable_Affliction"] = true,
            ["Earth's_Fury"] = true,
            ["Frightening_Shout"] = true,
            ["Dismantle"] = true,
            ["Hammer_of_Justice"] = true,
            ["Scatter_Shot"] = true,
            ["Hammer_Blow"] = true,
            ["Freezing_Arrow_Effect"] = true,
            ["Revenge_Stun"] = true,
            ["Frostbrand_Attack"] = true,
            ["Frost_Shock"] = true,
            ["Arcane_Torrent"] = true,
            ["Web_Spray"] = true,
            ["Suffering"] = true,
            ["Deep_Freeze"] = true,
            ["Snatch"] = true,
            ["Nightmare"] = true,
            ["Intercept"] = true,
            ["Shattering_Throw"] = true,
            ["Inferno_Effect"] = true,
            ["Terrify"] = true,
            ["Dazed"] = true,
            ["Withered_Touch"] = true,
            ["Wound_Poison_VII"] = true,
            ["Bash"] = true,
            ["Concussive_Barrage"] = true,
        },
        ["oUF_Lunar_Arena"] = {
            ["Mortal_Wound"] = true,
            ["Pounce_Bleed"] = true,
            ["Roar_of_Recovery"] = true,
            ["Soul_Strike"] = true,
            ["Web_Shot"] = true,
            ["Widow's_Embrace"] = true,
            ["Blast_Wave"] = true,
            ["Adrenaline_Rush"] = true,
            ["Cyclone"] = true,
            ["Divine_Illumination"] = true,
            ["Shield_Wall"] = true,
            ["Psychic_Scream"] = true,
            ["Dancing_Rune_Weapon"] = true,
            ["Hex"] = true,
            ["Screams_of_the_Dead"] = true,
            ["Challenging_Shout"] = true,
            ["Piercing_Howl"] = true,
            ["Berserk"] = true,
            ["Intimidation"] = true,
            ["Froststorm_Breath"] = true,
            ["Tail_Lash"] = true,
            ["Shamanistic_Rage"] = true,
            ["Icebound_Fortitude"] = true,
            ["Stun"] = true,
            ["Drink"] = true,
            ["Wound_Poison_VI"] = true,
            ["Hand_of_Salvation"] = true,
            ["Gnaw"] = true,
            ["Cone_of_Cold"] = true,
            ["Sprint"] = true,
            ["Wound_Poison_IV"] = true,
            ["Nature's_Swiftness"] = true,
            ["Psychic_Horror"] = true,
            ["Stoneclaw_Stun"] = true,
            ["Metamorphosis"] = true,
            ["Recklessness"] = true,
            ["Hamstring"] = true,
            ["Fear_Ward"] = true,
            ["Deathchill"] = true,
            ["Unstable_Affliction"] = true,
            ["Hydra_Sputum"] = true,
            ["Pain_Suppression"] = true,
            ["Icebolt"] = true,
            ["Infected_Wounds"] = true,
            ["Curse_of_Fatigue"] = true,
            ["Aspect_of_the_Beast"] = true,
            ["Shield_Slam"] = true,
            ["Hungering_Cold"] = true,
            ["Blink"] = true,
            ["War_Stomp"] = true,
            ["Disarm"] = true,
            ["Stampede"] = true,
            ["Deadly_Throw"] = true,
            ["Impale"] = true,
            ["Frostbomb"] = true,
            ["Deep_Freeze"] = true,
            ["Crazed"] = true,
            ["Barkskin"] = true,
            ["Fiery_Payback"] = true,
            ["Master's_Call"] = true,
            ["Curse_of_Exhaustion"] = true,
            ["Indestructible"] = true,
            ["Demonic_Empowerment"] = true,
            ["Arcane_Power"] = true,
            ["Concussion_Blow"] = true,
            ["Aura_Mastery"] = true,
            ["Cheap_Shot"] = true,
            ["Slime_Burst"] = true,
            ["Wound_Poison"] = true,
            ["Seduction"] = true,
            ["Wound_Poison_V"] = true,
            ["Prowl"] = true,
            ["Call_of_the_Wild"] = true,
            ["Freezing_Arrow_Effect"] = true,
            ["Divine_Protection"] = true,
            ["The_Beast_Within"] = true,
            ["Frostbrand_Attack"] = true,
            ["Bestial_Wrath"] = true,
            ["Hammer_of_Injustice"] = true,
            ["Thunderclap"] = true,
            ["Retaliation"] = true,
            ["Dash"] = true,
            ["Wound_Poison_VII"] = true,
            ["Bash"] = true,
            ["Mace_Smash"] = true,
            ["Blood_Fury"] = true,
            ["Howl_of_Terror"] = true,
            ["Anti-Magic_Shell"] = true,
            ["Icy_Clutch"] = true,
            ["Inner_Focus"] = true,
            ["Terrifying_Roar"] = true,
            ["Survival_Instincts"] = true,
            ["Ravage"] = true,
            ["Seal_of_Command"] = true,
            ["Power_Infusion"] = true,
            ["Berserker_Rage"] = true,
            ["Aimed_Shot"] = true,
            ["Holy_Wrath"] = true,
            ["Desecration"] = true,
            ["Expose_Armor"] = true,
            ["Waylay"] = true,
            ["Slow"] = true,
            ["Fear"] = true,
            ["Starfall"] = true,
            ["Maim"] = true,
            ["Chilled"] = true,
            ["Silverwing_Flag"] = true,
            ["Spider_Web"] = true,
            ["Shockwave"] = true,
            ["Second_Wind"] = true,
            ["Wing_Clip"] = true,
            ["Freezing_Trap_Effect"] = true,
            ["Mortal_Strike"] = true,
            ["Blade_Twisting"] = true,
            ["Charge_Stun"] = true,
            ["Rupture"] = true,
            ["Cloak_of_Shadows"] = true,
            ["Netherstorm_Flag"] = true,
            ["Savage_Rend"] = true,
            ["Tidal_Force"] = true,
            ["Scare_Beast"] = true,
            ["Presence_of_Mind"] = true,
            ["Divine_Sacrifice"] = true,
            ["Terrify"] = true,
            ["Shadow_Dance"] = true,
            ["Wyvern_Sting"] = true,
            ["Garrote_-_Silence"] = true,
            ["Water_Tomb"] = true,
            ["Frostbolt"] = true,
            ["Avenger's_Shield"] = true,
            ["Kidney_Shot"] = true,
            ["Sacrifice"] = true,
            ["Challenging_Roar"] = true,
            ["Lichborne"] = true,
            ["Stop_Time"] = true,
            ["Blood_of_the_Fallen_Cultist"] = true,
            ["Blade_Flurry"] = true,
            ["Demon_Charge"] = true,
            ["Innervate"] = true,
            ["Repentance"] = true,
            ["Howling_Screech"] = true,
            ["Riposte"] = true,
            ["Earth's_Fury"] = true,
            ["Frostbolt_Volley"] = true,
            ["Improved_Fire_Nova_Totem"] = true,
            ["Venom_Web_Spray"] = true,
            ["Hysteria"] = true,
            ["Envenomed_Shot"] = true,
            ["Ice_Beam"] = true,
            ["Army_of_the_Dead"] = true,
            ["Hand_of_Protection"] = true,
            ["Startling_Roar"] = true,
            ["Mark_of_Blood"] = true,
            ["Turn_Evil"] = true,
            ["Ice_Block"] = true,
            ["Mind_Flay"] = true,
            ["Misdirection"] = true,
            ["Blind"] = true,
            ["Pounce"] = true,
            ["Vampiric_Blood"] = true,
            ["Web_Wrap"] = true,
            ["Intimidating_Shout"] = true,
            ["Suffering"] = true,
            ["Divine_Plea"] = true,
            ["Spell_Reflection"] = true,
            ["Rapid_Fire"] = true,
            ["Sap"] = true,
            ["Tendon_Rip"] = true,
            ["Shield_Block"] = true,
            ["Curse_of_Doom"] = true,
            ["Invisibility"] = true,
            ["Killing_Spree"] = true,
            ["Silencing_Shot"] = true,
            ["Deterrence"] = true,
            ["Impact"] = true,
            ["Frost_Blast"] = true,
            ["Astral_Shift"] = true,
            ["Unbreakable_Armor"] = true,
            ["Earthbind"] = true,
            ["Concussive_Shot"] = true,
            ["Warsong_Flag"] = true,
            ["Frenzied_Regeneration"] = true,
            ["Shackle_Undead"] = true,
            ["Strangulate"] = true,
            ["Hand_of_Sacrifice"] = true,
            ["Mirror_Image"] = true,
            ["Will_of_the_Forsaken"] = true,
            ["Frostfire_Bolt"] = true,
            ["Vile_Vomit"] = true,
            ["Evocation"] = true,
            ["Curse_of_Agony"] = true,
            ["Withered_Touch"] = true,
            ["Aftermath"] = true,
            ["Typhoon"] = true,
            ["Bladestorm"] = true,
            ["Judgement_of_Justice"] = true,
            ["Intercept"] = true,
            ["Savage_Roar"] = true,
            ["Death_Coil"] = true,
            ["Sonic_Blast"] = true,
            ["Summoning_Disorientation"] = true,
            ["Nightmare"] = true,
            ["Avenging_Wrath"] = true,
            ["Polymorph"] = true,
            ["Hammer_Blow"] = true,
            ["Evasion"] = true,
            ["Shadowfury"] = true,
            ["Dismantle"] = true,
            ["Frightening_Shout"] = true,
            ["Scatter_Shot"] = true,
            ["Divine_Shield"] = true,
            ["Icy_Veins"] = true,
            ["Crippling_Poison"] = true,
            ["Revenge_Stun"] = true,
            ["Dispersion"] = true,
            ["Nature's_Grasp"] = true,
            ["Gushing_Wound"] = true,
            ["Web_Spray"] = true,
            ["Arcane_Torrent"] = true,
            ["Fury_of_the_Fallen_Cultist"] = true,
            ["Snatch"] = true,
            ["Frost_Shock"] = true,
            ["Bloodlust"] = true,
            ["Shattering_Throw"] = true,
            ["Inferno_Effect"] = true,
            ["Stampy's_Stompy-Stomp"] = true,
            ["Rip"] = true,
            ["Dazed"] = true,
            ["Hammer_of_Justice"] = true,
            ["Cripple"] = true,
            ["Concussive_Barrage"] = true,
        },
        ["oUF_Lunar_Raid"] = {
            ["Mark_of_Blood"] = true,
            ["Aimed_Shot"] = true,
            ["Garrote_-_Silence"] = true,
            ["Tranquility"] = true,
            ["Mortal_Strike"] = true,
            ["Misdirection"] = true,
            ["Polymorph"] = true,
            ["Warsong_Flag"] = true,
            ["Innervate"] = true,
            ["Divine_Shield"] = true,
            ["Freezing_Arrow_Effect"] = true,
            ["Divine_Protection"] = true,
            ["Mortal_Wound"] = true,
            ["Wound_Poison_IV"] = true,
            ["Netherstorm_Flag"] = true,
            ["Shield_Wall"] = true,
            ["Soul_Strike"] = true,
            ["Freezing_Trap_Effect"] = true,
            ["Wound_Poison"] = true,
            ["Spell_Reflection"] = true,
            ["Wound_Poison_VI"] = true,
            ["Unbreakable_Armor"] = true,
            ["Wyvern_Sting"] = true,
            ["Shield_Block"] = true,
            ["Sap"] = true,
            ["Wound_Poison_V"] = true,
            ["Hand_of_Protection"] = true,
            ["Wound_Poison_VII"] = true,
            ["Silverwing_Flag"] = true,
            ["Ice_Block"] = true,
        },
        ["oUF_Lunar_PartyPet"] = {
            ["Mark_of_Blood"] = true,
            ["Aimed_Shot"] = true,
            ["Freezing_Trap_Effect"] = true,
            ["Mortal_Strike"] = true,
            ["Entangling_Roots"] = true,
            ["Howl_of_Terror"] = true,
            ["Polymorph"] = true,
            ["Warsong_Flag"] = true,
            ["Scatter_Shot"] = true,
            ["Mortal_Wound"] = true,
            ["Freezing_Arrow_Effect"] = true,
            ["Wound_Poison_IV"] = true,
            ["Shield_Wall"] = true,
            ["Fear"] = true,
            ["Netherstorm_Flag"] = true,
            ["Tranquility"] = true,
            ["Misdirection"] = true,
            ["Soul_Strike"] = true,
            ["Wound_Poison"] = true,
            ["Spell_Reflection"] = true,
            ["Hungering_Cold"] = true,
            ["Wound_Poison_VI"] = true,
            ["Wyvern_Sting"] = true,
            ["Shield_Block"] = true,
            ["Sap"] = true,
            ["Wound_Poison_V"] = true,
            ["Hand_of_Protection"] = true,
            ["Wound_Poison_VII"] = true,
            ["Ice_Block"] = true,
            ["Cyclone"] = true,
        },
        ["oUF_Lunar_Target"] = {
            ["Mortal_Wound"] = true,
            ["Evasion"] = true,
            ["Howl_of_Terror"] = true,
            ["Roar_of_Recovery"] = true,
            ["Anti-Magic_Shell"] = true,
            ["Survival_Instincts"] = true,
            ["Nature's_Grasp"] = true,
            ["Hysteria"] = true,
            ["Adrenaline_Rush"] = true,
            ["Hungering_Cold"] = true,
            ["Hand_of_Protection"] = true,
            ["Ice_Block"] = true,
            ["Cyclone"] = true,
            ["Mark_of_Blood"] = true,
            ["Aimed_Shot"] = true,
            ["Divine_Illumination"] = true,
            ["Shield_Wall"] = true,
            ["Misdirection"] = true,
            ["Dancing_Rune_Weapon"] = true,
            ["Vampiric_Blood"] = true,
            ["Tendon_Rip"] = true,
            ["Spell_Reflection"] = true,
            ["Barkskin"] = true,
            ["Sap"] = true,
            ["Shield_Block"] = true,
            ["Silverwing_Flag"] = true,
            ["Master's_Call"] = true,
            ["Deterrence"] = true,
            ["Freezing_Trap_Effect"] = true,
            ["Mortal_Strike"] = true,
            ["Warsong_Flag"] = true,
            ["Aura_Mastery"] = true,
            ["Cloak_of_Shadows"] = true,
            ["Netherstorm_Flag"] = true,
            ["Drink"] = true,
            ["Wound_Poison"] = true,
            ["Divine_Sacrifice"] = true,
            ["Shadow_Dance"] = true,
            ["Wyvern_Sting"] = true,
            ["Sprint"] = true,
            ["Avenging_Wrath"] = true,
            ["Tranquility"] = true,
            ["Killing_Spree"] = true,
            ["Scatter_Shot"] = true,
            ["Divine_Shield"] = true,
            ["Freezing_Arrow_Effect"] = true,
            ["Sacrifice"] = true,
            ["Garrote_-_Silence"] = true,
            ["Soul_Strike"] = true,
            ["Unbreakable_Armor"] = true,
            ["Frenzied_Regeneration"] = true,
            ["Fear"] = true,
            ["Mirror_Image"] = true,
            ["Wound_Poison_VI"] = true,
            ["Intimidation"] = true,
            ["Wound_Poison_V"] = true,
            ["Blade_Flurry"] = true,
            ["Berserker_Rage"] = true,
            ["Divine_Protection"] = true,
            ["Wound_Poison_IV"] = true,
            ["Bloodlust"] = true,
            ["Dash"] = true,
            ["Innervate"] = true,
            ["Death_Coil"] = true,
            ["Wound_Poison_VII"] = true,
        },
        ["oUF_Lunar_Focus"] = {
            ["Mortal_Wound"] = true,
            ["Evasion"] = true,
            ["Howl_of_Terror"] = true,
            ["Roar_of_Recovery"] = true,
            ["Soul_Strike"] = true,
            ["Survival_Instincts"] = true,
            ["Nature's_Grasp"] = true,
            ["Hysteria"] = true,
            ["Adrenaline_Rush"] = true,
            ["Hungering_Cold"] = true,
            ["Hand_of_Protection"] = true,
            ["Berserker_Rage"] = true,
            ["Cyclone"] = true,
            ["Mark_of_Blood"] = true,
            ["Aimed_Shot"] = true,
            ["Divine_Illumination"] = true,
            ["Shield_Wall"] = true,
            ["Misdirection"] = true,
            ["Dancing_Rune_Weapon"] = true,
            ["Vampiric_Blood"] = true,
            ["Tendon_Rip"] = true,
            ["Hex"] = true,
            ["Spell_Reflection"] = true,
            ["Barkskin"] = true,
            ["Sap"] = true,
            ["Shield_Block"] = true,
            ["Silverwing_Flag"] = true,
            ["Master's_Call"] = true,
            ["Freezing_Trap_Effect"] = true,
            ["Mortal_Strike"] = true,
            ["Warsong_Flag"] = true,
            ["Frenzied_Regeneration"] = true,
            ["Mirror_Image"] = true,
            ["Netherstorm_Flag"] = true,
            ["Drink"] = true,
            ["Wound_Poison_VI"] = true,
            ["Seduction"] = true,
            ["Wound_Poison_V"] = true,
            ["Shadow_Dance"] = true,
            ["Wyvern_Sting"] = true,
            ["Sprint"] = true,
            ["Avenging_Wrath"] = true,
            ["Tranquility"] = true,
            ["Killing_Spree"] = true,
            ["Entangling_Roots"] = true,
            ["Scatter_Shot"] = true,
            ["Divine_Shield"] = true,
            ["Freezing_Arrow_Effect"] = true,
            ["Divine_Protection"] = true,
            ["Anti-Magic_Shell"] = true,
            ["Unbreakable_Armor"] = true,
            ["Fear"] = true,
            ["Aura_Mastery"] = true,
            ["Intimidation"] = true,
            ["Cloak_of_Shadows"] = true,
            ["Wound_Poison"] = true,
            ["Divine_Sacrifice"] = true,
            ["Blade_Flurry"] = true,
            ["Ice_Block"] = true,
            ["Wound_Poison_IV"] = true,
            ["Sacrifice"] = true,
            ["Bloodlust"] = true,
            ["Dash"] = true,
            ["Innervate"] = true,
            ["Death_Coil"] = true,
            ["Wound_Poison_VII"] = true,
        },
        ["oUF_Lunar_Party"] = {
            ["Mortal_Wound"] = true,
            ["Mace_Smash"] = true,
            ["Pounce_Bleed"] = true,
            ["Blood_Fury"] = true,
            ["Howl_of_Terror"] = true,
            ["Roar_of_Recovery"] = true,
            ["Anti-Magic_Shell"] = true,
            ["Terrifying_Roar"] = true,
            ["Survival_Instincts"] = true,
            ["Ravage"] = true,
            ["Blind"] = true,
            ["Pounce"] = true,
            ["Adrenaline_Rush"] = true,
            ["Divine_Plea"] = true,
            ["Power_Infusion"] = true,
            ["Berserker_Rage"] = true,
            ["Cyclone"] = true,
            ["Aimed_Shot"] = true,
            ["Holy_Wrath"] = true,
            ["Divine_Illumination"] = true,
            ["Shield_Wall"] = true,
            ["Psychic_Scream"] = true,
            ["Dancing_Rune_Weapon"] = true,
            ["Fear"] = true,
            ["Hex"] = true,
            ["Screams_of_the_Dead"] = true,
            ["Starfall"] = true,
            ["Maim"] = true,
            ["Berserk"] = true,
            ["Intimidation"] = true,
            ["Silencing_Shot"] = true,
            ["Silverwing_Flag"] = true,
            ["Strangulate"] = true,
            ["Tail_Lash"] = true,
            ["Will_of_the_Forsaken"] = true,
            ["Freezing_Trap_Effect"] = true,
            ["Mortal_Strike"] = true,
            ["Icebound_Fortitude"] = true,
            ["Charge_Stun"] = true,
            ["Stun"] = true,
            ["Cloak_of_Shadows"] = true,
            ["Netherstorm_Flag"] = true,
            ["Drink"] = true,
            ["Clearcasting"] = true,
            ["Wound_Poison_VI"] = true,
            ["Tidal_Force"] = true,
            ["Scare_Beast"] = true,
            ["Divine_Sacrifice"] = true,
            ["Bladestorm"] = true,
            ["Shadow_Dance"] = true,
            ["Wyvern_Sting"] = true,
            ["Sprint"] = true,
            ["Garrote_-_Silence"] = true,
            ["Wound_Poison_IV"] = true,
            ["Nature's_Swiftness"] = true,
            ["Stoneclaw_Stun"] = true,
            ["Metamorphosis"] = true,
            ["Kidney_Shot"] = true,
            ["Sonic_Blast"] = true,
            ["Unbreakable_Armor"] = true,
            ["Lichborne"] = true,
            ["Blood_of_the_Fallen_Cultist"] = true,
            ["Fear_Ward"] = true,
            ["Blade_Flurry"] = true,
            ["Demon_Charge"] = true,
            ["Innervate"] = true,
            ["Death_Coil"] = true,
            ["Repentance"] = true,
            ["Evasion"] = true,
            ["Polymorph"] = true,
            ["Pain_Suppression"] = true,
            ["Icebolt"] = true,
            ["Improved_Fire_Nova_Totem"] = true,
            ["Aspect_of_the_Beast"] = true,
            ["Hysteria"] = true,
            ["Shield_Slam"] = true,
            ["Hungering_Cold"] = true,
            ["Army_of_the_Dead"] = true,
            ["Hand_of_Protection"] = true,
            ["Judgement_of_Justice"] = true,
            ["Stampy's_Stompy-Stomp"] = true,
            ["Turn_Evil"] = true,
            ["Disarm"] = true,
            ["Misdirection"] = true,
            ["Impale"] = true,
            ["Web_Wrap"] = true,
            ["Intimidating_Shout"] = true,
            ["Entangling_Roots"] = true,
            ["Spell_Reflection"] = true,
            ["Rapid_Fire"] = true,
            ["Barkskin"] = true,
            ["Mark_of_Blood"] = true,
            ["Icy_Veins"] = true,
            ["Sap"] = true,
            ["Seal_of_Command"] = true,
            ["Shield_Block"] = true,
            ["Invisibility"] = true,
            ["Indestructible"] = true,
            ["Demonic_Empowerment"] = true,
            ["Soul_Strike"] = true,
            ["Astral_Shift"] = true,
            ["Arcane_Power"] = true,
            ["Wound_Poison_VII"] = true,
            ["Shockwave"] = true,
            ["Warsong_Flag"] = true,
            ["Frenzied_Regeneration"] = true,
            ["Viper_Sting"] = true,
            ["Cheap_Shot"] = true,
            ["Tendon_Rip"] = true,
            ["Killing_Spree"] = true,
            ["Mirror_Image"] = true,
            ["Guardian_Spirit"] = true,
            ["Aura_Mastery"] = true,
            ["Hammer_of_Injustice"] = true,
            ["Terrify"] = true,
            ["Wound_Poison"] = true,
            ["Gnaw"] = true,
            ["Ice_Block"] = true,
            ["Seduction"] = true,
            ["Sacrifice"] = true,
            ["Impact"] = true,
            ["Blink"] = true,
            ["Wound_Poison_V"] = true,
            ["Shamanistic_Rage"] = true,
            ["Summoning_Disorientation"] = true,
            ["Nightmare"] = true,
            ["Intercept"] = true,
            ["Shadowfury"] = true,
            ["Avenging_Wrath"] = true,
            ["Tranquility"] = true,
            ["Bloodrage"] = true,
            ["War_Stomp"] = true,
            ["Recklessness"] = true,
            ["Frightening_Shout"] = true,
            ["Scatter_Shot"] = true,
            ["Divine_Shield"] = true,
            ["Freezing_Arrow_Effect"] = true,
            ["Divine_Protection"] = true,
            ["The_Beast_Within"] = true,
            ["Dispersion"] = true,
            ["Nature's_Grasp"] = true,
            ["Prowl"] = true,
            ["Web_Spray"] = true,
            ["Vampiric_Blood"] = true,
            ["Fury_of_the_Fallen_Cultist"] = true,
            ["Startling_Roar"] = true,
            ["Concussion_Blow"] = true,
            ["Bloodlust"] = true,
            ["Revenge_Stun"] = true,
            ["Inferno_Effect"] = true,
            ["Deep_Freeze"] = true,
            ["Retaliation"] = true,
            ["Dash"] = true,
            ["Evocation"] = true,
            ["Bash"] = true,
            ["Hammer_of_Justice"] = true,
        },
        ["oUF_Lunar_X_Target"] = {
        },
        ["oUF_Lunar_PlayerPet"] = {
            ["Mortal_Wound"] = true,
            ["Freezing_Trap_Effect"] = true,
            ["Mortal_Strike"] = true,
            ["Howl_of_Terror"] = true,
            ["Polymorph"] = true,
            ["Soul_Strike"] = true,
            ["Wound_Poison_VI"] = true,
            ["Hungering_Cold"] = true,
            ["Wound_Poison_V"] = true,
            ["Hand_of_Protection"] = true,
            ["Ice_Block"] = true,
            ["Cyclone"] = true,
            ["Mark_of_Blood"] = true,
            ["Aimed_Shot"] = true,
            ["Garrote_-_Silence"] = true,
            ["Wound_Poison_IV"] = true,
            ["Shield_Wall"] = true,
            ["Mend_Pet"] = true,
            ["Misdirection"] = true,
            ["Scatter_Shot"] = true,
            ["Freezing_Arrow_Effect"] = true,
            ["Fear"] = true,
            ["Entangling_Roots"] = true,
            ["Spell_Reflection"] = true,
            ["Tranquility"] = true,
            ["Feed_Pet"] = true,
            ["Bestial_Wrath"] = true,
            ["Sap"] = true,
            ["Wyvern_Sting"] = true,
            ["Wound_Poison"] = true,
            ["Wound_Poison_VII"] = true,
            ["Death_Coil"] = true,
            ["Shield_Block"] = true,
        },
    }
    Lunar_Default_Options["HiddenAuras"] = {
        ["oUF_Lunar_Player"] = {
        },
        ["oUF_Lunar_Party"] = {
        },
        ["oUF_Lunar_PartyPet"] = {
        },
        ["oUF_Lunar_X_Target"] = {
        },
        ["oUF_Lunar_Target"] = {
        },
        ["oUF_Lunar_Raid"] = {
        },
        ["oUF_Lunar_Focus"] = {
        },
        ["oUF_Lunar_PlayerPet"] = {
        },
    }
end
-- Make sure each frame type is has a table
for _, fType in ipairs(oUF_Lunar_FrameNames) do
    if not Lunar_Default_Options.SelfCastPriority[fType] then
        Lunar_Default_Options.SelfCastPriority[fType] = {}
    end
    if not Lunar_Default_Options.PriorityAuras[fType] then
        Lunar_Default_Options.PriorityAuras[fType] = {}
    end
    if not Lunar_Default_Options.HiddenAuras[fType] then
        Lunar_Default_Options.HiddenAuras[fType] = {}
    end
end