local function debug (message)
    DEFAULT_CHAT_FRAME:AddMessage(message)
end
-- Smoothing
local smoothing = {}
local function Smooth(self, Value)
	if Value ~= self:GetValue() then
		smoothing[self] = Value
	else
		smoothing[self] = nil
	end
end

local f, min, max, ceil = CreateFrame("Frame"), math.min, math.max, math.ceil
f:SetScript("OnUpdate", function()
	local limit = 30/GetFramerate()
	for bar, Value in pairs(smoothing) do
		local cur = bar:GetValue()
		local new = cur + min((Value-cur)/2.5, max(Value-cur, limit))
		bar:SetValue_Smooth(new)
		if cur == Value or abs(cur - Value) < 2 then
            bar:SetValue_Smooth(ceil(new))
			smoothing[bar] = nil
		end
	end
end)

-- Color Gradient and Alpha
local function ColorAlphaGradient(perc, ...)
    if perc >= 0.99 then
        local r, g, b, a = select(select('#', ...) - 3, ...)
        return r, g, b, a
    elseif perc <= 0.01 then
        local r, g, b, a= ...
        return r, g, b, a
    end
    
    local num = select('#', ...) / 4
    
    local segment, relperc = math.modf(perc*(num-1))
    
    local r1, g1, b1, a1, r2, g2, b2 , a2= select((segment*4)+1, ...)

    return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc, a1 + (a2-a1)*relperc;
end

-- Set Direction
local function SetDirection(StatusBar)
    smoothing[StatusBar] = nil
    local BarWidth = StatusBar.BarWidth or 1
    
    StatusBar.MainTexture:SetAllPoints(StatusBar)
    StatusBar.BackTexture:SetAllPoints(StatusBar)
    -- For Smoothbar
    if StatusBar.Smooth then
        if StatusBar.ReverseBars then
            if StatusBar.InvertBars then
                StatusBar.SetValue_Smooth = function(StatusBar, Value)
                    Value = Value <= StatusBar.MaxValue and Value>=0 and Value or StatusBar.MaxValue
                    StatusBar.MainTexture:SetPoint('TOPRIGHT', StatusBar, 'TOPLEFT', BarWidth - (BarWidth * Value / StatusBar.MaxValue), 0);
                    StatusBar.Value = Value
                end
            else
                StatusBar.SetValue_Smooth = function(StatusBar, Value)
                    Value = Value <= StatusBar.MaxValue and Value>=0 and Value or StatusBar.MaxValue
                    StatusBar.MainTexture:SetPoint('TOPLEFT', StatusBar, 'TOPRIGHT', - (BarWidth * Value / StatusBar.MaxValue), 0);
                    StatusBar.Value = Value
                end
            end
        else
            if StatusBar.InvertBars then
                StatusBar.SetValue_Smooth = function(StatusBar, Value)
                    Value = Value <= StatusBar.MaxValue and Value>=0 and Value or StatusBar.MaxValue
                    StatusBar.MainTexture:SetPoint('TOPLEFT', StatusBar, 'TOPRIGHT', (BarWidth * Value / StatusBar.MaxValue) - BarWidth, 0);
                    StatusBar.Value = Value
                end
            -- Default function
            else
                StatusBar.SetValue_Smooth = function(StatusBar, Value)
                    Value = Value <= StatusBar.MaxValue and Value>=0 and Value or StatusBar.MaxValue
                    StatusBar.MainTexture:SetPoint('TOPRIGHT', StatusBar, 'TOPLEFT', (BarWidth * Value / StatusBar.MaxValue), 0);
                    StatusBar.Value = Value
                end
            end
        end
        StatusBar.SetValue = Smooth
        StatusBar:SetValue_Smooth(StatusBar.MaxValue)
        
    -- For no SmoothBar
    else
        if StatusBar.ReverseBars then
            if StatusBar.InvertBars then
                StatusBar.SetValue = function(StatusBar, Value)
                    Value = Value <= StatusBar.MaxValue and Value>=0 and Value or StatusBar.MaxValue
                    StatusBar.Value = Value
                    StatusBar.MainTexture:SetPoint('TOPRIGHT', StatusBar, 'TOPLEFT', BarWidth - (BarWidth * Value / StatusBar.MaxValue), 0);
                end
            else
                StatusBar.SetValue = function(StatusBar, Value)
                    Value = Value <= StatusBar.MaxValue and Value>=0 and Value or StatusBar.MaxValue
                    StatusBar.Value = Value
                    StatusBar.MainTexture:SetPoint('TOPLEFT', StatusBar, 'TOPRIGHT', - (BarWidth * Value / StatusBar.MaxValue), 0);
                end
            end
        else
            if StatusBar.InvertBars then
                StatusBar.SetValue = function(StatusBar, Value)
                    Value = Value <= StatusBar.MaxValue and Value>=0 and Value or StatusBar.MaxValue
                    StatusBar.Value = Value
                    StatusBar.MainTexture:SetPoint('TOPLEFT', StatusBar, 'TOPRIGHT', (BarWidth * Value / StatusBar.MaxValue) - BarWidth, 0);
                end
            -- Default function
            else
                StatusBar.SetValue = function(StatusBar, Value)
                    Value = Value <= StatusBar.MaxValue and Value>=0 and Value or StatusBar.MaxValue
                    StatusBar.Value = Value
                    StatusBar.MainTexture:SetPoint('TOPRIGHT', StatusBar, 'TOPLEFT', (BarWidth * Value / StatusBar.MaxValue), 0);
                end
            end
        end
        StatusBar:SetValue(StatusBar.MaxValue)
    end
end
-- Create Prototype
local Prototype = CreateFrame('StatusBar', name)
do
    Prototype.SetDirection = SetDirection
    Prototype.BarWidth = 1
    Prototype.MaxValue = 1
    Prototype.smoothGradient = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,}
    
    Prototype.SetReverse = function(StatusBar, IsReverse)
        StatusBar.ReverseBars = IsReverse;
        StatusBar:SetDirection();
    end
    
    Prototype.SetInvert = function(StatusBar, IsInvert)
        StatusBar.InvertBars = IsInvert;
        StatusBar:SetDirection();
    end
    
    Prototype.SetStatusBar_BackTexture = function(StatusBar, ...)
        StatusBar.BackTexture:SetTexture(...);
    end
    
    -- ColorGradient with alpha
    Prototype.ColorAlphaGradient = ColorAlphaGradient
    
    -- Begin Replacement Functions
    Prototype.GetMinMaxValues = function(StatusBar)
        return 0 , StatusBar.MaxValue;
    end
    
    Prototype.GetValue = function(StatusBar)
        return (StatusBar.Value or 0)
    end
    
    Prototype.GetStatusBarColor = function(StatusBar)
        return StatusBar.MainTexture:GetVertexColor();
    end
    
    Prototype.GetStatusBarTexture = function(StatusBar)
        local MainTexture = StatusBar.MainTexture
        return (MainTexture)
    end
    
    Prototype.SetMinMaxValues = function(StatusBar, min, max)
        StatusBar.MaxValue = (max > 0) and max or 1
    end

    Prototype.SetStatusBarColor = function(StatusBar, r, g, b, a)
        StatusBar.MainTexture:SetVertexColor(r, g, b, a);
    end
    
    Prototype.SetStatusBar_BackColor = function(StatusBar, r, g, b, a)
        StatusBar.BackTexture:SetVertexColor(r, g, b, a);
    end
    
    Prototype.SetStatusBarTexture = function(StatusBar, ...)
        StatusBar.MainTexture:SetTexture(...);
        StatusBar.BackTexture:SetTexture(...);
    end
    
    Prototype.OldSetWidth = Prototype.SetWidth
    Prototype.SetWidth = function(StatusBar, ...)
        StatusBar:SetValue(StatusBar.MaxValue)
        StatusBar.BarWidth = ...
        StatusBar:OldSetWidth(...)
        StatusBar:SetDirection()
    end
end
local MetaUberStatusBar = {__index = Prototype}
local MetaUberStatusBar_smoothGradient = {__index = Prototype.smoothGradient}
-- Uber Bar
function Create_UberStatusBar(name, parent)
    local StatusBar = CreateFrame('StatusBar', name, parent)
    StatusBar:SetValue(0)
    StatusBar:SetStatusBarColor(0, 0, 0, 0)
    setmetatable(StatusBar, MetaUberStatusBar)
    
    StatusBar.smoothGradient = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,}
        
	StatusBar.MainTexture = StatusBar:CreateTexture(nil, 'BORDER')
	StatusBar.MainTexture:SetAllPoints(StatusBar)
    
	StatusBar.BackTexture = StatusBar:CreateTexture(nil, 'BACKGROUND')
	StatusBar.BackTexture:SetAllPoints(StatusBar)
    
    --get it done.
    StatusBar:SetDirection()
    StatusBar:SetStatusBarTexture(0,0,0,1)
    return StatusBar;
end