local function debug (message)
    DEFAULT_CHAT_FRAME:AddMessage(message)
end

local function message(msg)
    DEFAULT_CHAT_FRAME:AddMessage(msg)
end

-- Declare locals
local MenuName = 'Lunar Unit Frames';
local ProfileSaved = false;
local NameCounter = 1
local 
    KA,
    OptionsMenu,
    MinimapSettings,
    FrameUnLock,
    Build_Spell_ID_Database,
    PreviousProfileState,
    PreviousSelectedProfile,
    CombatLogParser,
    KA_Update_Via_UnitAura;
-- Prepare pointers to external functions
local
    Profile,
    DragableFrames, 
    OpenUnitFrames, 
    ShowHideFrames, 
    PartyUpdate, 
    AdjustRangeAlpha, 
    AdjustFrame, 
    AdjustAuraSize, 
    AdjustSpacers, 
    AdjustScale, 
    RefreshFrames,
    Textures_Hash,
    Font_Hash,
    AdjustTextures,
    AdjustColors,
    oUF_Lunar_FrameTypes,
    default_options,
    AdjustFrameWidth,
    PartyUpdate,
    ToggleAuraMonitoring,
    ShowHideBlizzardFrames,
    AdjustBarHeight,
    AdjustFont,
    AdjustTextVisibility,
    AdjustFrameHeight;
--
local function debugTexture(frame)
    frame.MainTexture = frame:CreateTexture(nil, "BORDER")
    frame.MainTexture:SetAllPoints(frame)
    frame.MainTexture:SetPoint("CENTER")
    frame.MainTexture:SetTexture(.4,.4,.8, 0.8) 
end
-- Localize game api's
local L_UnitIsFriend = UnitIsFriend
local L_UnitIsPlayer = UnitIsPlayer
local L_UnitClass = UnitClass
local L_UnitExists = UnitExists
local L_UnitIsUnit = UnitIsUnit
-- Database for all SpellId's
local Spell_ID_Database = {}

local fontpath = "Fonts\\FRIZQT__.TTF"
local CheckBox_Tex = "Interface\\Buttons\\UI-CheckBox-Check"
local DisabledCheckBox_Tex = "Interface\\Buttons\\UI-CheckBox-Check-Disabled"
local MinimapIcon = 'Interface\\Addons\\oUF_Lunar_Config\\media\\Earth_Icon'
local BrightTexture = 'Interface\\Addons\\oUF_Lunar\\media\\Melli.tga'

local DisplayNames = {
    Global = 'Global',
    oUF_Lunar_Player = "Player", 
    oUF_Lunar_PlayerPet = "Player Pet", 
    oUF_Lunar_Party = "Party", 
    oUF_Lunar_PartyPet = "Party Pets", 
    oUF_Lunar_Raid = "Raid", 
    oUF_Lunar_Arena = "Arena", 
    oUF_Lunar_ArenaPet = "Arena Pets", 
    oUF_Lunar_Focus = "Focus", 
    oUF_Lunar_Target = "Target", 
    oUF_Lunar_X_Target = "Target of *",
}
---------------------------------------------------- Class Icon coords  
local ClassIcon_Locations = {
    ["DEATHKNIGHT"] = { 0.25, 0.50, 0.50, 0.75 },
    ["DRUID"] = { 0.75, 1.00, 0.00, 0.25 },
    ["HUNTER"] = { 0.00, 0.25, 0.25, 0.50 },
    ["MAGE"] = { 0.25, 0.50, 0.00, 0.25 },
    ["PALADIN"] = { 0.00, 0.25, 0.50, 0.75 },
    ["PRIEST"] = { 0.50, 0.75, 0.25, 0.50 },
    ["ROGUE"] = { 0.50, 0.75, 0.00, 0.25 },
    ["SHAMAN"] = { 0.25, 0.50, 0.25, 0.50 },
    ["WARLOCK"] = { 0.75, 1.00, 0.25, 0.50 },    
    ["WARRIOR"] = { 0.00, 0.25, 0.00, 0.25 },
}
local ClassIndex = {
    [1] = "DEATHKNIGHT",
    [2] = "DRUID",
    [3] = "HUNTER",
    [4] = "MAGE",
    [5] = "PALADIN",
    [6] = "PRIEST",
    [7] = "ROGUE",
    [8] = "SHAMAN",
    [9] = "WARLOCK",    
    [10] = "WARRIOR",
    DEATHKNIGHT = 1,
    DRUID = 2,
    HUNTER = 3,
    MAGE = 4,
    PALADIN = 5,
    PRIEST = 6,
    ROGUE = 7,
    SHAMAN = 8,
    WARLOCK = 9,    
    WARRIOR = 10
}
-- Background and border for panels
local narrowBorder = {
    bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
    edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", tile = true, tileSize = 32, edgeSize = 16, 
    insets = { left = 4, right = 4, top = 4, bottom = 4 } 
}

local BoxBorder = {
--~     bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", 
    bgFile = "", 
    edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", tile = true, tileSize = 32, edgeSize = 16, 
    insets = { left = -4, right = -4, top = -4, bottom = -4 } 
}
--[[****************************************************************************************************************************************************]]--
--                                                   COMMON FUNCTIONS

local dummy  = function() end
-- Profile backup functions
local function BackupProfile()
    if not ProfileSaved then
        PreviousProfileState = deepcopy(oUF_Lunar_Profiles)
        PreviousSelectedProfile = oUF_Lunar_SelectedProfile
        ProfileSaved = true
    end
    local static1 = getglobal("StaticPopup1")
    static1:SetFrameLevel(99)
--~     static1:ClearAllPoints()
--~     static1:SetPoint("TOP", 0, -30)
end

local function RestoreProfile()
    if ProfileSaved then
        ProfileSaved = false
        local SelectedProfile = PreviousSelectedProfile
        oUF_Lunar_Profiles = deepcopy(PreviousProfileState)
        message("Lunar Unit Frames: Changes have been canceled.")
        Profile = oUF_Lunar_SetProfile(SelectedProfile)
    end
end

local function ReleaseBackup()
    ProfileSaved = false
end

--  Checkbox and Slider
local function template(type, name, parent, description, yPos, xPos)
    local yPos = yPos or 0
    local xPos = xPos or 25
    local name = "Lunar_"..(name or "")..NameCounter
    NameCounter = NameCounter + 1
    
    local frame = CreateFrame(type, name, parent, "Options"..type.."Template")
    frame:SetPoint("TOPLEFT", parent, "TOPLEFT", xPos, yPos)
    frame.text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    frame.text:SetText(description)
    frame.text:SetPoint("LEFT", frame, "RIGHT", 12, 0)
    frame.text:SetTextColor(1,1,1)
 
    if type == "Slider" then        
--~         frame.ValueText = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
--~         frame.ValueText = getglobal(name.."Text")
--~         frame.ValueText:SetPoint("TOP", frame, "BOTTOM", 0, 0);
--~         frame.ValueText:SetTextColor(1,1,1)
        
        frame.TextLeft = getglobal(name.."Low")
        frame.TextLeft:SetFontObject("GameFontHighlight")
        
        frame.TextRight = getglobal(name.."High")
        frame.TextRight:SetFontObject("GameFontHighlight")
        
        frame.TextCenter = getglobal(name.."Text")
        frame.TextCenter:ClearAllPoints()
        frame.TextCenter:SetPoint('TOP', frame, 'BOTTOM', 0, 2)
        frame.TextCenter:SetFontObject("GameFontHighlight")
    end
    
    return frame;
end

-- Tooltip
local function Tooltip_OnEnter()
    if Profile.Tutorial_Tooltips then
        GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT")
        GameTooltip:ClearLines()
        GameTooltip:AddLine(this.TooltipText, 1, 1, 1, 1);
        if type(this.TooltipFunc) == 'function' then
            this.TooltipFunc()
        end
        GameTooltip:Show()
    end
end
local function ToolTip_OnLeave()
    GameTooltip:Hide()
end
local function MakeTooltip(frame, tip, func)
    frame.TooltipText = tip
    frame.TooltipFunc = func
    frame:SetScript("OnEnter", Tooltip_OnEnter)
    frame:SetScript("OnLeave", ToolTip_OnLeave)
end

-- SubMenu
local function SubMenu_Template(name)
    local SubMenu = CreateFrame("Frame", name, InterfaceOptionsFramePanelContainer)
    
    SubMenu.name = name
    SubMenu.parent = MenuName
    SubMenu.addonname = MenuName
    SubMenu:Hide()
    InterfaceOptions_AddCategory(SubMenu)
    
    SubMenu.cancel = RestoreProfile
    SubMenu:SetScript("OnShow", BackupProfile)
--~     SubMenu:SetScript("OnHide", ReleaseBackup)
    SubMenu.okay = ReleaseBackup
    
--~     InterfaceOptionsFramePanelContainer:SetPoint('BOTTOMLEFT', InterfaceOptionsFrame, 'BOTTOMLEFT', 200,40)
    return SubMenu;
end

-- Make a frame movable
local function OnDragStart()
    this:StartMoving()
end
local function OnDragStop()
    this:StopMovingOrSizing()
end
local function Make_Movable(frame)
    frame:EnableMouse(true)
    frame:SetMovable(true)
    frame:RegisterForDrag("LeftButton")
    frame:SetScript("OnDragStart", OnDragStart);
    frame:SetScript("OnDragStop", OnDragStop);
end

-- Check for a value in a list
local function ListCheck(query, list)
    for _, item in ipairs(list) do
        if query == item then
            return true;
        end
    end
end
-- Reload Panel after reset to defaults
local function ResetPage(Panel, ...)
    Panel:Hide()
    Profile = oUF_Lunar_SetProfile(oUF_Lunar_SelectedProfile, false)
    Panel:Show()
end

local function Lunar_ToggleFrameLock()
    if not FrameUnLock and InCombatLockdown() then
        return
    end
    FrameUnLock = not FrameUnLock
    Lunar_Dragging(FrameUnLock, DragableFrames, Profile, OpenUnitFrames, Lunar_ToggleFrameLock)
    OptionsMenu.SubMenu_1.locking:SetChecked(not FrameUnLock)
    if FrameUnLock then
        InterfaceOptionsFrame:Hide()
    end
end
-- Create minimap button
local MinimapInfo = {
    icon = MinimapIcon,
    position = -45, -- default only. after first use, SavedVariables used
    drag = "CIRCLE", -- default only. after first use, SavedVariables used
    left = function()
        Lunar_ToggleFrameLock()
    end,
    right = function()
        InterfaceOptionsFrame_OpenToCategory(OptionsMenu.SubMenu_1)
        InterfaceOptionsFrame_OpenToCategory(OptionsMenu)
    end,
    enabled = 1 -- default only. after first use, SavedVariables used
}

local function UpdateMinimapButton(enable)
    local Name = "Lunar Unit Frames"
    local Button = getglobal(Name.."MinimapButton")
    if not Button then
        MinimapInfo.enabled = enable and 1 or 0
        MyMinimapButton:Create(Name,Profile.Minimap,MinimapInfo)
    else
        Button.modSettings = Profile.Minimap
        MyMinimapButton:SetEnable(Name,enable);
    end
end
-- MakeFontString
local function MakeFontString(self, inputSize, s)
    local size = inputSize or 12
    local Text = self:CreateFontString(nil, 'OVERLAY')
    Text:SetFont(fontpath, size)
    Text:SetTextColor(1,1,1)
    Text:SetShadowOffset(1, -1)
    Text:SetText(s or "")
    return Text;
end

local function Animation_Setup(self, name)
    local Grow = 1.5
    local Dur = 0.5
    local group = self:CreateAnimationGroup(name)
    if name == 'ZoomIn' then
        group.action1 = group:CreateAnimation("SCALE", name)
        group.action1:SetScale(Grow, Grow) 
        group.action1:SetDuration(Dur)
        group.action1:SetEndDelay(99)  
        
    elseif name == 'ZoomOut' then
        group.action1 = group:CreateAnimation("SCALE", name)
        group.action1:SetScale(Grow, Grow) 
        group.action1:SetDuration(0)
        group.action1:SetOrder(1)
        
        group.action2 = group:CreateAnimation("SCALE", name)
        group.action2:SetScale(1/Grow, 1/Grow) 
        group.action2:SetDuration(Dur)
        group.action2:SetOrder(2)
    elseif name == 'Flash' then
        group.action1 = group:CreateAnimation("ALPHA", name)
        group.action1:SetChange(-0.7) 
        group.action1:SetDuration(Dur)
        group:SetLooping("BOUNCE")
    end
    self[name] = group
end
local function EnableButtonFlash(button)
    Animation_Setup(button, 'Flash')
    button.OldDisable = button.Disable
    button.Disable = function()
        button.Flash:Stop()
        button:OldDisable()
    end
    button.OldEnable = button.Enable
    button.Enable = function()
        button:OldEnable()
        button.Flash:Play()
    end
    button:SetScript('OnHide', function()
        this.Flash:Stop()
    end)
end
local function EnableScrollFrameButtonFlashing(ScrollFrame)
    local name = ScrollFrame:GetName()
    EnableButtonFlash(getglobal(name.."ScrollBar".."ScrollUpButton"))
    EnableButtonFlash(getglobal(name.."ScrollBar".."ScrollDownButton")) 
end

--[[****************************************************************************************************************************************************]]--
--                                                   MAIN - INFO & CREDITS
local function Main()

    local Main = CreateFrame("Frame", nil, InterfaceOptionsFramePanelContainer)
    
    local function MakeFontString(inputSize, s)
        local size = inputSize or 12
        local Text = Main:CreateFontString(nil, 'OVERLAY')
        Text:SetFont(fontpath, size)
        Text:SetTextColor(1,1,1)
        Text:SetShadowOffset(1, -1)
        Text:SetText(s)
        return Text;
    end
    
    Main.name = MenuName
    Main:Hide()
    
    Main.title = MakeFontString(18, "Lunar Unit Frames")
    Main.title:SetPoint("TOP", Main, "TOP", 0, -15);
    Main.title:SetTextColor(.4,.4,.8,1)
    Main.title:SetWidth("600")
    
    Main.credits = MakeFontString(10,
        "Designer & Writer: Max (aka Hondor)\n"..
        "Assistant Designer: Zeb (aka Gakizbak)"
    )
    Main.credits:SetPoint("TOP", Main, "TOP", 0, -40);
    Main.credits:SetWidth("600")
    
    Main.intro = MakeFontString(12,
        "Thanks for using Lunar, I hope you like it. You can always find the latest version on www.wowinterface.com. Please feel free to suggest new features or report problems.\n\n"..
        "There are 4 sub menus to the left with options for the unit frames, all with tooltips to explain.\n\n"..
        "Some important things to know:\n\n"..
        "*** Hold the ALT key and mouseclick on auras to toggle priority:\n"..
        "         Left = hide, Middle = self-cast priority, Right = anyone-cast priority)\n\n"..
        "*** Hold the ALT key and mouseover a party or raid frame to see it's normal buffs/debuffs\n\n"..
        '*** While "Custom Party Filtering" is enabled, the party frame only shows selected raid members.\n'..
        "         Raid members can be added/removed from the BG party through the Right-Click menu."
    )
    Main.intro:SetPoint("TOPLEFT", Main, "TOPLEFT", 25, -80);
    Main.intro:SetJustifyH("LEFT");
    Main.intro:SetWidth("600")
    
    return Main;
end

--[[****************************************************************************************************************************************************]]--
--                                                   SUBFRAME 1 - GENERIC OPTIONS
local function Generic_Options()
        
	local SubMenu = SubMenu_Template("Options")
        
    -- Show tooltips
    SubMenu.Tutorial_Tooltips = template("CheckButton", "Tutorial_Tooltips", SubMenu, "Show tutorial tooltips", -25) 
        MakeTooltip(SubMenu.Tutorial_Tooltips,
            'When checked, tutorial tooltips like this will be shown.'
        )
        SubMenu.Tutorial_Tooltips:SetScript("OnClick", function() 
            Profile.Tutorial_Tooltips = this:GetChecked()
        end);
        SubMenu.Tutorial_Tooltips:SetScript("OnShow", function() 
            this:SetChecked(Profile.Tutorial_Tooltips)
        end);
        
    -- Monitor for new auras
    SubMenu.GatherNewAuras = template("CheckButton", "GatherNewAuras", SubMenu, "Monitor for new auras", -50) 
        MakeTooltip(SubMenu.GatherNewAuras,
            'This monitors combat log for new auras and adds to the database. Turn off to reduce CPU usage.'
        )
        SubMenu.GatherNewAuras:SetScript("OnClick", function() 
            ToggleAuraMonitoring(this:GetChecked())
        end);
        SubMenu.GatherNewAuras:SetScript("OnShow", function() 
            SubMenu.GatherNewAuras:SetChecked(Profile.GatherNewAuras.enabled)
        end);
    
    -- Show minimap icon
    SubMenu.Minimap = template("CheckButton", "Minimap", SubMenu, "Enable minimap button", -75) 
        MakeTooltip(SubMenu.Minimap,
            'Enable Minimap Button'
        )
        SubMenu.Minimap:SetScript("OnClick", function() 
            local checked = this:GetChecked()
            UpdateMinimapButton(checked)
        end);
        SubMenu.Minimap:SetScript("OnShow", function() 
            this:SetChecked(Profile.Minimap.enabled)
        end);
    
    -- Show blizzard buffs
    SubMenu.BlizzardBuffs = template("CheckButton", "BlizzardBuffs", SubMenu, "Enable blizzard's default buffs/debuffs", -100) 
        MakeTooltip(SubMenu.BlizzardBuffs,
            "This will show buffs and debuffs in the game's normal position"
        )
        SubMenu.BlizzardBuffs:SetScript("OnClick", function() 
            local checked = this:GetChecked()
            Profile.BlizzardFrames.BuffFrame = checked or false
            ShowHideBlizzardFrames('BuffFrame')
        end);
        SubMenu.BlizzardBuffs:SetScript("OnShow", function() 
            this:SetChecked(Profile.BlizzardFrames.BuffFrame)
        end);
    
    -- Show blizzard castbars
    SubMenu.BlizzardCastbars = template("CheckButton", "BlizzardCastbars", SubMenu, "Enable blizzard's default cast bars", -125) 
        MakeTooltip(SubMenu.BlizzardCastbars,
            "This will show the game's default cast bars"
        )
        SubMenu.BlizzardCastbars:SetScript("OnClick", function() 
            local checked = this:GetChecked()
            Profile.BlizzardFrames.Castbars = checked or false
            ShowHideBlizzardFrames('Castbars')
        end);
        SubMenu.BlizzardCastbars:SetScript("OnShow", function() 
            this:SetChecked(Profile.BlizzardFrames.Castbars)
        end);
        
    -- Lock frames
    SubMenu.locking = template("CheckButton", "oUF_Lunar_Lock_CheckBox", SubMenu, "Lock all frames", -150) 
        MakeTooltip(SubMenu.locking,
            'When unlocked, any unit frame may be dragged to any position on the screen.\n'..
            'The Positions will be saved to the current Profile, any may be reset using the reset positions button.\n'..
            'Do not use during combat.'
        )
        SubMenu.locking:SetScript("OnClick", function()
            Lunar_ToggleFrameLock()
        end);
        SubMenu.locking:SetScript("OnShow", function() 
            this:SetChecked(not FrameUnLock)
        end);
        
    -- Alt on mouse over
--~     SubMenu.Alt_To_MouseOver = template("CheckButton", "Alt_To_MouseOver", SubMenu, "Mouseover buffs require ALT key", -175) 
--~         MakeTooltip(SubMenu.Alt_To_MouseOver,
--~             "When unchecked, mouseover-buffs will always show when you mouseover a frame. When checked, the ALT key must be held."
--~         )
--~         SubMenu.Alt_To_MouseOver:SetScript("OnClick", function() 
--~             Profile.Alt_To_MouseOver = this:GetChecked() or false
--~             AdjustFrame('Global')
--~         end);
--~         SubMenu.Alt_To_MouseOver:SetScript("OnShow", function() 
--~             this:SetChecked(Profile.Alt_To_MouseOver)
--~         end);
        
    -- HealComm
    SubMenu.HealComm = template("CheckButton", "HealComm", SubMenu, "Incoming heal indication", -175) 
        MakeTooltip(SubMenu.HealComm,
            "This will enable HealComm.\nHealComm will show a white bar on a unit's health bar to indicate incoming heals.\nIt works best when your allies also have LibHealComm-3.0"
        )
        SubMenu.HealComm:SetScript("OnClick", function() 
            Profile.HealComm = this:GetChecked() or false
            AdjustFrame('Global')
        end);
        SubMenu.HealComm:SetScript("OnShow", function() 
            this:SetChecked(Profile.HealComm)
        end);
        
    -- MinimumRaid
    SubMenu.MinimumRaid = template("CheckButton", "MinimumRaid", SubMenu, "5 player raid as party", -200) 
        MakeTooltip(SubMenu.MinimumRaid,
            "When checked, 5 player raids will be treated as a party."
        )
        SubMenu.MinimumRaid:SetScript("OnClick", function() 
            Profile.MinimumRaid = this:GetChecked() and 5 or 0
            PartyUpdate('GUILD_ROSTER_UPDATE')
        end);
        SubMenu.MinimumRaid:SetScript("OnShow", function() 
            this:SetChecked(Profile.MinimumRaid==5)
        end);

    -- Party Filter Enable
    SubMenu.PartyFilterEnable = template("CheckButton", "PartyFilterEnable", SubMenu, "Enable custom party filtering", -225) 
        MakeTooltip(SubMenu.PartyFilterEnable,
            "While you're in a raid, this features replaces the normal party member frames with up to 4 raid members of your choice."
        )
        SubMenu.PartyFilterEnable:SetScript("OnClick", function() 
            local checked = this:GetChecked()
            Profile.PartyFilter.Enabled = checked or false
            PartyUpdate('GUILD_ROSTER_UPDATE')
        end);
        SubMenu.PartyFilterEnable:SetScript("OnShow", function() 
            this:SetChecked(Profile.PartyFilter.Enabled)
        end);
        
    -- Party Filter Guild
    SubMenu.PartyFilterGuild = template("CheckButton", "PartyFilterGuild", SubMenu, "Auto add guildmates to custom party", -250, 50) 
        MakeTooltip(SubMenu.PartyFilterGuild,
            "This feature automatically adds guildmates (who are in your raid) to your custom party."
        )
        SubMenu.PartyFilterGuild:SetScript("OnClick", function() 
            local checked = this:GetChecked()
            Profile.PartyFilter.PartyFilterAllowGuild = checked or false
            PartyUpdate('GUILD_ROSTER_UPDATE')
        end);
        SubMenu.PartyFilterGuild:SetScript("OnShow", function() 
            this:SetChecked(Profile.PartyFilter.PartyFilterAllowGuild)
        end);
    -- Party Filter Friends
    SubMenu.PartyFilterFriends = template("CheckButton", "PartyFilterFriends", SubMenu, "Auto add friends to custom party", -275, 50) 
        MakeTooltip(SubMenu.PartyFilterFriends,
            "This feature automatically adds friends (who are on your friends list and in your raid) to your custom party."
        )
        SubMenu.PartyFilterFriends:SetScript("OnClick", function() 
            local checked = this:GetChecked()
            Profile.PartyFilter.PartyFilterAllowFriends = checked or false
            PartyUpdate('GUILD_ROSTER_UPDATE')
        end);
        SubMenu.PartyFilterFriends:SetScript("OnShow", function() 
            this:SetChecked(Profile.PartyFilter.PartyFilterAllowFriends)
        end);
    -- Font Selection
    -- scrolling font window
    do
        local LineHeight = 20
        local RowCount = 12
        local L_FauxScrollFrame_Update = FauxScrollFrame_Update
        local Fonts_KeyList = {}
        local FontCount = 0
        local ScrollBoxWidth = 250
        
        -- Build Fonts keylist
        for entry in pairs(Font_Hash) do
            tinsert(Fonts_KeyList, entry)
        end
        table.sort(Fonts_KeyList)
        FontCount = getn(Fonts_KeyList)
        local ScrollWindow = CreateFrame("ScrollFrame", "Lunar_Font_ScrollFrame", SubMenu, "FauxScrollFrameTemplate")
        EnableScrollFrameButtonFlashing(ScrollWindow)
        ScrollWindow:SetBackdrop(narrowBorder)
        ScrollWindow:SetBackdropColor(0,0,0,1)
        local SharedMedia_Notify = MakeFontString(ScrollWindow, 10, 'Get more fonts with\n the SharedMedia or \nGrizzly Shared Media Addon')
        SharedMedia_Notify:SetPoint('BOTTOM', 0, 10)
        -- FontScrollFrame Update function on scrolling
        ScrollWindow.ScrollUpdate = function(ScrollFrame)
            local lineplusoffset, key, tex;
            if FontCount > RowCount then
                L_FauxScrollFrame_Update(ScrollFrame,FontCount,RowCount,LineHeight);
                SharedMedia_Notify:Hide()
            else
                SharedMedia_Notify:Show()
            end
            local Keys = Fonts_KeyList
            local DB = Font_Hash
            local CustomFrameData = Profile.CustomFrames[this.frameType]
            for line=1, RowCount do --update each row
                local frame = ScrollFrame.ScrollButtons[line]
                lineplusoffset = line + ScrollFrame.offset
                if lineplusoffset <= FontCount then
                    key = Keys[lineplusoffset]
                    tex = DB[key]
                    frame.FontName = key
                    frame.Des:SetText(key)
                    frame.Des:SetFont(Font_Hash[key], 14)
                    frame:Show();
                    if key == Profile.GlobalFont then
                        frame.SelectedTexture:Show()
                    else
                        frame.SelectedTexture:Hide()
                    end
                else
                    frame:Hide();
                end
            end
        end
    
        ScrollWindow:SetWidth(ScrollBoxWidth)
        ScrollWindow:SetHeight(RowCount*LineHeight*1.2)
        ScrollWindow:SetPoint('TOPRIGHT', SubMenu, 'TOPRIGHT', -40, -25)
        
        local ScrollButtons = {}
        local function OnMouseDown()
            this:SetWidth(ScrollBoxWidth*0.95)
            this:SetHeight(LineHeight*0.95)
            Profile.GlobalFont = this.FontName
            AdjustFont('Global')
        end
        local function OnMouseUp()
            this:SetWidth(ScrollBoxWidth)
            this:SetHeight(LineHeight)
            ScrollWindow.ScrollUpdate(ScrollWindow)
        end
        
        for i=1, RowCount do
            local frame = CreateFrame('Button', 'ScrollButton'..i, ScrollWindow)
            frame:SetWidth(ScrollBoxWidth)
            frame:SetHeight(LineHeight)
            frame:SetPoint('CENTER', ScrollWindow, 'TOP', 0, -15 -(LineHeight/2) -((LineHeight+2) * (i-1)))
            frame.SelectedTexture = frame:CreateTexture(nil, "BORDER") 
            frame.SelectedTexture:SetTexture(Textures_Hash['Outline'])
            frame.SelectedTexture:SetPoint('TOPLEFT', frame, 'TOPLEFT', 10,0)
            frame.SelectedTexture:SetPoint('BOTTOMRIGHT', frame, 'BOTTOMRIGHT', -10,0)
            frame.SelectedTexture:SetVertexColor(0,1,0,1)
            frame.Des = MakeFontString(frame, 12)
            frame.Des:SetPoint("CENTER")
            frame:SetScript('OnMouseDown', OnMouseDown)
            frame:SetScript('OnMouseUp', OnMouseUp)
            
            tinsert(ScrollButtons, frame)
        end
        ScrollWindow.ScrollButtons = ScrollButtons

        -- Activate scrollbox scripts
        ScrollWindow:SetScript("OnVerticalScroll", function(self, offset) 
            if FontCount > RowCount then
                FauxScrollFrame_OnVerticalScroll(this, offset, LineHeight, ScrollWindow.ScrollUpdate);
            else
                SharedMedia_Notify:Show()
            end
        end );
        ScrollWindow:SetScript("OnShow", function() 
            ScrollFrame_OnLoad(ScrollWindow)
            ScrollWindow.ScrollUpdate(ScrollWindow)
        end );

    end
    -- Font Size Slider
    do
        local Slider = template("Slider", 'GlobalFontSizeModify', SubMenu, "Font Size", -385, 505)
        Slider:SetWidth(170)
        Slider.text:ClearAllPoints()
        Slider.text:SetPoint("BOTTOM", Slider, "TOP", 0,3)
        Slider.text:SetFont(fontpath, 10)
        Slider.TextLeft:SetText("Smaller");
        Slider.TextRight:SetText("Bigger");
        Slider:SetValueStep(0.25)
        Slider:SetMinMaxValues(-24, 24);
        SubMenu.FontSlider = Slider
        
        Slider:SetScript("OnShow", function()
            this:SetValue(Profile.GlobalFontSizeModify)
        end);
        
        Slider:SetScript("OnValueChanged", function()
            Profile.GlobalFontSizeModify = this:GetValue()
            AdjustFont('Global')
        end);
        
        MakeTooltip(Slider, 'This will adjust all font sizes on the frames') 
    end
    -- Mouseover key modifier dropdown
    SubMenu.MouseOverKey_DropDown = CreateFrame("button", "MouseOverKey_DropDown", SubMenu, "UIDropDownMenuTemplate"); 
    MakeTooltip(SubMenu.MouseOverKey_DropDown,
        'MouseOver buffs will only show when the selected condition is true and you mouseover a frame.\nTo show buffs all the time, select "None"'
    )
    SubMenu.MouseOverKey_DropDown.Des = MakeFontString(SubMenu, 10, "Condition for MouseOver Buffs")
    SubMenu.MouseOverKey_DropDown.Des:SetPoint("BOTTOM", SubMenu.MouseOverKey_DropDown, "TOP", 0,3)
    local InfoKeys = {
        "Is Alt Key Down",
        "Is Control Key Down",
        "Is Shift Key Down",
        "Is Left Alt Key Down",
        "Is Left Control Key Down",
        "Is Left Shift Key Down",
        "Is Right Alt Key Down",
        "Is Right Control Key Down",
        "Is Right Shift Key Down",
        "None",
    }
    SubMenu.MouseOverKey_DropDown:SetPoint("BOTTOM", SubMenu, "BOTTOM", 0, 15);
    UIDropDownMenu_SetWidth(SubMenu.MouseOverKey_DropDown, 180)

    SubMenu.MouseOverKey_DropDown.Description = SubMenu.MouseOverKey_DropDown:CreateFontString(nil, "OVERLAY", "GameFontNormalsmall");
    SubMenu.MouseOverKey_DropDown.Description:SetPoint("CENTER", SubMenu.MouseOverKey_DropDown, "CENTER", 0, 2);
    SubMenu.MouseOverKey_DropDown.Description:SetTextColor(1,1,1)
    SubMenu.MouseOverKey_DropDown.Description:SetText('Condition for mouseover buffs');
    
    SubMenu.MouseOverKey_DropDown:SetScript("OnShow", function()
        UIDropDownMenu_Initialize(SubMenu.MouseOverKey_DropDown, function() 
            local info = UIDropDownMenu_CreateInfo();
            for _, method in ipairs(InfoKeys) do
                info.text = method;
                info.value = 0;
                info.func = function() 
                    Profile.MOB_key = method ~= "None" and method
                    SubMenu.MouseOverKey_DropDown.Description:SetText(method);
                    AdjustFrame('Global')
                end; 
                info.owner = SubMenu.MouseOverKey_DropDown; 
                info.checked = nil; 
                info.icon = nil; 
                UIDropDownMenu_AddButton(info, 1);
            end
        end)
        SubMenu.MouseOverKey_DropDown.Description:SetText(Profile.MOB_key or "None");
    end)
    -- reset frame positions
                
    SubMenu.ResetFramePositions = CreateFrame("Button", "ResetFramePositions", SubMenu, "UIPanelButtonGrayTemplate"); -------reset Profile
        SubMenu.ResetFramePositions:SetPoint("BOTTOMLEFT", SubMenu, 25, 25);
        SubMenu.ResetFramePositions:SetWidth("180")
        SubMenu.ResetFramePositions:SetHeight("22")
        SubMenu.ResetFramePositions:SetText("Reset frame positions")
        MakeTooltip(SubMenu.ResetFramePositions,
            "This will delete all saved frame position information for this Profile.\n"..
            "After reloading, the frames will be set to the default position."
        )
        SubMenu.ResetFramePositions:SetScript("OnClick", function() 
            oUF_Lunar_Profiles [ oUF_Lunar_SelectedProfile ].FramePosition = deepcopy(default_options.FramePosition)
            message('Lunar Unit Frames: '..oUF_Lunar_SelectedProfile..' frame positions have been reset.')
            Profile = oUF_Lunar_SetProfile(oUF_Lunar_SelectedProfile, false)
        end);
                
    SubMenu.default = function()
        Profile.Tutorial_Tooltips = deepcopy(default_options.Tutorial_Tooltips)
        Profile.GatherNewAuras = deepcopy(default_options.GatherNewAuras)
        oUF_Lunar_Profiles [ oUF_Lunar_SelectedProfile ].FramePosition = deepcopy(default_options.FramePosition)
        ResetPage(SubMenu)
    end
            
    return SubMenu;
end

--[[****************************************************************************************************************************************************]]--
--                                                   SUBFRAME 2 - AURAS TABLE
local function Auras_Table()
    local SubMenu = SubMenu_Template("Auras")
    local KA_Tooltip_Strings = {}
    local Cached
    
    local function CreateTooltipCache()
        KA_Tooltip_Strings = {}
        for aura in pairs(KA) do
            local spellLink = KA[aura].spellLink
            local class = KA[aura].class
            local TooltipString = " "
            if spellLink then
                GameTooltip:SetOwner(WorldFrame, "ANCHOR_PRESERVE")
                GameTooltip:ClearLines()
                GameTooltip:SetHyperlink(KA[aura].spellLink)
                for line=1,GameTooltip:NumLines() do
                    local lineText = getglobal("GameTooltipTextLeft" .. line):GetText()
                    if lineText then
                        TooltipString = TooltipString..' '..lineText
                    end

                    lineText = getglobal("GameTooltipTextRight" .. line):GetText()
                    if lineText then
                        TooltipString = TooltipString..' '..lineText
                    end
                end
            end
            TooltipString = TooltipString..'_'..(class or "")
            KA_Tooltip_Strings[aura] = gsub(strlower(TooltipString), " ", "_")
        end
        return KA_Tooltip_Strings;
    end
    local AuraList = {}
    local IncludeTooltipSearch
    
    local function ModifyCustomAuraList(Type, Name, Status)
        Profile["HiddenAuras"][Type][Name] = (Status == "Hidden") or nil
        Profile["PriorityAuras"][Type][Name] = (Status == "Priority") or nil
        Profile["SelfCastPriority"][Type][Name] = (Status == "SelfCast") or nil
    end
    
    local function GetSpellToggleStatus(Type, Name)
        if Profile.HiddenAuras[Type][Name] then
            return("Hidden")
        elseif Profile.PriorityAuras[Type][Name] then
            return("Priority")
        elseif Profile.SelfCastPriority[Type][Name] then
            return("SelfCast")
        end            
    end
    
    local function GetNextStatus(Status)
        local TargetStatus
        if Status == "Priority" then
            TargetStatus = "SelfCast"                
        elseif Status == "SelfCast" then
            TargetStatus = "Hidden"
        elseif not Status then
            TargetStatus = "Priority"
        end
        -- if Hidden, go Normal (TargetStatus=nil)
        return TargetStatus
    end
        
    local function SpellToggleStatus(FrameName, AuraName)
        local TargetStatus
    
        if FrameName == "Global" then
            -- Make all entries match last frame type
            local LastFrameType = oUF_Lunar_FrameTypes[getn(oUF_Lunar_FrameTypes)]
            TargetStatus = GetNextStatus(GetSpellToggleStatus(LastFrameType, AuraName))
            
            -- Apply to all FrameTypes
            for _, FrameName in ipairs(oUF_Lunar_FrameTypes) do
                ModifyCustomAuraList(FrameName, AuraName, TargetStatus)
            end
        elseif AuraName == "Global" then    
            -- Make all entries match last type
            local LastAura = AuraList[getn(AuraList)] 
            TargetStatus = GetNextStatus(GetSpellToggleStatus(FrameName, LastAura))
            
            -- Apply to all Auras
            for _, AuraName in ipairs(AuraList) do
                ModifyCustomAuraList(FrameName, AuraName, TargetStatus)
            end
        else
            TargetStatus = GetNextStatus(GetSpellToggleStatus(FrameName, AuraName))
            ModifyCustomAuraList(FrameName, AuraName, TargetStatus)
        end
    end
    
    local function SearchFilter(aura)
    
        if SubMenu.Only_Show_Modified then
            local found
            for _, FrameName in ipairs(oUF_Lunar_FrameTypes) do
                if GetSpellToggleStatus(FrameName, aura) then
                    found = true
                end
            end
            if not found then return end
        end
        
        local SearchWord = SubMenu.SearchWord or false

        if not SearchWord then return true end
        SearchWord = strlower(SearchWord)
        
        if SearchWord == strlower(KA[aura].buff) then
            return true
        end
        
        local TitleFind = strfind(strlower(aura), SearchWord)
        
        if TitleFind and (TitleFind == 1 or IncludeTooltipSearch) then -- search the aura's name
            return true
        end
        
        if IncludeTooltipSearch and KA_Tooltip_Strings[aura] and strmatch(KA_Tooltip_Strings[aura], SearchWord) then -- search the aura's tooltip
            return true
        end
    end
    ---------------------------------------------------- Sort auras
    SubMenu.SortAurasDropDownMenu = CreateFrame("Frame", "SortAurasDropDownMenu", SubMenu, "UIDropDownMenuTemplate"); 
        
        local AuraCount = 0
        local sortMethods = { "Alphabetical", "Class", "Buff/Debuff" }
        local classes = {}
        
        local function sortAuras(method)
            if getn(AuraList) < 2 then return end
            
            if method == sortMethods[1] then 
                table.sort(AuraList)
            elseif method == sortMethods[2] then
                local ClassOrder = {}
                for aura in pairs(KA) do
                    local class = KA[aura].class or "unknown"
                    if not ClassOrder[class] then
                        ClassOrder[class] = {}
                    end
                    table.insert(ClassOrder[class], tostring(aura))                        
                end
                AuraList = {}
                table.sort(ClassOrder, function (a, b)
                  return tostring(a) > tostring(b)
                end)
                for class in pairs(ClassOrder) do
                    table.sort(ClassOrder[class])
                    for _, aura in ipairs(ClassOrder[class]) do
                        if SearchFilter(aura) then
                            table.insert(AuraList, aura)
                        end
                    end
                end
            elseif method == sortMethods[3] then
                table.sort(AuraList, function (a, b)
                  return ((KA[a].buff or 0) > (KA[b].buff or 0))
                end)
            end
            SubMenu.AurasScrollFrame_Update()
        end
    
        SubMenu.SortAurasDropDownMenu:SetPoint("TOPRIGHT", SubMenu, "TOPRIGHT", -25, -10);
        UIDropDownMenu_SetWidth(SubMenu.SortAurasDropDownMenu, 150)

        SubMenu.SortAurasDropDownMenu.Description = SubMenu.SortAurasDropDownMenu:CreateFontString(nil, "OVERLAY", "GameFontNormalsmall");
        SubMenu.SortAurasDropDownMenu.Description:SetPoint("CENTER", SubMenu.SortAurasDropDownMenu, "CENTER", 0, 2);
        SubMenu.SortAurasDropDownMenu.Description:SetTextColor(1,1,1)
        SubMenu.SortAurasDropDownMenu.Description:SetText('Sort Auras in order of:');
        
        SubMenu.SortAurasDropDownMenu:SetScript("OnShow", function()
            UIDropDownMenu_Initialize(SubMenu.SortAurasDropDownMenu, function() 
                local info = UIDropDownMenu_CreateInfo();
                for _, method in ipairs(sortMethods) do
                    info.text = method;
                    info.value = 0;
                    info.func = function() 
                        Profile.sortMethod = method
                        if AuraCount > 1 then
                            sortAuras(method)
                        end
                        SubMenu.SortAurasDropDownMenu.Description:SetText(method)
                    end; 
                    info.owner = SubMenu.SortAurasDropDownMenu; 
                    info.checked = nil; 
                    info.icon = nil; 
                    UIDropDownMenu_AddButton(info, 1);
                end
            end)
        end)
        
    ---------------------------------------------------- Scroll Frame
    local TableWidth = 650
    local AurasScrollFrame = CreateFrame("ScrollFrame", "AurasScrollFrame", SubMenu, "FauxScrollFrameTemplate");
    EnableScrollFrameButtonFlashing(AurasScrollFrame)
    SubMenu.AurasScrollFrame = AurasScrollFrame
        local RowCount = 13
        local max = RowCount +1
        
        SubMenu.AurasScrollFrame_Tooltip = CreateFrame("CheckButton", "AurasScrollFrame_Tooltip", SubMenu)
            SubMenu.AurasScrollFrame_Tooltip:SetAllPoints(AurasScrollFrame)

            MakeTooltip(SubMenu.AurasScrollFrame_Tooltip,
                "This table is used to filter auras.\n"..
                "Click the squares, column names, or aura names to change the squares."               
            )

        AurasScrollFrame:SetPoint("TOPLEFT", SubMenu, "TOPLEFT", 10, -75); --adjust scroll box
        AurasScrollFrame:SetWidth(TableWidth)
        AurasScrollFrame:SetHeight(335)
        AurasScrollFrame:SetBackdrop(narrowBorder);
        AurasScrollFrame:SetBackdropColor(0,0,0,1);
                    
        local AurasScrollButtons = {}
        local ColumnWidth = (TableWidth-180)/(getn(oUF_Lunar_FrameTypes)+1)
        
        -- Build column titles
        
        StaticPopupDialogs["Lunar_Confirm_Column"] = {
            button1 = "Do it.",
            button2 = "Cancel",
            timeout = 20,
            whileDead = 1,
            hideOnEscape = 1,
            exclusive = 1,
        };
        local PopupDialog = StaticPopupDialogs["Lunar_Confirm_Column"] 
        
        local Confirmed = false
        local function ColumnTitleOnClick_2(frameType)
            SpellToggleStatus(frameType, "Global")
            SubMenu.AurasScrollFrame_Update()
            RefreshFrames('Aura')
            Confirmed = true
        end
        local function ColumnTitleOnClick_1(self)
            if not Confirmed then
                local frameType = self.frameType
                PopupDialog.OnAccept = function()
                    ColumnTitleOnClick_2(frameType)
                end
                PopupDialog.text = "\nThis will change the "..self.Name.." frame aura settings for every aura shown right now.\n"
                local Popup = StaticPopup_Show ("Lunar_Confirm_Column");
                Popup:SetFrameLevel(99)
                Popup:SetBackdrop(narrowBorder)
                Popup:SetBackdropColor(0,0,0,1)
                Popup.frameType = self.frameType
            else
                ColumnTitleOnClick_2(self.frameType)
            end
        end
        local function BuildColumn(name, i)
            local ColumnTitle = CreateFrame('Button', 'LunarColumnTitle'..i, AurasScrollFrame, i>1 and "UIPanelButtonGrayTemplate")
            ColumnTitle:SetWidth(ColumnWidth)
            ColumnTitle:SetHeight(35)
            ColumnTitle.Des = ColumnTitle:CreateFontString(nil, 'OVERLAY', "GameFontNormalSmall")
            ColumnTitle.Des:SetPoint('CENTER')
            ColumnTitle.Des:SetText(name)
            
            if i > 1 then
                ColumnTitle:SetPoint("TOPLEFT", 175 + ColumnWidth*(i-1), 32);  
                local frameType = oUF_Lunar_FrameTypes[i-1]
                ColumnTitle.Name = name
                ColumnTitle.frameType = frameType
                ColumnTitle:SetScript('OnClick', ColumnTitleOnClick_1)
                MakeTooltip(ColumnTitle,
                    "Click here to change the "..name.." frame aura settings for all of these auras.\n"..
                    "You can do a search to filter the table before using this button."
                )
                
                ColumnTitle.Des:SetJustifyH("CENTER");
                ColumnTitle.Des:SetWidth(ColumnWidth)
                -- make dividers between columns
                local border = AurasScrollFrame:CreateTexture(nil, "BORDER")
                border:SetPoint("LEFT", ColumnTitle, "LEFT")
                border:SetPoint("TOP", AurasScrollFrame, "TOP", 2, -10)
                border:SetPoint("BOTTOM", AurasScrollFrame, "BOTTOM", 2, 10)
                border:SetWidth(1)
                border:SetTexture(.5,.5,.5, 1) 
            end
            AurasScrollFrame["ColumnTitle"..i] = ColumnTitle
        end
        
        BuildColumn("Spell Name", 1)
        for i, frameType in ipairs(oUF_Lunar_FrameTypes) do 
            BuildColumn(DisplayNames[frameType], i+1)
        end
        AurasScrollFrame["ColumnTitle1"]:SetPoint("TOPLEFT", 35, 20);

        local function ScrollButton_Tooltip()
            if oUF_Lunar_KnownAuras[this.IndexName].spellLink then
                GameTooltip:SetHyperlink(oUF_Lunar_KnownAuras[this.IndexName].spellLink)
            end
        end
        local function ScrollButton_OnLeave()
            GameTooltip:Hide()
        end
        local function ScrollButton_OnClick()
            if not this.IndexName then return end
            SpellToggleStatus("Global", this.IndexName)                    
            SubMenu.AurasScrollFrame_Update()
            RefreshFrames('Aura')
        end
        local function SpellToggle_Tooltip()
            local Status = GetSpellToggleStatus(this.FrameName, this.IndexName)
            local NextStatus = GetNextStatus(Status)
            local DisplayName = DisplayNames[this.FrameName]
            if Status == 'SelfCast' then 
                Status = 'Priority only when you cast it'
            elseif Status == 'Priority' then
                Status = 'Always Priority'
            end
            if NextStatus == 'SelfCast' then 
                NextStatus = 'Priority only when you cast it'
            elseif NextStatus == 'Priority' then
                NextStatus = 'Always Priority'
            end
            local text = 'For the '..DisplayName..' frame, '..(gsub(this.IndexName, '_', ' '))..' is currently '..(Status or "Normal")..'.\nClick this square to change it to '..(NextStatus or "Normal")..'.'
            GameTooltip:AddLine(text, 1, 1, 1, 1);
        end
        local function SpellToggle_OnClick()
            if not this.IndexName then return end
            SpellToggleStatus(this.FrameName, this.IndexName)
            SubMenu.AurasScrollFrame_Update()
            RefreshFrames('Aura')
            GameTooltip:Hide()
            this:GetScript('OnEnter')()
        end
        for i=1,RowCount do -- build rows
            local ScrollButton  = CreateFrame("button", "ScrollButton", SubMenu)
            ScrollButton:SetPoint("TOPLEFT", AurasScrollFrame, "TOPLEFT", 35, 10 - (i*24));
            ScrollButton:SetHeight(24)
            ScrollButton:SetWidth(180)
            MakeTooltip(ScrollButton,nil, ScrollButton_Tooltip)           
            ScrollButton:SetScript("OnLeave", ScrollButton_OnLeave)
            ScrollButton:SetScript("OnClick", ScrollButton_OnClick)
            
            ScrollButton.MainTexture = ScrollButton:CreateTexture(nil, "BORDER")
            ScrollButton.MainTexture:SetPoint("LEFT")
            ScrollButton.MainTexture:SetHeight(20)
            ScrollButton.MainTexture:SetWidth(20)
            
            ScrollButton.ClassIcon = ScrollButton:CreateTexture(nil, "BORDER")
            ScrollButton.ClassIcon:SetPoint("CENTER", ScrollButton, "LEFT", -15, 0)
            ScrollButton.ClassIcon:SetHeight(20)
            ScrollButton.ClassIcon:SetWidth(20)
            
            ScrollButton.MainTexture:SetTexture(.4,.4,.8, 0.7) 
            
            ScrollButton.spellName = ScrollButton:CreateFontString(nil, 'OVERLAY', "GameFontNormalSmall")
            ScrollButton.spellName:SetTextColor(1,1,1)
            ScrollButton.spellName:SetWidth(150)
            ScrollButton.spellName:SetHeight(12)
            ScrollButton.spellName:SetJustifyH("LEFT");
            ScrollButton.spellName:SetPoint("LEFT", 25, 0);
            
            ScrollButton.FrameName = FrameName
            
            --build boxes for each frame type
            ScrollButton.SpellToggle = {}
            for i, FrameName in ipairs(oUF_Lunar_FrameTypes) do 
                local SpellToggle = CreateFrame("button", "SpellToggle_"..FrameName, ScrollButton)
                local trueFrame = _G[FrameName]
                
                SpellToggle:SetPoint("TOP", ScrollButton, "TOP")
                SpellToggle:SetPoint("LEFT", AurasScrollFrame["ColumnTitle"..i+1], "LEFT", ColumnWidth/2 -11, 0)
                SpellToggle:SetHeight(22)
                SpellToggle:SetWidth(22) 
                
                SpellToggle.MainTexture = SpellToggle:CreateTexture(nil, "BORDER")
                SpellToggle.MainTexture:SetHeight(10)
                SpellToggle.MainTexture:SetWidth(10) 
                SpellToggle.MainTexture:SetPoint("CENTER")
                SpellToggle.MainTexture:SetTexture(.4,.4,.8, 0.8) 
                SpellToggle.MainTexture:Hide()
                
                SpellToggle:SetScript('OnClick', SpellToggle_OnClick)   
                SpellToggle:SetScript('OnEnter', SpellToggle_OnEnter) 
                MakeTooltip(SpellToggle,nil, SpellToggle_Tooltip)
                SpellToggle.FrameName = FrameName                
                ScrollButton.SpellToggle[FrameName] = SpellToggle
            end
            

            AurasScrollButtons["ScrollButton"..i] = ScrollButton                
        end
        
        local function AurasScrollFrame_OnShow()
            local searchword = SubMenu.SearchWord
            AuraList = {};
            for aura in pairs(KA) do
                if not searchword or SearchFilter(aura) then
                    table.insert(AuraList, aura)
                end
            end
            
            AuraCount = getn(AuraList)
            sortAuras(Profile.sortMethod or "Alphabetical")
            
            if AuraCount > RowCount+1 then 
                max = AuraCount
            else
                max = RowCount +1
            end
        end
        
        function SubMenu.AurasScrollFrame_Update() -- on scroll box update
            local line; -- 1 through 10 of our window to scroll
            local lineplusoffset; -- an index into our data calculated from the scroll offset
            FauxScrollFrame_Update(AurasScrollFrame,max,RowCount,24);
            for line=1, RowCount do --update each row
                local frame = AurasScrollButtons["ScrollButton"..line]
                lineplusoffset = line + AurasScrollFrame.offset
                local IndexName = AuraList[lineplusoffset]
                if lineplusoffset <= AuraCount then
                    frame.spellName:SetText(gsub(IndexName, "_", " ")); 
                    
                    if KA[IndexName].class and ClassIcon_Locations[KA[IndexName].class] then
                        frame.ClassIcon:SetTexture("Interface\\WorldStateFrame\\Icons-Classes")
                        local ClassIcon = ClassIcon_Locations[KA[IndexName].class]
                        frame.ClassIcon:SetTexCoord(ClassIcon[1], ClassIcon[2], ClassIcon[3], ClassIcon[4])
                        frame.ClassIcon:SetAlpha(1)
                    elseif KA[IndexName].class == "non-class-specific" then
                        frame.ClassIcon:SetAlpha(0)
                    else
                        frame.ClassIcon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
                        frame.ClassIcon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
                        frame.ClassIcon:SetAlpha(1)
                    end
                    
                    if KA[IndexName].buff == "DEBUFF" then
                        frame.spellName:SetTextColor(1,.5,.5)
                    elseif KA[IndexName].buff == "BUFF" then
                        frame.spellName:SetTextColor(.5,1,.5)
                    else
                        frame.spellName:SetTextColor(1,1,1)
                    end
                    frame.MainTexture:SetTexture( KA[IndexName].icon )
                    frame.MainTexture:SetTexCoord(0.05, 0.9, 0.05, 0.9)
                    frame.IndexName = IndexName
                    
                   
                    frame:Show();
                else
                    frame:Hide();
                end

                for _, FrameName in ipairs(oUF_Lunar_FrameTypes) do -- update each box in each row
                    local Toggle = frame.SpellToggle[FrameName]
                    local Status = GetSpellToggleStatus(FrameName, IndexName)
                    
                    if Status == "Hidden" or not IndexName then
                        Toggle.MainTexture:Hide()
                    elseif Status == "Priority" then
                        Toggle.MainTexture:Show()
                        Toggle.MainTexture:SetWidth(22)
                        Toggle.MainTexture:SetHeight(22)
                        Toggle.MainTexture:SetTexture(.4,.4,.8, 0.8)
                    elseif Status == "SelfCast" then
                        Toggle.MainTexture:Show()
                        Toggle.MainTexture:SetWidth(22)
                        Toggle.MainTexture:SetHeight(22)
                        Toggle.MainTexture:SetTexture(1,1,1, 0.8)
                    else
                        Toggle.MainTexture:Show()
                        Toggle.MainTexture:SetWidth(10)
                        Toggle.MainTexture:SetHeight(10)
                        Toggle.MainTexture:SetTexture(.4,.4,.8, 0.8)
                    end
                    Toggle.IndexName = IndexName
                end 
            end
        end
        
        AurasScrollFrame:SetScript("OnVerticalScroll", function(self, offset) --activate scrollbox scripts
            if AuraCount > RowCount then
                FauxScrollFrame_OnVerticalScroll(this, offset, 24, SubMenu.AurasScrollFrame_Update);
            end
        end );
        AurasScrollFrame:SetScript("OnShow", function() 
            AurasScrollFrame_OnShow()
            SubMenu.AurasScrollFrame_Update()
        end );

    ---------------------------------------------------- Search Auras
    
    local function RefineAuraList()
        ScrollFrame_OnLoad(AurasScrollFrame)
        self = SubMenu.AuraSearch
        local text = self:GetText() ~= "<Search Auras>" and self:GetText() or ""
        SubMenu.SearchWord = gsub(strlower(text), " ", "_") or false
        AurasScrollFrame_OnShow()
        SubMenu.AurasScrollFrame_Update()
        self.Count:SetText('('..(getn(AuraList) or "")..' shown)')
    end
    
    SubMenu.AuraSearch = CreateFrame("EditBox", "AuraSearch", SubMenu, "InputBoxTemplate");---------------- new Profile entry
        SubMenu.AuraSearch:SetPoint("TOPLEFT", SubMenu, "TOPLEFT", 25, -10);
        SubMenu.AuraSearch:SetWidth(150);
        SubMenu.AuraSearch:SetHeight(20);
        SubMenu.AuraSearch:SetAutoFocus(false);
        SubMenu.AuraSearch:SetMaxLetters(30);
        SubMenu.AuraSearch:SetScript("OnShow", function ()
            local SW = SubMenu.SearchWord
            local text
            
            if SW and SW ~= "" then 
                text = SW
                SubMenu.AuraSearch:SetJustifyH("LEFT")
            else
                text = "<Search Auras>"
                SubMenu.AuraSearch:SetJustifyH("CENTER")
            end
            this:SetText(text)
            this.Count:SetText('('..(getn(AuraList) or "")..' shown)')
        end)
        MakeTooltip(SubMenu.AuraSearch,
            "Type here to search all aura names and their tooltips."
        )
        
        SubMenu.AuraSearch:SetScript("OnEditFocusGained", function ()
            SubMenu.AuraSearch:SetJustifyH("LEFT")
            this:SetText("")
        end)
        SubMenu.AuraSearch:SetScript("OnTextChanged", function ()
            RefineAuraList(this)
            IncludeTooltipSearch = false
        end)
        SubMenu.AuraSearch:SetScript("OnEnterPressed", function ()
            if not Cached then -- update the cache if needed
                CreateTooltipCache()
            end
            IncludeTooltipSearch = true
            RefineAuraList(this)
            this:ClearFocus()
        end)
        
        SubMenu.AuraSearch.Count = SubMenu.AuraSearch:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        
        SubMenu.AuraSearch.Count:SetText('( )')
        SubMenu.AuraSearch.Count:SetPoint("TOP", SubMenu.AuraSearch, "BOTTOM", 0, -2)
        SubMenu.AuraSearch.Count:SetTextColor(1,1,1)
    
    SubMenu.ShowModified = template("CheckButton", "ShowModified", SubMenu, "Only show modified", -10, 220) 
    MakeTooltip(SubMenu.ShowModified,
        'When checked, auras that are modified on any frame will be shown in this list.'
    )
    SubMenu.ShowModified:SetScript("OnClick", function() 
        SubMenu.Only_Show_Modified = this:GetChecked()
        RefineAuraList()
    end);
    
        
    SubMenu.refresh = function()
        SubMenu.AurasScrollFrame_Update()
        SubMenu.AurasScrollFrame_Tooltip:SetFrameLevel((AurasScrollFrame:GetFrameLevel()) - 2)
    end
    SubMenu.default = function()
        Profile.HiddenAuras = deepcopy(default_options.HiddenAuras)
        Profile.PriorityAuras = deepcopy(default_options.PriorityAuras)
        Profile.SelfCastPriority = deepcopy(default_options.SelfCastPriority)
        Profile.sortMethod = deepcopy(default_options.sortMethod)
        ResetPage(SubMenu)
    end
    return SubMenu;
end


--[[****************************************************************************************************************************************************]]--
--                                                   SUBFRAME 3 - CUSTOM FRAMES
local function Customize_Frames()
    if getn(oUF_Lunar_FrameTypes) == 0 then return end
    local SubMenu = SubMenu_Template("Customize Frames")   

    -- Handle Container backdrop
    do
        local SubMenu_Old_OnShow = SubMenu:GetScript('OnShow')
        InterfaceOptionsFramePanelContainer.OldBackdrop = InterfaceOptionsFramePanelContainer:GetBackdrop()
        SubMenu:SetScript('OnShow', function()
            InterfaceOptionsFramePanelContainer:SetBackdrop(nil)
            SubMenu_Old_OnShow()
        end)
        local SubMenu_Old_OnHide = SubMenu:GetScript('OnHide')
        SubMenu:SetScript('OnHide', function()
            InterfaceOptionsFramePanelContainer:SetBackdrop(InterfaceOptionsFramePanelContainer.OldBackdrop)
            if SubMenu_Old_OnHide then
                SubMenu_Old_OnHide()
            end
        end)
    end
    
    SubMenu.SelectedPanel = "Global"
    local NoColor = {1,1,1,1}
    local NoColorHalfAlpha = {1,1,1,0.5}
    local Popup
    local DemoBarWidth = 160
    
    -- Common Functions
    local function MakeFontString(parent, inputSize, s)
        local size = inputSize or 12
        local Text = parent:CreateFontString(nil, 'OVERLAY')
        Text:SetFont(fontpath, size)
        Text:SetTextColor(1,1,1)
        Text:SetShadowOffset(1, -1)
        Text:SetWidth("600")
        Text:SetText(s)
        Text:SetPoint('CENTER')
        return Text;
    end
    -- Check if all types have same value for variable
    local function AllSame(var, TableSize)
        local first1, first2, first3, first4, test1, test2, test3, test4, T, V
        local PC = Profile.CustomFrames
        for i, fType in ipairs(oUF_Lunar_FrameTypes) do
            V = PC[fType][var]
            if TableSize and V then
                if getn(V) ~= TableSize then
                    return false;
                end
                
                if i == 1 then                    
                    first1, first2, first3, first4 = unpack(V)
                else
                    test1, test2, test3, test4 = unpack(V)
                    if (first1 ~= test1) or (first2 ~= test2) or (first3 ~= test3) or (first4 ~= test4) then
                        return false;
                    end
                end
            else                
                if i == 1 then
                    first1 = V
                else
                    test1 = V
                    if (first1 ~= test1) then
                        return false;
                    end
                end                
            end
        end
        
        return true, V;
    end
    
    -- Save and apply settings for any frame type
    local function ProcessAllTypes(var, val, func, frameType, tableSize)
        if frameType and frameType ~= 'Global' then
            if val ~= "PreviousValues" then
                local CustomFrameData = Profile.CustomFrames[frameType]
                if tableSize and CustomFrameData[var] then
                    for i=1, tableSize do
                        CustomFrameData[var][i] = val[i]
                    end
                else
                    CustomFrameData[var] = val
                end
                
                if func then
                    func(frameType)
                end
            end
        else
            for _, fType in ipairs(oUF_Lunar_FrameTypes) do
                if val ~= "PreviousValues" then
                    local CustomFrameData = Profile.CustomFrames[fType]
                    if tableSize and CustomFrameData[var] then
                        for i=1, tableSize do
                            CustomFrameData[var][i] = val[i]
                        end
                    else
                        CustomFrameData[var] = val
                    end
                end
                
                if func then
                    func(fType)
                end
            end
        end
    end
    -- Set bar colors
    local function SetManyVertex(frameList, frameType, BarType, number)
        local same = true
        local N  = number or 5
        local Color, r,g,b,a

        -- Assign Color
        if frameType == 'Global' then
            same, Color = AllSame(BarType..'Color'..N, 4)
        else
            local CustomFrameData = Profile.CustomFrames[frameType]
            Color = CustomFrameData[BarType..'Color'..N]
        end

        -- Error check
        if not (same and Color and getn(Color)==4) then
            if N== 99 then
                Color = NoColorHalfAlpha
            else
                Color = NoColor
            end
        end
        -- Single or multiple input
        r,g,b,a = unpack(Color);
        if frameList.MainTexture then
            if frameList.SetStatusBar_BackColor and N == 99 then
                frameList:SetStatusBar_BackColor(r,g,b,a)
            else
                frameList.MainTexture:SetVertexColor(r,g,b,a)
            end
        else
            for _, frame in ipairs(frameList) do
                if frame.SetStatusBar_BackColor and N == 99 then
                    frame:SetStatusBar_BackColor(r,g,b,a)
                else
                    frame.MainTexture:SetVertexColor(r,g,b,a)
                end
            end
        end        
        return r,g,b,a;
    end
    
    -- Set bar textures
    local function Set_The_Texture(frame, frameType, BarType)
        local same = true
    
        if frameType == 'Global' then
            same, texName = AllSame(BarType..'Texture')
        else
            texName = Profile.CustomFrames[frameType][BarType..'Texture']
        end
        local tex = Textures_Hash[texName]
        
        if not same then
            local r,g,b,a = unpack(NoColor)
            frame.MainTexture:SetTexture(r,g,b,a)
            if frame.SetStatusBar_BackTexture then
                frame:SetStatusBar_BackTexture(r/2,g/2,b/2,a/2)
            end
        else
            frame.MainTexture:SetTexture(tex)
            if frame.SetStatusBar_BackTexture then
                frame:SetStatusBar_BackTexture(tex)
            end
        end
        if frame.Description then
            frame.Description:SetText(texName)
        end
            
        return (same and tex and texName or "")
    end

    -- Create Popup Window for texture and color options
    do    
        local LineHeight = 32
        local RowCount = 8
        local L_FauxScrollFrame_Update = FauxScrollFrame_Update
        local Textures_KeyList = {}
        local TexCount = 0
        
        -- Build textures keylist
        for entry in pairs(Textures_Hash) do
            tinsert(Textures_KeyList, entry)
        end
        table.sort(Textures_KeyList)
        TexCount = getn(Textures_KeyList)
        
        -- Create frame
        Popup = CreateFrame("Frame", 'TextureSelector', UIParent)
        Popup:SetPoint("CENTER");
        Popup:SetHeight(345)
        Popup:SetWidth(500)
        Popup:SetBackdrop(narrowBorder)
        Popup:SetBackdropColor(0,0,0,1);
        Popup:Hide()
        Make_Movable(Popup)
        
        Popup.Des = MakeFontString(Popup, 18, "")
        Popup.Des:SetPoint("TOP", 0, -15)
        
        Popup.Previous = {}
        
        -- close the popup and move demo bar back to main menu
        Popup.Close = function()
            InterfaceOptionsFrame_OpenToCategory(SubMenu)
            InterfaceOptionsFrame_OpenToCategory(SubMenu)
            
            local DemoBar = Popup.DemoBar
            DemoBar:MoveIt(DemoBar.OldParent, DemoBar.NormalLocation)
            
            Popup:Hide()
        end
        
        Popup.Cancel = CreateFrame("Button", nil, Popup, "OptionsButtonTemplate"); -------reset Profile
        Popup.Cancel:SetPoint("BOTTOMRIGHT", Popup, "BOTTOMRIGHT", -15, 25);
        Popup.Cancel:SetText("Cancel")
        Popup.Cancel:SetScript("OnClick", function()
            local frameType = Popup.frameType;
            local BarType = Popup.BarType;
            -- Restore previous data
            Profile.CustomFrames = deepcopy(Popup.Previous)
            -- Apply Previous Textures
            ProcessAllTypes(
                BarType..'Texture', 
                "PreviousValues", 
                AdjustTextures, 
                frameType ~= 'Global' and frameType
            );
            -- Apply previous color gradient
            for i=1, 5 do
                ProcessAllTypes(
                    BarType..'Color'..i, 
                    "PreviousValues", 
                    AdjustColors, 
                    frameType, 
                    4
                )
            end
            -- Apply previous background color
            ProcessAllTypes(
                BarType..'Color'.."99", 
                "PreviousValues", 
                AdjustColors, 
                frameType, 
                4
            )
            -- Close
            Popup.Close();
        end)
        
        Popup.Okay = CreateFrame("Button", "Save & Close", Popup, "OptionsButtonTemplate"); -------reset Profile
        Popup.Okay:SetPoint("RIGHT", Popup.Cancel, "LEFT", -10, 0);
        Popup.Okay:SetText("Done")
        Popup.Okay:SetScript("OnClick", Popup.Close) 
            

    
        -- scrolling texture window
        local ScrollWindow = CreateFrame("ScrollFrame", 'TextureScrollFrame', Popup, "FauxScrollFrameTemplate")
        EnableScrollFrameButtonFlashing(ScrollWindow)
        local SharedMedia_Notify = MakeFontString(ScrollWindow, 10)
        SharedMedia_Notify:SetText('Get more textures with\n the SharedMedia or \nGrizzly Shared Media Addon')
        SharedMedia_Notify:SetPoint('BOTTOM', 0, 10)
        -- TextureScrollFrame Update function on scrolling
        ScrollWindow.ScrollUpdate = function(ScrollFrame)
            local lineplusoffset, key, tex;
            if TexCount > RowCount then
                L_FauxScrollFrame_Update(ScrollFrame,TexCount,RowCount,LineHeight);
                SharedMedia_Notify:Hide()
            else
                SharedMedia_Notify:Show()
            end
            local Keys = Textures_KeyList
            local DB = Textures_Hash
            local CustomFrameData = Profile.CustomFrames[this.frameType]
            for line=1, RowCount do --update each row
                local frame = ScrollFrame.ScrollButtons[line]
                lineplusoffset = line + ScrollFrame.offset
                if lineplusoffset <= TexCount then
                    key = Keys[lineplusoffset]
                    tex = DB[key]
                    frame.TextureName = key
                    frame.MainTexture:SetTexture(tex)
                    frame.Des:SetText(key)
                    frame:Show();
                else
                    frame:Hide();
                end
            end
        end
    
        ScrollWindow:SetWidth(DemoBarWidth)
        ScrollWindow:SetHeight(RowCount*LineHeight +15)
        ScrollWindow:SetPoint('TOPLEFT', Popup, 'TOPLEFT', 25, -50)
        Popup.ScrollWindow = ScrollWindow
        
        local ScrollButtons = {}
        for i=1, RowCount do
            local frame = CreateFrame('Button', 'ScrollButton'..i, ScrollWindow)
            frame:SetWidth(DemoBarWidth)
            frame:SetHeight(LineHeight)
            frame:SetPoint('CENTER', ScrollWindow, 'TOP', 0, -(LineHeight/2) -((LineHeight+2) * (i-1)))
            
            frame.MainTexture = frame:CreateTexture()
            frame.MainTexture:SetAllPoints(frame)
            
            frame.Des = MakeFontString(frame, 12, "")
            
            frame:SetScript('OnMouseDown', function()
                frame:SetWidth(DemoBarWidth*0.95)
                frame:SetHeight(LineHeight*0.95)
                
                local P = Popup
                local frameType = P.frameType
                local BarType = P.BarType
                local TexName = this.TextureName
                
                ProcessAllTypes(
                    BarType..'Texture', 
                    TexName, 
                    AdjustTextures, 
                    frameType ~= 'Global' and frameType
                )
                
                SetManyVertex(P.DemoBar, frameType, BarType)
                Set_The_Texture(P.DemoBar, frameType, BarType)
            end)
            frame:SetScript('OnMouseUp', function()
                frame:SetWidth(DemoBarWidth)
                frame:SetHeight(LineHeight)
            end)
            
            Animation_Setup(frame, 'ZoomIn')
            Animation_Setup(frame, 'ZoomOut') 
            local function OnEnter()
                this:SetFrameLevel(20)
                this.ZoomIn:Play()
                this.ZoomOut:Stop()
            end
            local function OnLeave()
                this.ZoomIn:Stop()
                this.ZoomOut:Play()
                this:SetFrameLevel(10)
            end
            frame:SetScript('OnEnter', OnEnter)
            frame:SetScript('OnLeave', OnLeave)
            tinsert(ScrollButtons, frame)
        end

        -- Activate scrollbox scripts
        ScrollWindow:SetScript("OnVerticalScroll", function(self, offset) 
            if TexCount > RowCount then
                FauxScrollFrame_OnVerticalScroll(this, offset, LineHeight, ScrollWindow.ScrollUpdate);
            else
                SharedMedia_Notify:Show()
            end
        end );

        -- Build color boxes
        local ColorBoxes = {}
        local BoxSize = 36
        
        Popup:SetScript("OnShow", function() 
            -- Backup all settings
            Popup.Previous = deepcopy(Profile.CustomFrames)
            
            local P = this
            local frameType = P.frameType
            local BarType = P.BarType
            local DemoBar = P.DemoBar
            local ScrollWindow = P.ScrollWindow
            local ScrollButtons = P.ScrollWindow.ScrollButtons
            local ColorSlider = P.ColorSlider
            
            ColorPickerFrame:Hide()
            ScrollFrame_OnLoad(ScrollWindow)
            ScrollWindow.ScrollUpdate(ScrollWindow)
            SetManyVertex(ScrollButtons, frameType, BarType);
                        
            if P.BarType == 'Power' then
                for i=1, 5 do
                    P.ColorBoxes[i]:Hide()
                end
            else
                for i=1, 5 do
                    if not P.ColorBoxes[i]:IsShown() then
                        P.ColorBoxes[i]:Show()
                    end
                end
            end
            
            --Slider
            ColorSlider:SetMinMaxValues(0, 1);
            ColorSlider:SetValueStep(1);
            local var = BarType..'ClassColor'
            if frameType == "Global" then
                local same, val = AllSame(var)
                if same then
                    ColorSlider:SetValue(val or 0)
                end
            else
                ColorSlider:SetValue(Profile.CustomFrames[frameType][BarType.."ClassColor"] or 0)
            end
            local val = ColorSlider:GetValue()
            for i =1, 5 do
                -- Dynamic Gradient
                if val == 0 then
                    ColorBoxes[i]:Show()
                    ColorBoxes[i+5]:Hide()
                    ColorBoxes[i+10]:Hide()
                    Popup.BoxNumber = 5
                -- Class Colors
                else
                    ColorBoxes[i]:Hide()
                    ColorBoxes[i+5]:Show()
                    ColorBoxes[i+10]:Show()
                    Popup.BoxNumber = ClassIndex[select(2, UnitClass('player'))] + 5
                end
            end
            Popup.DemoBar:BarAnimation(true)
        end );
        Popup.ScrollWindow.ScrollButtons = ScrollButtons
            
        local function Box_OnShow(self)
            if not self.Color then self.Color = {} end
            self.Color[1], self.Color[2], self.Color[3], self.Color[4] = SetManyVertex(self, Popup.frameType, Popup.BarType, self.BoxNumber)
        end
        
        -- define new functions for color picker
        local function On_Change(R,G,B,A)
            local CFP = ColorPickerFrame
            local OSF = OpacitySliderFrame
            local N = Popup.BoxNumber
            local Box = ColorBoxes[N]
            local Color = Box.Color
            local frameType = Popup.frameType
            local BarType = Popup.BarType
            if R then
                Color[1] = R
                Color[2] = G
                Color[3] = B
                Color[4] = A
            else
                Color[1], Color[2], Color[3] = CFP:GetColorRGB()
                Color[4] = 1 - OSF:GetValue();
            end
            
            -- Set all real frames and option frames to new colors
            ProcessAllTypes(BarType..'Color'..N, Color, AdjustColors, frameType, 4)
            SetManyVertex(ScrollButtons, frameType, BarType, N)
            Popup.DemoBar:BarAnimation(true)
            SetManyVertex(Box, frameType, BarType, N)
        end
        
        local function BoxMaker(i)
            local Box = CreateFrame("Button", "ColorBox"..i, Popup)
            Box.BoxNumber = i
            local PositionIndex
            if (i==99) then 
                PositionIndex=13
            elseif i > 5 then
                PositionIndex = i-5
            else
                PositionIndex = i
            end
            local yOffset = 56 * math.ceil(PositionIndex/5)
            Box:SetPoint('CENTER', ScrollWindow, 'TOPRIGHT', 90 +(((PositionIndex-1)%5)*40), -yOffset)
            Box:SetHeight(BoxSize)
            Box:SetWidth(BoxSize)
            Box.MainTexture = Box:CreateTexture(nil, 'OVERLAY')
            Box.MainTexture:SetWidth(BoxSize-13)
            Box.MainTexture:SetHeight(BoxSize-13)
            Box.MainTexture:SetPoint('CENTER')
            Box.MainTexture:SetTexture(BrightTexture)
            Box:SetBackdrop(BoxBorder)
            

            
            Box:SetScript('OnMouseDown', function()
                local Box = this
                local frameType = Popup.frameType
                local CustomFrameData = Profile.CustomFrames[frameType]
                local CFP = ColorPickerFrame
                local BarType = Popup.BarType
                local P = Popup
                local BoxNumber = Box.BoxNumber
                local BC = Box.Color
                local DemoBar = P.DemoBar
                
                -- error check
                if not BC then 
                    return;
                elseif not P then 
                    return;
                end

                -- Delete old functions
                CFP.func = nil
                CFP.opacityFunc = nil

                -- make box appear smaller to give 'pushed in' look
                Box:SetHeight(BoxSize*0.9)
                Box:SetWidth(BoxSize*0.9)
                
                -- Assign Selected box to this one
                P.BoxNumber = BoxNumber
                
                -- Assign Demo Bars to this one
                SetManyVertex(ScrollButtons, frameType, BarType, BoxNumber)
                SetManyVertex(DemoBar, frameType, BarType, BoxNumber)
                
                -- Set Color Picker settings
                CFP.BoxNumber = BoxNumber
                CFP.hasOpacity = true
                CFP.opacity = (1 - select(4, unpack(BC)))
                CFP:SetColorRGB(unpack(BC));
                ColorSwatch:SetTexture(unpack(BC))
                
                -- Save color values
                CFP.R, CFP.G, CFP.B, CFP.A = BC[1],BC[2],BC[3],BC[4]
                
                
                -- Set functions
                CFP.func = On_Change
                CFP.opacityFunc = On_Change
                CFP.cancelFunc  = function()
                    local R,G,B,A = CFP.R, CFP.G, CFP.B, CFP.A
                    On_Change(R,G,B,A)
                end
                
                -- Move Color picker
                CFP:SetParent(Box)
                CFP:SetBackdrop(narrowBorder)
                CFP:SetBackdropColor(0,0,0,1);
                CFP:ClearAllPoints()
                CFP:SetPoint('TOPLEFT', P, 'TOPRIGHT')

                CFP:Show()
                Popup.DemoBar:BarAnimation(true)
            end)
            Box:SetScript('OnMouseUp', function()
                this:SetHeight(BoxSize)
                this:SetWidth(BoxSize)
            end)
            
            Box:SetScript("OnShow", Box_OnShow);
            return Box
        end
        
        -- Prepare ColorBoxes
        do
            for count = 1, 5 do
                local Box = BoxMaker(count)
                Box.Des = MakeFontString(Box, 10, i)
                Box.Des:SetText(count)
                Box.Des:ClearAllPoints()
                Box.Des:SetPoint('BOTTOM', Box, 'CENTER', 0, (BoxSize/2)+3)
                ColorBoxes[count] = Box
            end
            local count = 6
            local class
            for i = 1, 10 do
                class = ClassIndex[i]
                local Box = BoxMaker(count)
                local ClassIcon = Box:CreateTexture(nil, "OVERLAY")
                ClassIcon:SetPoint("BOTTOM", Box, "CENTER", 0, (BoxSize/2),0)
                ClassIcon:SetHeight(20)
                ClassIcon:SetWidth(20)
                ClassIcon:SetTexture("Interface\\WorldStateFrame\\Icons-Classes")
                ClassIcon:SetTexCoord(unpack(ClassIcon_Locations[class]))
                Box.ClassIcon = ClassIcon
                Box.Class = class
                ColorBoxes[count] = Box
                count = count +1
            end
        end
                
        -- Background Color Box
        do
            local Box = BoxMaker(99)
            Box.Des = MakeFontString(Box, 10, i)
            Box.Des:SetText('Background')     
            Box.Des:ClearAllPoints()
            Box.Des:SetPoint('BOTTOM', Box, 'CENTER', 0, (BoxSize/2)+3)
            ColorBoxes[99] = Box
        end
        
        Popup.ColorBoxes = ColorBoxes
        
        -- Color Slider
        do
            local Slider = template("Slider", 'ColorSlider', Popup, description, -65, 400)
            Slider:SetScale(0.75)
            Slider:ClearAllPoints()
            Slider:SetPoint('BOTTOM', Popup.ColorBoxes[99], 'TOP', 0, 180)
            Slider.TextLeft:ClearAllPoints()
            Slider.TextRight:ClearAllPoints()
            Slider.TextLeft:SetPoint('RIGHT', Slider, 'LEFT', -5, 0)
            Slider.TextRight:SetPoint('LEFT', Slider, 'RIGHT', 5, 0)
            Slider.TextLeft:SetText("Dynamic Gradient");
            Slider.TextRight:SetText("Class Colors");
            Slider:SetWidth(80)
            Popup.ColorSlider = Slider
            
            Slider:SetScript("OnValueChanged", function()
                local val = this:GetValue()
                for i =1, 5 do
                    -- Dynamic Gradient
                    if val == 0 then
                        ColorBoxes[i]:Show()
                        ColorBoxes[i+5]:Hide()
                        ColorBoxes[i+10]:Hide()
                        Popup.BoxNumber = 5
                    -- Class Colors
                    else
                        ColorBoxes[i]:Hide()
                        ColorBoxes[i+5]:Show()
                        ColorBoxes[i+10]:Show()
                        Popup.BoxNumber = ClassIndex[select(2, UnitClass('player'))] + 5
                    end
                end
                local frameType = Popup.frameType
                if frameType == "Global" then
                    for _, fType in ipairs(oUF_Lunar_FrameTypes) do
                        Profile.CustomFrames[fType][Popup.BarType.."ClassColor"] = this:GetValue()
                        AdjustColors(fType)
                    end
                else
                    Profile.CustomFrames[Popup.frameType][Popup.BarType.."ClassColor"] = this:GetValue()
                    AdjustColors(frameType)                
                end
                Popup.DemoBar:BarAnimation(true)
                ColorPickerFrame:Hide()
            end);
            
            MakeTooltip(Slider, 'This will change the color system.\n'..
            'Dynamic Gradient will show color based on bar percent, Class Color will always show the color corresponding to that class.') 
        end
    end
    
    -- Bar Animation
    local function BarAnimation(bar, enable)
        local ClassBoxNumber = Popup.BoxNumber and Popup.BoxNumber>5 and Popup.BoxNumber<15 and Popup.BoxNumber
--~         local CIndex = ClassBoxNumber and ClassIndex[ClassBoxNumber-5]
--~         for i=1,10 do
--~             if 
--~         end
        local frameType = bar.frameType
        local BarType = bar.BarType
        local SG = bar.smoothGradient
        local k
        -- Set colors
        if frameType == 'Global' then
--~             if BarType == 'Health' then
                for i=1, 5 do
                    k = (i-1)*4
                    SG[k+1],SG[k+2],SG[k+3],SG[k+4] = SetManyVertex(bar, frameType, BarType, (ClassBoxNumber or i))
                end
--~             end
            SetManyVertex(bar, frameType, BarType, 99)
        else
            local CustomFrameData = Profile.CustomFrames[frameType]
--~             if BarType == 'Health' then
                for i=1, 5 do
                    k = (i-1)*4
                    SG[k+1],SG[k+2],SG[k+3],SG[k+4] = unpack(CustomFrameData[BarType..'Color'..(ClassBoxNumber or i)])
                end
--~             end
            
            bar:SetStatusBar_BackColor(unpack(CustomFrameData[BarType..'Color99']))
        end
        
        -- animate it
        if enable and not bar.Animated then
            bar.Animated = true
            local StartTime = GetTime()
            local floor = math.floor
            local CycleTime = 3 -- Variable
            local BarMax = DemoBarWidth
            local L_GetTime = GetTime
            
            -- OnUpdate
            bar:SetScript('OnUpdate', function()
                local Elapsed = L_GetTime() - StartTime
                local Percent = Elapsed % CycleTime / CycleTime
                local Way = floor((Elapsed % (CycleTime*2)) /CycleTime )

                if Way==1 then -- Alternate bar direction after each cycle
                    Percent = 1- Percent
                end
                -- Set size
                this:SetValue(BarMax*Percent)
                -- Set color
                this:SetStatusBarColor(bar.ColorAlphaGradient(Percent, unpack(this.smoothGradient)))
                -- Set Text
                this.PercentText:SetText((floor(Percent*20)*5)..'%')
            end)
        elseif not enable then
            bar:SetScript('OnUpdate', nil)
            bar.Animated = false
        end
    end
    
    -- Check Box OnClick
    local function CheckBox_OnClick(self, var, varFunction, Tab)
        local PC = Profile.CustomFrames
        local Checked = self:GetChecked()
        local frameType = Tab.frameType
        if frameType == "Global" then
            for _, fType in ipairs(oUF_Lunar_FrameTypes) do
                PC[fType][var] = Checked or false
                if varFunction then
                    varFunction(fType, Checked)
                else
                    AdjustFrame(fType) 
                end
            end
            self:SetCheckedTexture(CheckBox_Tex)
        else
            PC[frameType][var] = Checked or false
            if varFunction then
                varFunction(frameType, Checked)
            else
                AdjustFrame(frameType) 
            end
        end
    end
    
    -- Check box template
    local function CheckScript(Tab, var, Voffset, description, varFunction, ...)
        local frameType = Tab.frameType
        local function CheckBox_OnShow(self)
            if frameType == "Global" then
                local same, val = AllSame(var)
                if same then
                    self:SetCheckedTexture(CheckBox_Tex)
                    self:SetChecked(val)
                else
                    self:SetCheckedTexture(DisabledCheckBox_Tex)
                    self:SetChecked(true)
                end
            else
                self:SetChecked(Profile.CustomFrames[frameType][var])
            end
        end

        local box = template("CheckButton", nil, Tab.Panel, description, -(Voffset*14.6) +11)
        box:SetWidth(19)
        box:SetHeight(19)
        box.text:SetFontObject("GameFontNormalSmall")
        box.text:SetPoint('LEFT', box, 'RIGHT', 4,0)
        
        Type_OnClick = CheckBox_OnClick
        box:SetScript("OnClick", function() 
            Type_OnClick(this, var, varFunction, Tab)
        end);
        
        box:SetScript("OnShow", function()
            CheckBox_OnShow(this, var, varFunction, Tab)
        end)
        MakeTooltip(box, ...)   
        return (box)
    end
    
    --Demo for current healthbar texture
    local function Create_DemoBar(BarType, Height, Tab)
        local DemoBar = Create_UberStatusBar(nil, Tab.Panel)
        local frameType = Tab.frameType
        local DisplayName = DisplayNames[frameType]
        
        DemoBar:EnableMouse(true)
        DemoBar.OldParent = Tab.Panel
        DemoBar.BarType = BarType
        DemoBar.frameType = frameType
        DemoBar.smoothGradient = {
            1,1,1,1, --Color1
            1,1,1,1, --Color2
            1,1,1,1, --Color3
            1,1,1,1, --Color4
            1,1,1,1, --Color5
        }
        DemoBar:SetHeight(Height)
        DemoBar:SetWidth(DemoBarWidth)
        DemoBar:SetMinMaxValues(0, DemoBarWidth)
        DemoBar.BarAnimation = BarAnimation
    
        DemoBar.Description = MakeFontString(DemoBar, 12, "")
        DemoBar.Description:SetPoint('CENTER')
        
        DemoBar.Title = MakeFontString(DemoBar, 12, BarType.."Bar Texture")
        DemoBar.Title:SetPoint('BOTTOM', DemoBar, 'TOP', 0, 2)
        
        DemoBar.PercentText = MakeFontString(DemoBar, 10, "")
        DemoBar.PercentText:SetPoint('CENTER', DemoBar.Title, 'CENTER', 80,0)
        
        DemoBar.MoveIt = function(self, parent, destination)
            self:SetParent(parent)
            self:ClearAllPoints()
            self:SetPoint(unpack(destination));
        end
        
        local function OnClick()
            Popup.DemoBar = DemoBar
            Popup.frameType = frameType
            Popup.BarType = BarType
            Popup.Des:SetText(DisplayName..' '..BarType..' Bar')
            
            if DemoBar:GetParent() ~= Popup then
                -- Move to Popup window
                DemoBar:MoveIt(Popup, DemoBar.MoveToLocation)
                Popup.DemoBar = DemoBar
                Popup:Show()
                InterfaceOptionsFrame:Hide()
            else
                DemoBar:GetParent():Hide()
                DemoBar:MoveIt(DemoBar.OldParent, DemoBar.NormalLocation)
                InterfaceOptionsFrame_OpenToCategory(SubMenu)
                InterfaceOptionsFrame_OpenToCategory(SubMenu)
            end
        end
        
        DemoBar:SetScript('OnShow', function()
            local frameType = frameType
            Set_The_Texture(this, frameType, BarType)
            local var = this.BarType.."ClassColor"
            if frameType == "Global" then
                local same, var = AllSame(var)
                Popup.BoxNumber = (same and var==1) and (ClassIndex[select(2, UnitClass('player'))] + 5) or 5
            else
                if Profile.CustomFrames[frameType][var] == 1 then
                    Popup.BoxNumber = ClassIndex[select(2, UnitClass('player'))] + 5
                else
                    Popup.BoxNumber = 5
                end
            end
            this:BarAnimation(true)      
            if BarType~='Cast' then
                this:SetInvert(Tab.InvertBars:GetChecked())
                this:SetReverse(Tab.ReverseBars:GetChecked())
            end
        end)
        DemoBar:SetScript('OnHide', function()
            DemoBar:BarAnimation(false)
        end)
        
        -- Open the texture popup and set to current frame type
        DemoBar:SetScript('OnMouseUp', OnClick)
        
        DemoBar.Modify = CreateFrame("Button", "Modify"..BarType.."Demo", Tab.Panel, "OptionsButtonTemplate"); -------reset Profile
        DemoBar.Modify:SetPoint("LEFT", DemoBar, "RIGHT", 10, 0);
        DemoBar.Modify:SetText("Modify")
        MakeTooltip(DemoBar.Modify, "Click here to change the textures and colors for this bar.")
        DemoBar.Modify:SetScript("OnClick", OnClick)
        
        return DemoBar;
    end
    
    -- PanelLayout
    local function PanelLayout(Tab, columnName)  
        local HealthDemo
        local PowerDemo
        local CastDemo
        local frameType = Tab.frameType
        
        -- Define all check boxes for each frame type
        Tab.Enabled = CheckScript(Tab, "Enabled", 1,
            "Enable", 
            function(frameType) 
                ShowHideFrames(frameType)
                PartyUpdate()            
            end,                
            'When checked, the '..columnName..' frame will be shown.'
        );
        
        Tab.ShowWhileRaid = CheckScript(Tab, "ShowWhileRaid", 2,
            "Enable while in raid",
            function(frameType) 
                ShowHideFrames(frameType)
                PartyUpdate()
            end,
            "When checked, the "..columnName.." will be shown while you're in a raid."
        );
        
        Tab.ShowTargetOf = CheckScript(Tab, "ShowTargetOf", 3,
            "Enable a target frame for this type",
            function(frameType) 
                ShowHideFrames('oUF_Lunar_X_Target')
                PartyUpdate()
            end,
            "When checked, the target of "..columnName.." will be shown."
        );
        
        -- Bar Control
        local DesBar = MakeFontString(Tab.Panel, 10, "Bar Control")
        DesBar:SetJustifyH("LEFT");
        DesBar:SetTextColor(0.5,0.5,1)
        
        Tab.InvertBars = CheckScript(Tab, "InvertBars", 5,
            "Inverted bars",
            function(fType, Checked)
                HealthDemo:SetInvert(Checked)
                PowerDemo:SetInvert(Checked)
                AdjustFrame(fType)
            end,
            'As the unit gets low on health or power, the bars will grow and become more visable.\n'..
            'This feature is designed for healers, to emphasize units that need attention.'
        );
        DesBar:SetPoint("BOTTOMLEFT", Tab.InvertBars, "TOPLEFT")

        Tab.ReverseBars = CheckScript(Tab, "ReverseBars", 6,
            "Reverse bars",
            function(fType, Checked)
                HealthDemo:SetReverse(Checked)
                PowerDemo:SetReverse(Checked)
                AdjustFrame(fType)
            end,
            'Reverses direction of health and power bar fill.'
        );
        
        Tab.SmoothBars = CheckScript(Tab, "SmoothBars", 7,
            "Smooth bars",
            nil,
            'When checked, health and power changes will happen smoothly.\n Disabling will improve performance.'
        );
        
        -- Bar Text
        local DesBarText = MakeFontString(Tab.Panel, 10, "Bar Text")
        DesBarText:SetJustifyH("LEFT");
        DesBarText:SetTextColor(0.5,0.5,1)
        
        Tab.ValueText = CheckScript(Tab, "ValueText", 9,
            "Health and power percentages",
            AdjustTextVisibility,
            "Shows the unit's health and power percentages."
        );
        DesBarText:SetPoint("BOTTOMLEFT", Tab.ValueText, "TOPLEFT") 
        
        Tab.TotalText = CheckScript(Tab, "TotalText", 10,
            "Health and power detail",
            AdjustTextVisibility,
            "This will show the exact ammount of health and power on the bars."
        );
        
        Tab.Deficit = CheckScript(Tab, "Deficit", 11,
            "Health and power deficits",
            AdjustTextVisibility,
            "This will show a negative number for the health and power, representing how much has been lost."
        );        
        
        Tab.HideTextWhenFull = CheckScript(Tab, "HideTextWhenFull", 12,
            "Hide health and power when full",
            AdjustTextVisibility,
            "Hide health and power when full."
        );                
        
        --
        Tab.NameColor_Class = CheckScript(Tab, "NameColor_Class", 13,
            "Names with class color",
            AdjustTextVisibility,
            "Shows the unit's name with it's class color."
        );
        
        Tab.NameColor_Level = CheckScript(Tab, "NameColor_Level", 14,
            "Names with level color",
            AdjustTextVisibility,
            "Shows the unit's name colored by it's level compared to yours."
        );
        
        -- Features
        local DesMisc = MakeFontString(Tab.Panel, 10, "Features")
        DesMisc:SetJustifyH("LEFT");
        DesMisc:SetTextColor(0.5,0.5,1)
        
        Tab.ClassPortrait = CheckScript(Tab, "ClassPortrait", 16,
            "Class/type in portrait",
            nil,
            "When checked, the unit's class icon will show in the portrait.\nThe background will light up red when the unit has threat."
        );
        
        Tab.FlatPortrait = CheckScript(Tab, "FlatPortrait", 17,
            "2d portrait in portrait",
            nil,
            "When checked, the unit's 3d portrait will show in the portrait.\nThe background will light up red when the unit has threat."
        );
        
        Tab.ModelPortrait = CheckScript(Tab, "ModelPortrait", 18,
            "3d portrait in portrait",
            nil,
            "When checked, the unit's 3d portrait will show in the portrait.\nThe background will light up red when the unit has threat."
        );
        
        Tab.FrameBackDrop = CheckScript(Tab, "FrameBackDrop", 19,
            "Black border & background",
            nil,
            "Show black background over entire frame"
        );
        
        Tab.ComboPoints = CheckScript(Tab, "ComboPoints", 20,
            "Combo Points",
            nil,
            "Show Combo Points for Rogues and Druids."
        );

        Tab.RuneFrame = CheckScript(Tab, "RuneFrame", 20,
            "DeathKnight Runes",
            nil,
            "Show blizzard's default DeathKnight runes frame."
        );
        
        Tab.RestIcon = CheckScript(Tab, "RestIcon", 21,
            "Rest Icon",
            nil,
            "Show the rest icon when gaining rested bonus."
        );
        
        Tab.PvpIcon = CheckScript(Tab, "PvpIcon", 22,
            "PvP Icon",
            nil,
            "Show the PvP faction Icon (only shows when pvp is enabled)."
        );
        DesMisc:SetPoint("BOTTOMLEFT", Tab.ClassPortrait, "TOPLEFT")
        
        -- Aura Control
        local DesAura = MakeFontString(Tab.Panel, 10, "Aura Control")
        DesAura:SetJustifyH("LEFT");
        DesAura:SetTextColor(0.5,0.5,1)
        
        Tab.ShowCastable = CheckScript(Tab, "ShowCastable", 24,
            "Hide buffs/debuffs cast by your group",
            nil,
            "When checked, all buffs cast by a party or raid member will be hidden."
        );
        DesAura:SetPoint("BOTTOMLEFT", Tab.ShowCastable, "TOPLEFT")
        
        Tab.CooldownSpiral = CheckScript(Tab, "CooldownSpiral", 25,
            "Buff/Debuff cooldown spirals",
            nil,
            "Show cooldown spiral animation for any buff or debuff."
        );
        
        Tab.CooldownSpiralPriority = CheckScript(Tab, "CooldownSpiralPriority", 26,
            "Priority Aura cooldown spirals",
            nil,
            "Show cooldown spiral animation for Priority Auras."
        );
            
        Tab.Animations = CheckScript(Tab, "Animations", 27,
            "Debuff animations",
            nil,
             "When checked, all debuffs on friendly units will show an animation as they are first applied, to make them more apparent."
        );

        Tab.MouseOverAuras = CheckScript(Tab, "MouseOverAuras", 28,
            "Only show buffs/debuffs on mouseover",
            nil,
             "When checked, normal buffs and debuffs will only show on mouseover.\n"..
             "NOTE: the ALT key must be held while using mouseover."
        );         
        
        --Slider bar templates
        local Hposition = 1        
        local function SliderScript(min,max, var, description, varFunction, LeftRight, Tip)
            local Slider = template("Slider", var, Tab.Panel, description, -Hposition*30 -133, 470)
            Slider:SetScale(0.80)
            Slider:SetValueStep(2);
            
            -- Special properties for Left,Right,Off values
            if LeftRight then
                Slider.TextLeft:SetText("Left");
                Slider.TextRight:SetText("Right");
                Slider:SetWidth(120)
                Slider.TextCenter:SetText("")
                Slider.TextCenter.OldSetText = Slider.TextCenter.SetText
                Slider.TextCenter.SetText = dummy
            else
                Slider.TextLeft:SetText(tostring(min))
                Slider.TextRight:SetText(tostring(max))
                Slider:SetWidth(220)
            end
            local OnValueChanged
            if frameType == "Global" then
                OnValueChanged = function()
                    this.TextCenter:SetText(this:GetValue())
                    for _, fType in ipairs(oUF_Lunar_FrameTypes) do
                        Profile.CustomFrames[fType][var] = this:GetValue()
                        varFunction(fType)
                    end
                    if not this.Thumb:IsShown() then
                        this.Thumb:Show()
                    end
                end
            else
                OnValueChanged = function()
                    local val = this:GetValue()
                    this.TextCenter:SetText(val)
                    Profile.CustomFrames[frameType][var] = val
                    varFunction(frameType)
                end
            end
            
            Slider:SetScript("OnShow", function()
                this.Thumb = getglobal(this:GetName()..'Thumb')
                local CustomFrameData = Profile.CustomFrames[frameType]
                this:SetMinMaxValues(min, max);
                if frameType == "Global" then
                    local same, val = AllSame(var)
                    if same then
                        this:SetValue(val)
                        this.TextCenter:SetText(val)
                    else
                        -- Hide slider button
                        this.Thumb:Hide()
                        this.TextCenter:SetText('***')
                    end
                else
                    this:SetValue(CustomFrameData[var])
                    this.TextCenter:SetText(CustomFrameData[var])
                end
                Slider:SetScript("OnValueChanged", OnValueChanged)
            end)
            Slider:SetScript('OnHide', function()
                Slider:SetScript("OnValueChanged", nil)
            end)
            MakeTooltip(Slider, Tip)   
            Hposition = Hposition + 1
            
            return Slider;
        end
        
        -- Define all Sliders for each frame type
        Tab.FrameScale = SliderScript(24,300, "FrameScale", 
           "Frame Scale",
            AdjustScale,
            false,
            "This will set the scale of the frame."
        );
        Tab.FrameScale:SetValueStep(4);
        
        Tab.FrameWidth = SliderScript(32,300, "FrameWidth", 
            "Frame Width",
            AdjustFrameWidth,
            false,
            "This will adjust the width of the entire frame."
        );
        Tab.FrameWidth:SetValueStep(4);

        Tab.FrameHeight = SliderScript(10,100, "FrameHeight", 
            "Frame Height",
            AdjustFrameHeight,
            false,
            "This will adjust the Height of the entire frame."
        );
        Tab.FrameHeight:SetValueStep(1);
        
        Tab.DefaultSpacerY = SliderScript(-100,100, "DefaultSpacerY", 
           "Group unit spacing",
            AdjustSpacers,
            false,
            "This will set the vertical distance between members of a group"
        );
        
        Tab.PowerHeight = SliderScript(0,100, "PowerHeight", 
           "Powerbar Height",
            AdjustBarHeight,
            false,
            "This will change the height of the power bar. Zero disables it"
        );
        Tab.PowerHeight:SetValueStep(1);

        Tab.CastBarHeight = SliderScript(0,100, "CastBarHeight", 
           "Castbar Height",
            AdjustBarHeight,
            false,
            "This will change the height of the cast bar. Zero disables it"
        );
        Tab.CastBarHeight:SetValueStep(1);
        
        Tab.RangeAlpha = SliderScript(0,100, "RangeAlpha", 
            "Out of range alpha",
            AdjustRangeAlpha,
            false,
            "When the "..columnName.." is out of your range, it will fade to this percent."
        );

        Tab.AuraSize = SliderScript(8,128, "AuraSize", 
            "Priority aura size",
            AdjustAuraSize,
            false,
            "This will set the icon size of all priority buffs.\n"..
            "For more information about priority auras and filters, check the tooltip for the table below."
        );
        
        Tab.AuraSize = SliderScript(-200,200, "AuraOffset", 
            "Priority aura offset",
            AdjustAuraSize,
            false,
            "This will slide the priority auras to the left or right.\n"..
            "Using this will allow you to place the auras in the center of the frame."
        );
        
        Tab.AuraSpacing = SliderScript(-5,5, "AuraSpacing", 
            "Aura Spacing",
            AdjustAuraSize,
            false,
            "This will adjust the distance between all auras"
        );
        Tab.AuraSpacing:SetValueStep(0.25);

        Tab.PortraitLoc = SliderScript(-1,1, "PortraitLoc", 
           "Name/Icon Location",
            AdjustFrame,
            true,
            "This will place the portrait/icon on the left or right side of the health bars."
        );
        
        Tab.AuraLoc = SliderScript(-1,1, "AuraLoc", 
           "Priority Aura Location",
            AdjustFrame,
            true,
            "This will place the priority auras on the left or right side of the health bars."
        );
        
        --special frame exceptions
        if not (frameType == 'oUF_Lunar_Player' or frameType == 'Global') then
            Tab.RestIcon:Hide()            
            Tab.PvpIcon:Hide()
            Tab.RuneFrame:Hide()
        end        
        if frameType == "oUF_Lunar_Raid" then
            Tab.Enabled:Hide()
        end
        if not (frameType == "oUF_Lunar_Target" or frameType == "oUF_Lunar_X_Target" or frameType == "oUF_Lunar_Focus") then
            Tab.ComboPoints:Hide()
        else
            Tab.NameColor_Class:Hide()
            Tab.NameColor_Level:Hide()
        end        
        if frameType == 'oUF_Lunar_Player' or frameType == 'oUF_Lunar_X_Target' or frameType == "oUF_Lunar_Raid" or frameType == "oUF_Lunar_PartyPet" or frameType == "oUF_Lunar_ArenaPet" then
            Tab.ShowTargetOf:Hide()
        end
        
        -- Custom textures        
        
        local MoveToLocation = {'TOP', Popup.ColorBoxes[99], 'BOTTOM', 0, -20}
        HealthDemo = Create_DemoBar('Health', 25, Tab)
        HealthDemo.NormalLocation = {"TOPLEFT", Tab.Panel, "TOPLEFT", 385, -20}
        HealthDemo.MoveToLocation = MoveToLocation
        HealthDemo:MoveIt(Tab.Panel, HealthDemo.NormalLocation)
        
        PowerDemo = Create_DemoBar('Power', 25, Tab)
        PowerDemo.NormalLocation = {"TOPLEFT", Tab.Panel, "TOPLEFT", 385, -60}
        PowerDemo.MoveToLocation = MoveToLocation
        PowerDemo:MoveIt(Tab.Panel, PowerDemo.NormalLocation)
        
        CastDemo = Create_DemoBar('Cast', 25, Tab)
        CastDemo.NormalLocation = {"TOPLEFT", Tab.Panel, "TOPLEFT", 385, -100}
        CastDemo.MoveToLocation = MoveToLocation
        CastDemo:MoveIt(Tab.Panel, CastDemo.NormalLocation)        
        
    end
    
    -- Create container for all frame type panels'
    do
        local FrameList = CreateFrame("Frame", "FrameList", SubMenu , "OptionsFrameListTemplate")
        FrameList:SetPoint("TOPLEFT", SubMenu, "TOPLEFT", -1, -8)
        FrameList:SetPoint("BOTTOMRIGHT", SubMenu, "BOTTOMRIGHT", 1, -1)
        FrameList.numTabs = getn(oUF_Lunar_FrameTypes)+1
        local LastTab
        -- Process each frame type
        local function BuildPanel(columnName, i)
            -- Create a tab to select a frame type panel
            local Tab = CreateFrame("Button", "FrameListTab"..i, SubMenu , "OptionsFrameTabButtonTemplate")
            LastTab = Tab
            if i==1 then 
                columnName = "Global" 
                Tab.frameType = "Global"
            else
                Tab.frameType = oUF_Lunar_FrameTypes[i-1]
            end
            Tab:SetText(columnName)
            
            Tab.Spacer = Tab:CreateTexture(nil, "BORDER")
            Tab.Spacer:SetPoint("BOTTOMLEFT", Tab, "BOTTOMLEFT", 8, -6)
            Tab.Spacer:SetPoint("BOTTOMRIGHT", Tab, "BOTTOMRIGHT", -8, -6)
            Tab.Spacer:SetHeight(16)
            Tab.Spacer:SetTexture("Interface\\OptionsFrame\\UI-OptionsFrame-Spacer")
            
            -- Create panel for frame-type specific options
            Tab.Panel = CreateFrame("Frame", "FrameListPanel"..i, Tab)
            Tab.Panel:SetAllPoints(FrameList)
            PanelLayout(Tab, columnName)
            Tab.Panel:SetScript("OnShow", function()
                SubMenu.SelectedPanel = Tab.frameType
            end)
            
            -- Set position for the tabs
            if i == 1 then
                Tab:SetPoint("BOTTOMLEFT", FrameList, "TOPLEFT", 3, -3)
                Tab.Spacer:Hide()
            else
                Tab:SetPoint("TOPLEFT", FrameList["Tab"..i-1], "TOPRIGHT", -18, 0)
                Tab.Panel:Hide()
            end
            
            -- OnClick script selects panel for current frame-type, hides other panels, suppress other tabs.
            FrameList["Tab"..i] = Tab
            Tab.Index = i
            Tab:SetScript("OnClick", function()
                FrameList.selectedTab = this.Index
                PanelTemplates_UpdateTabs(FrameList)
                for i = 1, FrameList.numTabs do
                    if i ~= this.Index then
                        FrameList["Tab"..i].Spacer:Show()
                        FrameList["Tab"..i].Panel:Hide()
                    else
                        FrameList["Tab"..i].Spacer:Hide()
                        FrameList["Tab"..i].Panel:Show()
                    end
                end
            end)
        end
        
        BuildPanel('Global', 1)
        for i, fType in ipairs(oUF_Lunar_FrameTypes) do
            BuildPanel(DisplayNames[fType], i+1)
        end

        local LastTabSpacer = LastTab:CreateTexture(nil, "BORDER")
        LastTabSpacer:SetPoint("BOTTOMLEFT", LastTab, "BOTTOMRIGHT", -8, -6)
        LastTabSpacer:SetPoint("RIGHT", LastTab.Panel, "RIGHT", -16, -6)
        LastTabSpacer:SetHeight(16)
        LastTabSpacer:SetTexture("Interface\\OptionsFrame\\UI-OptionsFrame-Spacer")
        
        FrameList.selectedTab = 1
        PanelTemplates_UpdateTabs(FrameList)
        
        getglobal("FrameListButton1"):Hide()
        getglobal("FrameListTop"):Hide()
        SubMenu.FrameList = FrameList
    end
    
    -- Assign blizzard's button
    SubMenu.default = function()       
        if SubMenu.SelectedPanel == "Global" then
            Profile.CustomFrames = deepcopy(default_options.CustomFrames)
        else
            Profile.CustomFrames[SubMenu.SelectedPanel] = deepcopy(default_options.CustomFrames[SubMenu.SelectedPanel])
        end
        ResetPage(SubMenu)
    end
    
    return SubMenu;
end

--[[****************************************************************************************************************************************************]]--
--                                                   SUBFRAME 4 - PROFILES
local function Profile_Options()
	local SubMenu = SubMenu_Template("Profiles")

    local function CopyProfile(name)
        if name == oUF_Lunar_SelectedProfile then
            message("Lunar Unit Frames: Cannot copy a Profile to itself. Please choose another Profile first.")
        else
            oUF_Lunar_Profiles[oUF_Lunar_SelectedProfile] = deepcopy(oUF_Lunar_Profiles[name])
            oUF_Lunar_Profiles[oUF_Lunar_SelectedProfile].Name = oUF_Lunar_SelectedProfile
            oUF_Lunar_Profiles[oUF_Lunar_SelectedProfile].timesloaded = 0
            return (oUF_Lunar_SetProfile())
        end
    end
    
    local function DeleteProfile(name)
        if name == oUF_Lunar_SelectedProfile then
            message("Lunar Unit Frames: Cannot delete a Profile that is in use. Please choose another Profile first.")
        else
            message('Lunar Unit Frames: Deleted Profile "'..name..'"')
            oUF_Lunar_Profiles[name] = nil
        end
        return (Profile)
    end    

    local function template_ProfileDropDown(frameName, frameDescription, frameFunction, yOffset)
        
        local frame = CreateFrame("Frame", frameName, SubMenu, "UIDropDownMenuTemplate"); ---------------- Profile choices
        frame:SetPoint("TOPLEFT", SubMenu, "TOPLEFT", 0, yOffset);
        UIDropDownMenu_SetWidth(frame, 220)

        frame.Description = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        frame.Description:SetPoint("CENTER", frame, "CENTER", 0, 3);
        frame.Description:SetTextColor(1,1,1)
        frame.Description:SetText(frameDescription);
        
        frame:SetScript("OnShow", function()
            UIDropDownMenu_Initialize(frame, function() 
                local info = UIDropDownMenu_CreateInfo();
                if oUF_Lunar_Profiles then
                    for self in pairs(oUF_Lunar_Profiles) do
                        local name = oUF_Lunar_Profiles[self].Name
                        info.text = name; 
                        info.value = 0; 
                        info.func = function() 
                            Profile = frameFunction(name)
                        end; 
                        info.owner = frame; 
                        info.checked = nil; 
                        info.icon = nil; 
                        UIDropDownMenu_AddButton(info, 1); 
                    end
                end
            end)
        end)
        return frame
    end
    
    SubMenu.Description = SubMenu:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    SubMenu.Description:SetPoint("TOPLEFT", SubMenu, "TOPLEFT", 20, -20);
    SubMenu.Description:SetTextColor(1,1,1)
    SubMenu.Description:SetText("Currently Selected Profile: ")
    
    SubMenu.ProfileText = SubMenu:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
    SubMenu.ProfileText:SetPoint("TOPLEFT", SubMenu, "TOPLEFT", 20, -40);
    SubMenu.ProfileText:SetTextColor(1,1,1)
    
    SubMenu.ProfileChange = template_ProfileDropDown("ProfileChange", "Change to a Profile", oUF_Lunar_SetProfile, -90)
--~             MakeTooltip(OptionsMenu.SubMenu_1.ProfileChange_Tooltip,
--~                 "Click the drop down arrow to choose a different Profile.",
--~                 "As long as the frame positions are locked, all current Profile settings will be saved automatically."
--~             )
    SubMenu.ProfileCopy = template_ProfileDropDown("ProfileCopy", "Copy a Profile", CopyProfile, -150)
--~             MakeTooltip(OptionsMenu.SubMenu_1.ProfileCopy,
--~                 "Click the drop down arrow to choose a Profile to copy.",
--~                 "All your current Profile settings will be changed to match the Profile you copy, exept for the Profile name."
--~             )
    SubMenu.ProfileDelete = template_ProfileDropDown("ProfileDelete", "Delete a Profile", DeleteProfile, -210)
--~             MakeTooltip(OptionsMenu.SubMenu_1.ProfileDelete,
--~                 "Click the drop down arrow to choose a Profile to delete.",
--~                 "Deleted Profiles will not be accessable to any of your characters.",
--~                 "You cannot delete a Profile that is in use."
--~             )
        
    SubMenu.ProfileEntry = CreateFrame("EditBox", "ProfileEntry", SubMenu, "InputBoxTemplate");---------------- new Profile entry
        SubMenu.ProfileEntry:SetPoint("TOPLEFT", SubMenu, "TOPLEFT", 25, -270);
        SubMenu.ProfileEntry:SetWidth(230);
        SubMenu.ProfileEntry:SetHeight(20);
        SubMenu.ProfileEntry:SetAutoFocus(false);
        SubMenu.ProfileEntry:SetMaxLetters(30);
        SubMenu.ProfileEntry:SetJustifyH("CENTER")
        SubMenu.ProfileEntry:SetScript("OnShow", function ()
            this:SetText("Create new, then press enter")
        end)
        MakeTooltip(SubMenu.ProfileEntry,
            "To make a new Profile, type the desired name into the box, and press enter.\n"..
            "The new Profile will have default settings, not a copy of your default Profile.\n"..
            "Once the new Profile has been created, you can copy any other Profile to it using the copy drop down."
        )
        SubMenu.ProfileEntry:SetScript("OnEditFocusGained", function ()
            this:SetText("")
        end)
        SubMenu.ProfileEntry:SetScript("OnEnterPressed", function ()
            local text = this:GetText()
            if string.len(gsub(text, " ", "")) == 0 then
                message('Lunar Unit Frames: Enter a name into the text box before pressing "Create".')
            else
                message('Lunar Unit Frames: Creating new Profile "'..text..'"...')
                Profile = oUF_Lunar_SetProfile(text, false)
                this:SetText("")
                this:ClearFocus()
            end
        end)
        
    SubMenu.ResetProfile = CreateFrame("Button", "ResetProfile", SubMenu, "UIPanelButtonGrayTemplate"); -------reset Profile
        SubMenu.ResetProfile:SetPoint("TOPLEFT", SubMenu, "TOPLEFT", 25, -350);
        SubMenu.ResetProfile:SetWidth("230")
        SubMenu.ResetProfile:SetHeight("22")
        SubMenu.ResetProfile:SetText("Reset Profile to default")
        MakeTooltip(SubMenu.ResetProfile,
            "This will delete all saved information for this Profile."                
        )
        SubMenu.ResetProfile:SetScript("OnClick", function() 
            Profile.version = nil
            message('Lunar Unit Frames: '..oUF_Lunar_SelectedProfile..' Profile has been reset to defaults.')
            Profile = oUF_Lunar_SetProfile()
        end);
        
    SubMenu.refresh = function()
        Profile = oUF_Lunar_Profiles[oUF_Lunar_SelectedProfile]
        SubMenu.ProfileText:SetText(oUF_Lunar_SelectedProfile);
    end
    
    SubMenu.default = function()
        ResetPage(SubMenu)
    end

    return SubMenu;
end
--[[****************************************************************************************************************************************************]]--
--                                                   SUBFRAME 4 - PROFILES
local function Modules()
	local SubMenu = SubMenu_Template("Modules")
        
    local Intro_Des = MakeFontString(SubMenu, 14, "Disabling these modules will prevent them from loading.\n You must reload for these changes to take effect.")    
    Intro_Des:SetPoint('TOP', 0, -20)
    
    -- Frame Type Modules
    do
        local function OnClick(self)
            Profile.LoadFrameTypes[self.Module] = self:GetChecked() and true or false
            SubMenu.Reload:Show()
        end
        local function OnShow(self)
            self:SetChecked(Profile.LoadFrameTypes[self.Module])
        end
        
        local function CheckScript(name, count)
            local Module_Checkbox = template('CheckButton', name, SubMenu, DisplayNames[name], -25*count -50, xPos)
            Module_Checkbox.Module = name
            MakeTooltip(Module_Checkbox, 'When unchecked, the '..DisplayNames[name]..' frames will no longer load during start up.')
            Module_Checkbox:SetScript("OnClick", OnClick);
            Module_Checkbox:SetScript("OnShow", OnShow);
            SubMenu[name] = Module_Checkbox
        end
        for count, name in ipairs(Profile.LoadFrameTypes) do
            CheckScript(name, count)
        end
    end
    
    -- Raid Size Slider
    do
        local function OnValueChanged(self)
            Profile.RaidGroupCount = this:GetValue()
            self.TextCenter:SetText(self:GetValue())
            SubMenu.Reload:Show()
        end        
        
        local Slider = template("Slider", 'RaidGroupCount', SubMenu, "Raid Group Count", -370, 25)
        Slider.TextLeft:SetText("");
        Slider.TextRight:SetText("");
        Slider:SetValueStep(1)
        Slider:SetMinMaxValues(1, 8);
        SubMenu.RaidSlider = Slider
        
        Slider:SetScript("OnShow", function()
            this:SetValue(Profile.RaidGroupCount)
            this.TextCenter:SetText(Profile.RaidGroupCount)
            this:SetScript("OnValueChanged", OnValueChanged)
        end);

        Slider:SetScript("OnHide", function()
            this:SetScript("OnValueChanged", nil)
        end);
        
        MakeTooltip(Slider, 'This will change the number of raid groups that load.') 
    end
    
    -- Reload Button
    do
        local Reload = CreateFrame("Button", "Reload", SubMenu, "OptionsButtonTemplate"); --Reload ui
        Reload:SetPoint("CENTER");
        Reload:SetWidth("100")
        Reload:SetText("ReloadUI");
        MakeTooltip(Reload,
            "This will reload your interface."
        )
        Reload:SetScript("OnClick", function() 
            ReloadUI()
        end);
--~         Animation_Setup(Reload, 'Flash')
--~         Reload:SetScript("OnShow", function() 
--~             this.Flash:Play()
--~         end);
--~         Reload:SetScript("OnHide", function() 
--~             this.Flash:Stop()
--~         end);
        Reload:SetScale(2)
        Reload:Hide()
        SubMenu.Reload = Reload
    end
    
    return SubMenu;
end
--[[****************************************************************************************************************************************************]]--
--                                                   BUILD OPTIONS MENU
function oUF_Lunar_BuildOptionsMenu(External_Functions)
    
    DragableFrames, 
    OpenUnitFrames, 
    ShowHideFrames, 
    PartyUpdate, 
    AdjustRangeAlpha, 
    AdjustFrame, 
    AdjustAuraSize, 
    AdjustSpacers, 
    AdjustScale, 
    RefreshFrames, 
    Textures_Hash,
    Font_Hash,
    AdjustTextures,
    AdjustColors,
    oUF_Lunar_FrameTypes,
    default_options,
    AdjustFrameWidth,
    PartyUpdate,
    ToggleAuraMonitoring,
    ShowHideBlizzardFrames,
    AdjustBarHeight,
    AdjustFont,
    AdjustTextVisibility,
    AdjustFrameHeight    = unpack(External_Functions)
    
    Profile = oUF_Lunar_Profiles[oUF_Lunar_SelectedProfile]
    
    -- Build Database of Spell Id's.
    if not oUF_Lunar_KnownAuras or not Profile.version or Profile.version < oUF_Lunar_Version then
        oUF_Lunar_Create_Aura_Database()
        oUF_Lunar_Create_Aura_Database = nil
    end
    
    -- Assign global saved variables to local.
    KA = oUF_Lunar_KnownAuras;
    MinimapSettings = Profile.Minimap
        
    -- Make InterfaceOptionsFrame at least 900 by 600, and movable
    do
        local Width =  940
        local Height = 560
        InterfaceOptionsFrame:SetMinResize(Width, Height)
        if InterfaceOptionsFrame:GetWidth() < Width then
            InterfaceOptionsFrame:SetWidth(Width)
        end
        if InterfaceOptionsFrame:GetHeight() < Height then
--~             InterfaceOptionsFrame:SetHeight(Height)
--~             InterfaceOptionsFrame:SetMinResize(Width, Height)
--~             InterfaceOptionsFrame:SetMaxResize(1200, 800)
--~             InterfaceOptionsFrame:SetResizable(true)
        end
        local function OnMouseDown()
            this:StartMoving() 
            this:SetClampedToScreen(false) 
        end
        local function OnMouseUp()
            this:StopMovingOrSizing() 
        end
        InterfaceOptionsFrame:SetMovable(true)
        InterfaceOptionsFrame:EnableMouse(true)
        InterfaceOptionsFrame:SetScript('OnMouseDown', OnMouseDown)
        InterfaceOptionsFrame:SetScript('OnMouseUp', OnMouseUp)
    end

    -- Build Main Page
    OptionsMenu = Main()
    InterfaceOptions_AddCategory(OptionsMenu)
    
    -- Build Sub Menus
    OptionsMenu.SubMenu_1 = Generic_Options()
    OptionsMenu.SubMenu_2 = Auras_Table()
    OptionsMenu.SubMenu_3 = Customize_Frames()
    OptionsMenu.SubMenu_4 = Profile_Options()
    OptionsMenu.SubMenu_5 = Modules()
    OptionsMenu.CombatLogParser = CombatLogParser
    OptionsMenu.Build_Spell_ID_Database = Build_Spell_ID_Database
    OptionsMenu.KA_Update_Via_UnitAura = KA_Update_Via_UnitAura

    -- Load welcome screen on first time.
    if not oUF_Lunar_WelcomeScreenShown then
        OptionsMenu.WelcomeScreen = oUF_Lunar_MakeWelcomeScreen(OptionsMenu, fontpath, narrowBorder)
        oUF_Lunar_WelcomeScreenShown = true
    end
    
    -- Build Minimap icon
    UpdateMinimapButton(Profile.Minimap.enabled==1)
    
    --end of options menu
    return OptionsMenu, UpdateMinimapButton;
end

--[[****************************************************************************************************************************************************]]--
--                                                   SCAN FOR NEW AURAS
-- cache the Spell_ID_Database
Build_Spell_ID_Database = function () -- this is local
    local name
    for i = 1, 100000 do
        name = GetSpellInfo(i)
        if name then
            Spell_ID_Database[name] = i
        end
    end
end

-- Get Class of unit or pet's owner
local function DetermineAuraSourceClass(unit)
    -- class assignment 
    if L_UnitIsFriend('player', unit) then
        local class
        if L_UnitIsPlayer(unit) then
            class = select(2, L_UnitClass(unit))
        else
            local petOwner
            if L_UnitExists('pet') and L_UnitIsUnit(unit, 'pet') then
                petOwner = "player"
            else
                for i = 1, GetNumRaidMembers() do
                    if L_UnitExists('raidpet'..i) and L_UnitIsUnit(unit, 'raidpet'..i) then
                        petOwner = "raid"..i
                    end
                end
                for i = 1, GetNumPartyMembers() do
                    if L_UnitExists('partyPet'..i) and L_UnitIsUnit(unit, 'partypet'..i) then
                        petOwner = "party"..i
                    end
                end
            end
            if petOwner then
                class  = select(2, L_UnitClass(petOwner))
            end
        end
        return class
    end
end

-- Build database entry
local function Build_KA_Entry(SpellId, filter)
    local SpellName, rank, icon = GetSpellInfo(SpellId)
    local IndexName = gsub(SpellName, " ", "_")
    local buff = (filter=='HELPFUL' and "BUFF") or "DEBUFF"
    local spellLink = GetSpellLink(SpellId)
    
    oUF_Lunar_KnownAuras[IndexName] = {
        rank = rank,
        icon = icon,
        buff = buff,
        spellLink = spellLink
    }
    return oUF_Lunar_KnownAuras[IndexName]
end

-- Update on UNIT_AURA event
function KA_Update_Via_UnitAura(unit, index, filter)
    if not (KA) then return end
    local name, rank, iconTexture, count, dType, duration, expireTime, caster = UnitAura(unit, index, filter)
    local IndexName = gsub(name, " ", "_")
    local AuraData = KA[IndexName]
    if not AuraData then
        local SpellId = Spell_ID_Database[name]
        if not SpellId then
            return 
        end
        KA[IndexName] = Build_KA_Entry(SpellId, filter)
        AuraData = KA[IndexName]
    end
    
    if not AuraData.class and caster then
        AuraData.class = DetermineAuraSourceClass(caster, UnitGUID(caster))
    end
end

-- Update on SPELL_AURA_APPLIED event
local function CombatLogOnEvent(_, event, ...)

--~         local argString = ""
--~         for i, item in ipairs({...}) do
--~             argString = argString..', arg'..i..'='..item
--~         end
--~         if string.find(string.lower(argString), 'totem') then
--~         end

    if arg2 == "SPELL_AURA_APPLIED" then
        local unit, _, _, _, _, SpellId, SpellName, _, buff_debuff = select(4, ...) 
        if not L_UnitExists(unit) then return end
        local AuraData = KA[IndexName]
        --build entry
        local IndexName = gsub(SpellName, " ", "_")
        if not AuraData then
            local isBuff = (buff_debuff == "BUFF") and 'HELPFUL'
            KA[IndexName] = Build_KA_Entry(SpellId, isBuff)
            AuraData = KA[IndexName]
        end
        
        --Handle class attribute
        local class  = DetermineAuraSourceClass(unit)
        if class then
            if not AuraData.Class_Confirmed then
                AuraData.class = class
                AuraData.Class_Confirmed = true
            elseif AuraData.class ~= class and AuraData.class ~= "non-class-specific" then
                AuraData.class = "non-class-specific"
            end
        end
    end
end

CombatLogParser = CreateFrame("frame")
CombatLogParser:SetScript("OnEvent", CombatLogOnEvent)

-- EOF


