local function debug (message)
    DEFAULT_CHAT_FRAME:AddMessage(message)
end


local fontpath = "Fonts\\FRIZQT__.TTF"
local DragHighlight = {}
local framesCache
local profileCache
local OpenUnitFramesCache
local ToggleFunctionCache

local narrowBorder = {
    bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", 
    edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", tile = true, tileSize = 32, edgeSize = 16, 
    insets = { left = 4, right = 4, top = 4, bottom = 4 } 
}

local function MakeFontString(parent)
    local Text = parent:CreateFontString(nil, 'OVERLAY')
    Text:SetFont(fontpath, 12, "OUTLINE")
    Text:SetTextColor(1,1,1)
    return Text;
end

local DraggingHelper = CreateFrame("frame", "DraggingHelper", UIParent)
DraggingHelper:Hide()
DraggingHelper:SetHeight(75)
DraggingHelper:SetWidth(340)
DraggingHelper:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
DraggingHelper:SetBackdrop(narrowBorder)

DraggingHelper:SetMovable(true)
DraggingHelper:EnableMouse(true)

DraggingHelper:SetScript("OnMouseDown",function() 
    this:StartMoving() 
    this:SetClampedToScreen(true) 
end)
DraggingHelper:SetScript("OnMouseUp",function() 
    this:StopMovingOrSizing() 
end)

DraggingHelper.Text = MakeFontString(DraggingHelper)
DraggingHelper.Text:SetPoint('TOP', DraggingHelper, "TOP", 0, -10) 
DraggingHelper.Text:SetText(
    "All unit frames are currently unlocked and may be dragged to another position."
)
--~ DraggingHelper.Text:SetDrawLayer("OVERLAY")
DraggingHelper.Text:SetWidth(300)

local Okay = CreateFrame("Button", "ok", DraggingHelper, "OptionsButtonTemplate")
    Okay:SetPoint("BOTTOM", DraggingHelper, "BOTTOM", 0, 10);
    Okay:SetText("Done")
---------------------------------------------------- Dragging 
function Lunar_Dragging(state, frames, profile, OpenUnitFrames, ToggleFunction)
    ToggleFunctionCache = ToggleFunction
--~     local oUF_Lunar_Lock_CheckBox = getglobal("oUF_Lunar_Lock_CheckBox")
    profile = profile or profileCache
    frames = frames or framesCache
    OpenUnitFrames = OpenUnitFrames or OpenUnitFramesCache
    
    framesCache = frames
    profileCache = profile
    OpenUnitFramesCache = OpenUnitFrames
    
    local name
    if profile [ "FramePosition" ] == nil
        then profile [ "FramePosition" ] = {}
    end 

    local function SavePositions(frame, name)
        if not profile.FramePosition[name] then debug('drag cant find '..name) return end
        local P = profile.FramePosition[name];
        local Par
        P.FromPoint, Par, P.ToPoint, P.X, P.Y = frame:GetPoint();
        P.Parent = Par and Par:GetName() or "UIParent"
    end
    
    for name in pairs(frames) do --process all frames in list
        local self = frames[name]

        if not self.dragTexture then         
            self.dragTexture = DraggingHelper:CreateTexture(nil, "BORDER")
            self.dragTexture:SetAllPoints(self)
            self.dragTexture:SetTexture(.4,.4,1,.4) 
            self.dragTexture:SetBlendMode('ADD') 
            self.dragTexture.text = MakeFontString(DraggingHelper)
            self.dragTexture.text:SetShadowOffset(2, -2)
            self.dragTexture.text:SetPoint('CENTER', self, 'CENTER') 
            self.dragTexture.text:SetText(self.DisplayName)
            
            self:SetScript("OnDragStart", function()
--~                 if(not oUF_Lunar_Lock_CheckBox:GetChecked())then
                    this.isMoving = true;
                    this:StartMoving()
                    this:SetClampedToScreen(true) 
--~                 end
            end);
            
            self:SetScript("OnDragStop", function() 
                if(this.isMoving == true)then
                    this:StopMovingOrSizing()
                end
            end);
        end
        
        if state then	
            self.dragTexture:Show()
            self:EnableMouse(true);
            self:SetMovable(true);
            self:RegisterForDrag("LeftButton");
--~             self:SetUserPlaced(true)
            
            DraggingHelper:Show()
        else
            SavePositions(self, name)
            self:SetUserPlaced(false)
            self:SetMovable(false);
            self:RegisterForDrag("");
            DraggingHelper:Hide()
            self:EnableMouse(false);
        end
--~         oUF_Lunar_Lock_CheckBox:SetChecked(not state)
    end
    
    if state then
        for i, self in ipairs(OpenUnitFrames) do
            self:EnableMouse(false)
        end
    else
        for i, self in ipairs(OpenUnitFrames) do
            self:EnableMouse(true)
        end
    end

end

Okay:SetScript("OnClick", function() 
    ToggleFunctionCache()
--~     InterfaceOptionsFrame:Show()
end);